@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *)
+*)
 
 open Eliom_lib
 
@@ -23,17 +23,12 @@ open Ocsigen_cookies
 
 include Eliom_common_base
 
-exception Eliom_Wrong_parameter (** Service called with wrong parameter names *)
 exception Eliom_Session_expired
-exception Eliom_Typing_Error of (string * exn) list
 
-exception Eliom_duplicate_registration of string
 exception Eliom_there_are_unregistered_services of
-  (string list * string list list * na_key_serv list)
-exception Eliom_page_erasing of string
+    (string list * string list list * na_key_serv list)
 exception Eliom_error_while_loading_site of string
 
-exception Eliom_404
 exception Eliom_do_redirection of string
 exception Eliom_do_half_xhr_redirection of string
 
@@ -73,7 +68,7 @@ let eliom_link_too_old : bool Polytables.key = Polytables.make_key ()
 (** The coservice does not exist any more *)
 
 let eliom_service_session_expired :
-    (full_state_name list * full_state_name list) Polytables.key =
+  (full_state_name list * full_state_name list) Polytables.key =
   Polytables.make_key ()
 (** If present in request data,  means that
     the service session cookies does not exist any more.
@@ -99,13 +94,13 @@ type cookie_exp =
    and many people do not understand why their session is closed, even if
    the session duration on server side is long.
    If you want this, you now have to set this manually.
- *)
+*)
 let default_client_cookie_exp () =
   CESome (Unix.time () +. 315532800.)
 
 type timeout =
   | TGlobal (** see global setting *)
-  | TNone   (** explicitely set no timeout *)
+  | TNone   (** explicitly set no timeout *)
   | TSome of float (** timeout duration in seconds *)
 
 
@@ -114,19 +109,19 @@ type timeout =
 (* The table of tables for each session. Keys are cookies *)
 module SessionCookies =
   Hashtbl.Make(struct
-                 type t = string
-                 let equal = (=)
-                 let hash = Hashtbl.hash
-               end)
+    type t = string
+    let equal = (=)
+    let hash = Hashtbl.hash
+  end)
 
 (* session groups *)
 type 'a sessgrp =
-    (string * cookie_level * (string, Ipaddr.t) leftright)
-    (* The full session group is the triple
-       (site_dir_string, scope, session group name).
-       The scope is the scope of group members (`Session by default).
-       If there is no session group,
-       we limit the number of sessions by IP address. *)
+  (string * cookie_level * (string, Ipaddr.t) leftright)
+(* The full session group is the triple
+   (site_dir_string, scope, session group name).
+   The scope is the scope of group members (`Session by default).
+   If there is no session group,
+   we limit the number of sessions by IP address. *)
 type perssessgrp = string (* same triple, marshaled *)
 
 let make_persistent_full_group_name ~cookie_level site_dir_string = function
@@ -142,120 +137,120 @@ let string_of_perssessgrp = id
 (* cookies information during page generation: *)
 
 type 'a one_service_cookie_info =
-    (* service sessions: *)
-    {sc_value:string             (* current value *);
-     sc_table:'a ref             (* service session table
-                                    ref towards cookie table
-                                  *);
-     sc_timeout:timeout ref      (* user timeout -
-                                    ref towards cookie table
-                                  *);
-     sc_exp:float option ref     (* expiration date ref
-                                    (server side) -
-                                    None = never
-                                    ref towards cookie table
-                                  *);
-     sc_cookie_exp:cookie_exp ref (* cookie expiration date to set *);
-     sc_session_group: cookie_level sessgrp ref
-       (* session group *);
-     mutable sc_session_group_node:string Ocsigen_cache.Dlist.node;
-   }
+  (* service sessions: *)
+  {sc_value:string             (* current value *);
+   sc_table:'a ref             (* service session table
+                                  ref towards cookie table
+                               *);
+   sc_timeout:timeout ref      (* user timeout -
+                                  ref towards cookie table
+                               *);
+   sc_exp:float option ref     (* expiration date ref
+                                  (server side) -
+                                  None = never
+                                  ref towards cookie table
+                               *);
+   sc_cookie_exp:cookie_exp ref (* cookie expiration date to set *);
+   sc_session_group: cookie_level sessgrp ref
+  (* session group *);
+   mutable sc_session_group_node:string Ocsigen_cache.Dlist.node;
+  }
 
 
 type one_data_cookie_info =
-    (* in memory data sessions: *)
-    {dc_value:string                    (* current value *);
-     dc_timeout:timeout ref             (* user timeout -
-                                           ref towards cookie table
-                                         *);
-     dc_exp:float option ref            (* expiration date ref (server side) -
-                                           None = never
-                                           ref towards cookie table
-                                         *);
-     dc_cookie_exp:cookie_exp ref       (* cookie expiration date to set *);
-     dc_session_group: cookie_level sessgrp ref (* session group *);
-     mutable dc_session_group_node:string Ocsigen_cache.Dlist.node;
-   }
+  (* in memory data sessions: *)
+  {dc_value:string                    (* current value *);
+   dc_timeout:timeout ref             (* user timeout -
+                                         ref towards cookie table
+                                      *);
+   dc_exp:float option ref            (* expiration date ref (server side) -
+                                         None = never
+                                         ref towards cookie table
+                                      *);
+   dc_cookie_exp:cookie_exp ref       (* cookie expiration date to set *);
+   dc_session_group: cookie_level sessgrp ref (* session group *);
+   mutable dc_session_group_node:string Ocsigen_cache.Dlist.node;
+  }
 
 type one_persistent_cookie_info =
-     {pc_value:string                    (* current value *);
-      pc_timeout:timeout ref             (* user timeout *);
-      pc_cookie_exp:cookie_exp ref       (* cookie expiration date to set *);
-      pc_session_group:perssessgrp option ref (* session group *)
-    }
+  {pc_value:string                    (* current value *);
+   pc_timeout:timeout ref             (* user timeout *);
+   pc_cookie_exp:cookie_exp ref       (* cookie expiration date to set *);
+   pc_session_group:perssessgrp option ref (* session group *)
+  }
 
 
 (*VVV heavy *)
 type 'a cookie_info1 =
-    (* service sessions: *)
-    (string option            (* value sent by the browser *)
-                              (* None = new cookie
-                                 (not sent by the browser) *)
-       *
+  (* service sessions: *)
+  (string option            (* value sent by the browser *)
+   (* None = new cookie
+      (not sent by the browser) *)
+   *
 
-       'a one_service_cookie_info session_cookie ref
-       (* SCNo_data = the session has been closed
-          SCData_session_expired = the cookie has not been found in the table.
-          For both of them, ask the browser to remove the cookie.
-        *)
-    )
-      (* This one is not lazy because we must check all service sessions
-         at each request to find the services *)
-      Full_state_name_table.t ref (* The key is the full session name *) *
+   'a one_service_cookie_info session_cookie ref
+   (* SCNo_data = the session has been closed
+      SCData_session_expired = the cookie has not been found in the table.
+      For both of them, ask the browser to remove the cookie.
+   *)
+  )
+    (* This one is not lazy because we must check all service sessions
+       at each request to find the services *)
+    Full_state_name_table.t ref (* The key is the full session name *) *
 
-    (* in memory data sessions: *)
-      (string option            (* value sent by the browser *)
-                                (* None = new cookie
-                                   (not sent by the browser) *)
-         *
+  (* in memory data sessions: *)
+  (string option            (* value sent by the browser *)
+   (* None = new cookie
+      (not sent by the browser) *)
+   *
 
-         one_data_cookie_info session_cookie ref
-         (* SCNo_data = the session has been closed
-            SCData_session_expired = the cookie has not been found in the table.
-            For both of them, ask the browser to remove the cookie.
-          *)
-      ) Lazy.t
-      (* Lazy because we do not want to ask the browser to unset the cookie
-         if the cookie has not been used, otherwise it is impossible to
-         write a message "Your session has expired" *)
-      Full_state_name_table.t ref (* The key is the full session name *) *
+   one_data_cookie_info session_cookie ref
+   (* SCNo_data = the session has been closed
+      SCData_session_expired = the cookie has not been found in the table.
+      For both of them, ask the browser to remove the cookie.
+   *)
+  ) Lazy.t
+    (* Lazy because we do not want to ask the browser to unset the cookie
+       if the cookie has not been used, otherwise it is impossible to
+       write a message "Your session has expired" *)
+    Full_state_name_table.t ref (* The key is the full session name *) *
 
-      (* persistent sessions: *)
-      ((string                  (* value sent by the browser *) *
-        timeout                 (* timeout at the beginning of the request *) *
-        float option            (* (server side) expdate
-                                   at the beginning of the request
-                                   None = no exp *) *
-        perssessgrp option      (* session group at beginning of request *))
-         option
-                                (* None = new cookie
-                                   (not sent by the browser) *)
-         *
+  (* persistent sessions: *)
+  ((string                  (* value sent by the browser *) *
+    timeout                 (* timeout at the beginning of the request *) *
+    float option            (* (server side) expdate
+                               at the beginning of the request
+                               None = no exp *) *
+    perssessgrp option      (* session group at beginning of request *))
+     option
+   (* None = new cookie
+      (not sent by the browser) *)
+   *
 
-         one_persistent_cookie_info session_cookie ref
-         (* SCNo_data = the session has been closed
-            SCData_session_expired = the cookie has not been found in the table.
-            For both of them, ask the browser to remove the cookie.
-          *)
-      ) Lwt.t Lazy.t
-      Full_state_name_table.t ref
+   one_persistent_cookie_info session_cookie ref
+   (* SCNo_data = the session has been closed
+      SCData_session_expired = the cookie has not been found in the table.
+      For both of them, ask the browser to remove the cookie.
+   *)
+  ) Lwt.t Lazy.t
+    Full_state_name_table.t ref
 
 
 type 'a cookie_info =
-    'a cookie_info1 (* unsecure *) *
-      'a cookie_info1 option (* secure, if https *)
+  'a cookie_info1 (* unsecure *) *
+  'a cookie_info1 (* secure *)
 
 
 
 (* non persistent cookies for services *)
 type 'a servicecookiestablecontent =
-    (full_state_name *
-     'a                  (* session table *) *
-     float option ref    (* expiration date by timeout
-                            (server side) *) *
-     timeout ref         (* user timeout *) *
-     cookie_level sessgrp ref   (* session group *) *
-     string Ocsigen_cache.Dlist.node (* session group node *))
+  (full_state_name *
+   'a                  (* session table *) *
+   float option ref    (* expiration date by timeout
+                          (server side) *) *
+   timeout ref         (* user timeout *) *
+   cookie_level sessgrp ref   (* session group *) *
+   string Ocsigen_cache.Dlist.node (* session group node *))
 
 type 'a servicecookiestable = 'a servicecookiestablecontent SessionCookies.t
 (* the table contains:
@@ -265,16 +260,16 @@ type 'a servicecookiestable = 'a servicecookiestablecontent SessionCookies.t
    - the timeout for the user (float option option) None -> see global config
      Some None -> no timeout
    - the group to which belongs the session
- *)
+*)
 
 (* non persistent cookies for in memory data *)
 type datacookiestablecontent =
-    (full_state_name *
-     float option ref        (* expiration date by timeout
-                                (server side) *) *
-     timeout ref             (* user timeout *) *
-     cookie_level sessgrp ref   (* session group *) *
-     string Ocsigen_cache.Dlist.node (* session group node *))
+  (full_state_name *
+   float option ref        (* expiration date by timeout
+                              (server side) *) *
+   timeout ref             (* user timeout *) *
+   cookie_level sessgrp ref   (* session group *) *
+   string Ocsigen_cache.Dlist.node (* session group node *))
 
 type datacookiestable = datacookiestablecontent SessionCookies.t
 
@@ -287,13 +282,13 @@ let ipv6mask = ref 56
 
 let get_mask4 m =
   match fst m with
-    | Some m -> m
-    | None -> !ipv4mask
+  | Some m -> m
+  | None -> !ipv4mask
 
 let get_mask6 m =
   match fst m with
-    | Some m -> m
-    | None -> !ipv6mask
+  | Some m -> m
+  | None -> !ipv6mask
 
 let network_of_ip k mask4 mask6 = match k with
   | Ipaddr.V4 ip -> Ipaddr.(V4 (V4.Prefix.(network (make mask4 ip))))
@@ -302,64 +297,40 @@ let network_of_ip k mask4 mask6 = match k with
 module Net_addr_Hashtbl =
   (* keys are IP address modulo "network equivalence" *)
   (struct
-     include Hashtbl.Make(struct
+    include Hashtbl.Make(struct
         type t = Ipaddr.t
         let equal a b = Ipaddr.compare a b = 0
-                            let hash = Hashtbl.hash
-                          end)
+        let hash = Hashtbl.hash
+      end)
 
-     let add m4 m6 t k v =
-       add t (network_of_ip k (get_mask4 m4) (get_mask6 m6)) v
+    let add m4 m6 t k v =
+      add t (network_of_ip k (get_mask4 m4) (get_mask6 m6)) v
 
-     let remove m4 m6 t k =
-       remove t (network_of_ip k (get_mask4 m4) (get_mask6 m6))
+    let remove m4 m6 t k =
+      remove t (network_of_ip k (get_mask4 m4) (get_mask6 m6))
 
-     let find m4 m6 t k =
-       find t (network_of_ip k (get_mask4 m4) (get_mask6 m6))
+    let find m4 m6 t k =
+      find t (network_of_ip k (get_mask4 m4) (get_mask6 m6))
 
-     let find_all m4 m6 t k =
-       find_all t (network_of_ip k (get_mask4 m4) (get_mask6 m6))
-
-     let replace m4 m6 t k v =
-       replace t (network_of_ip k (get_mask4 m4) (get_mask6 m6)) v
-
-     let mem m4 m6 t k =
-       mem t (network_of_ip k (get_mask4 m4) (get_mask6 m6))
-
-   end : sig
+  end : sig
 
      type key = Ipaddr.t
      type 'a t
      val create : int -> 'a t
-     val clear : 'a t -> unit
-     val copy : 'a t -> 'a t
      val add      : int option * 'bb -> int option * 'bb -> 'a t -> key -> 'a -> unit
      val remove   : int option * 'bb -> int option * 'bb -> 'a t -> key -> unit
      val find     : int option * 'bb -> int option * 'bb -> 'a t -> key -> 'a
-     val find_all : int option * 'bb -> int option * 'bb -> 'a t -> key -> 'a list
-     val replace  : int option * 'bb -> int option * 'bb -> 'a t -> key -> 'a -> unit
-     val mem      : int option * 'bb -> int option * 'bb -> 'a t -> key -> bool
-     val iter : (key -> 'a -> unit) -> 'a t -> unit
-     val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
-     val length : 'a t -> int
-
    end)
 
-type page_table_key =
-    {key_state : att_key_serv * att_key_serv;
-     key_kind: Ocsigen_http_frame.Http_header.http_method}
-
 module Serv_Table = Map.Make(struct
-  type t = page_table_key
-  let compare = compare
-end)
+    type t = page_table_key
+    let compare = compare
+  end)
 
 module NAserv_Table = Map.Make(struct
-  type t = na_key_serv
-  let compare = compare
-end)
-
-type anon_params_type = int
+    type t = na_key_serv
+    let compare = compare
+  end)
 
 type node_info = {
   ni_id : node_ref;
@@ -369,43 +340,31 @@ type node_info = {
 module Hier_set = String.Set
 
 type server_params =
-    {sp_request: Ocsigen_extensions.request;
-     sp_si: sess_info;
-     sp_sitedata: sitedata (* data for the whole site *);
-     sp_cookie_info: tables cookie_info;
-     sp_tab_cookie_info: tables cookie_info;
-     mutable sp_user_cookies: Ocsigen_cookies.cookieset;
-     (* cookies (un)set by the user during service *)
-     mutable sp_user_tab_cookies: Ocsigen_cookies.cookieset;
-     mutable sp_client_appl_name: string option; (* The application name,
-                                                    as sent by the browser *)
-     sp_suffix: Url.path option (* suffix *);
-     sp_full_state_name: full_state_name option
-                             (* the name of the session
-                                to which belong the service that answered
-                                (if it is a session service) *);
-     sp_client_process_info: client_process_info;
-    }
+  {sp_request: Ocsigen_extensions.request;
+   sp_si: sess_info;
+   sp_sitedata: sitedata (* data for the whole site *);
+   sp_cookie_info: tables cookie_info;
+   sp_tab_cookie_info: tables cookie_info;
+   mutable sp_user_cookies: Ocsigen_cookies.cookieset;
+   (* cookies (un)set by the user during service *)
+   mutable sp_user_tab_cookies: Ocsigen_cookies.cookieset;
+   mutable sp_client_appl_name: string option; (* The application name,
+                                                  as sent by the browser *)
+   sp_suffix: Url.path option (* suffix *);
+   sp_full_state_name: full_state_name option
+  (* the name of the session
+     to which belong the service that answered
+     (if it is a session service) *);
+   sp_client_process_info: client_process_info;
+  }
 
 and page_table = page_table_content Serv_Table.t
 
-and page_table_content =
-    Ptc of
+and page_table_content = [
+    `Ptc of
       (page_table ref * page_table_key, na_key_serv) leftright
-        Ocsigen_cache.Dlist.node option
-        (* for limitation of number of dynamic anonymous coservices *) *
-
-        ((anon_params_type * anon_params_type)
-           (* unique_id, computed from parameters type.
-              must be the same even if the actual service reference
-              is different (after reloading the site)
-              so that it replaces the former one
-           *) *
-            (int ref option (* max_use *) *
-               (float * float ref) option
-                 (* timeout and expiration date for the service *) *
-            (bool -> server_params -> Ocsigen_http_frame.result Lwt.t)
-            )) list
+        Ocsigen_cache.Dlist.node option *
+      (server_params, Ocsigen_http_frame.result) service list ]
 
 and naservice_table_content =
     (int (* generation (= number of reloads of sites
@@ -422,18 +381,10 @@ and naservice_table =
   | AVide
   | ATable of naservice_table_content NAserv_Table.t
 
-and dircontent =
-  | Vide
-  | Table of direlt ref String.Table.t
-
-and direlt =
-  | Dir of dircontent ref
-  | File of page_table ref
-
 and tables =
-    {mutable table_services : (int (* generation *) *
-                                 int (* priority *) *
-                                 dircontent ref) list;
+  {mutable table_services : (int (* generation *) *
+                             int (* priority *) *
+                             page_table dircontent ref) list;
      table_naservices : naservice_table ref;
      (* ref, and not mutable field because it simpler to use
         recursively with Dir of dircontent ref *)
@@ -460,15 +411,12 @@ and tables =
        (page_table ref * page_table_key, na_key_serv) leftright ->
        (page_table ref * page_table_key, na_key_serv) leftright
          Ocsigen_cache.Dlist.node
-       (* Add in a dlist
-          for limiting the number of dynamic anonymous coservices in each table
-          (and avoid DoS).
-          There is one dlist for each session, and one for each IP
-          in global tables.
-          The dlist parameter is the table and coservice number
-          for attached coservices,
-          and the coservice number for non-attached ones.
-       *)
+         (* We use a dlist for limiting the number of dynamic
+            anonymous coservices in each table (and avoid DoS).  There
+            is one dlist for each session, and one for each IP in
+            global tables.  The dlist parameter is the table and
+            coservice number for attached coservices, and the
+            coservice number for non-attached ones. *)
     }
 
 and sitedata =
@@ -497,7 +445,7 @@ and sitedata =
      ((full_state_name * (float option * bool)) list);
 
    site_value_table : Polytables.t; (* table containing evaluated
-				       lazy site values *)
+                                       lazy site values *)
 
    mutable registered_scope_hierarchies: Hier_set.t;
 
@@ -529,10 +477,14 @@ and sitedata =
    mutable max_persistent_data_tab_sessions_per_group : int option * bool;
    mutable max_anonymous_services_per_session : int * bool;
    mutable max_anonymous_services_per_subnet : int * bool;
+   mutable secure_cookies : bool;
    dlist_ip_table : dlist_ip_table;
    mutable ipv4mask : int option * bool;
    mutable ipv6mask : int option * bool;
- }
+   mutable application_script : bool (* defer *) * bool; (* async *)
+   mutable cache_global_data : (string list * int) option;
+   mutable html_content_type : string option;
+  }
 
 and dlist_ip_table = (page_table ref * page_table_key, na_key_serv)
     leftright Ocsigen_cache.Dlist.t Net_addr_Hashtbl.t
@@ -597,12 +549,13 @@ let make_server_params
     match si.si_client_process_info with
     | Some cpi -> cpi
     | None ->
-	let request_info = ri.Ocsigen_extensions.request_info in
-	{ cpi_ssl = Ocsigen_extensions.Ocsigen_request_info.ssl request_info;
-	  cpi_hostname = Ocsigen_extensions.get_hostname ri;
-	  cpi_server_port = Ocsigen_extensions.get_port ri;
-    cpi_original_full_path = Ocsigen_extensions.Ocsigen_request_info.original_full_path request_info;
-	}
+      let request_info = ri.Ocsigen_extensions.request_info in
+      { cpi_ssl = Ocsigen_extensions.Ocsigen_request_info.ssl request_info;
+        cpi_hostname = Ocsigen_extensions.get_hostname ri;
+        cpi_server_port = Ocsigen_extensions.get_port ri;
+        cpi_original_full_path =
+          Ocsigen_extensions.Ocsigen_request_info.original_full_path request_info;
+      }
   in
   { sp_request = ri;
     sp_si = si;
@@ -623,8 +576,8 @@ let get_sp_option () = Lwt.get sp_key
 
 let get_sp () =
   match Lwt.get sp_key with
-    | Some sp -> sp
-    | None -> failwith "That function cannot be called here because it needs information about the request or the site."
+  | Some sp -> sp
+  | None -> failwith "This function cannot be called here because it needs information about the request or the site."
 
 let sp_of_option sp =
   match sp with
@@ -656,7 +609,7 @@ let register_scope_hierarchy (name:string) =
         Hier_set.add name !registered_scope_hierarchies
     | Some sp ->
       if Hier_set.mem name !registered_scope_hierarchies ||
-	Hier_set.mem name sp.sp_sitedata.registered_scope_hierarchies
+         Hier_set.mem name sp.sp_sitedata.registered_scope_hierarchies
       then failwith (Printf.sprintf "the scope hierarchy %s has already been registered" name)
       else sp.sp_sitedata.registered_scope_hierarchies <-
         Hier_set.add name sp.sp_sitedata.registered_scope_hierarchies
@@ -764,15 +717,15 @@ let force_lazy_site_value v =
                          "force_lazy_site_value")
   in
   try Polytables.get
-	~table:sitedata.site_value_table
-	~key:v.lazy_sv_key
+    ~table:sitedata.site_value_table
+    ~key:v.lazy_sv_key
   with
     | Not_found ->
       let value = v.lazy_sv_fun () in
       Polytables.set
-	~table:sitedata.site_value_table
-	~key:v.lazy_sv_key
-	~value;
+        ~table:sitedata.site_value_table
+        ~key:v.lazy_sv_key
+        ~value;
       value
 
 let lazy_site_value_from_fun f =
@@ -788,7 +741,6 @@ let lazy_site_value_from_fun f =
 
 
 let empty_page_table () = Serv_Table.empty
-let empty_dircontent () = Vide
 let empty_naservice_table () = AVide
 
 let service_tables_are_empty t =
@@ -887,43 +839,6 @@ let get_mask6 sitedata = get_mask6 sitedata.ipv6mask
 (*****************************************************************************)
 open Lwt
 
-(* Split parameter list, removing those whose name starts with pref *)
-let split_prefix_param pref l =
-  let len = String.length pref in
-  List.partition (fun (n,_) ->
-    try
-      (String.sub n 0 len) = pref
-    with Invalid_argument _ -> false) l
-
-(* Special version for non localized parameters *)
-let prefixlength = String.length nl_param_prefix
-let prefixlengthminusone = prefixlength - 1
-
-let split_nl_prefix_param l =
-    let rec aux other map = function
-      | [] -> (map, other)
-      | ((n, v) as a)::l ->
-          if String.first_diff
-            n nl_param_prefix 0 prefixlengthminusone = prefixlength
-          then
-            try
-              let last = String.index_from n prefixlength '.' in
-              let nl_param_name =
-                String.sub n prefixlength (last - prefixlength)
-              in
-              let previous =
-                try String.Table.find nl_param_name map
-                with Not_found -> []
-              in
-              aux
-                other
-                (String.Table.add nl_param_name (a::previous) map)
-                l
-            with Invalid_argument _ | Not_found -> aux (a::other) map l
-          else aux (a::other) map l
-    in
-    aux [] String.Table.empty l
-
 (* The cookie name is
 
 sessionkind|S?|sitedirstring|"ref" ou "comet" ou ""|hiername
@@ -965,24 +880,11 @@ let getcookies secure cookie_level cookienamepref cookies =
     cookies
     Full_state_name_table.empty
 
-(* Remove all parameters whose name starts with pref *)
-let remove_prefixed_param pref l =
-  let len = String.length pref in
-  let rec aux = function
-    | [] -> []
-    | ((n,v) as a)::l ->
-        try
-          if (String.sub n 0 len) = pref then
-            aux l
-          else a::(aux l)
-        with Invalid_argument _ -> a::(aux l)
-  in aux l
-
 (* After an action, we do not take into account actual get params,
    but these ones: *)
 let eliom_params_after_action = Polytables.make_key ()
 
-(* After an ction, we get tab_cookies info from rc: *)
+(* After an action, we get tab_cookies info from rc: *)
 let tab_cookie_action_info_key = Polytables.make_key ()
 
 type cpi = client_process_info =  {
@@ -990,7 +892,7 @@ type cpi = client_process_info =  {
   cpi_hostname : string;
   cpi_server_port : int;
   cpi_original_full_path : string list;
-} deriving (Json)
+} [@@deriving json]
 
 let get_session_info req previous_extension_err =
   let req_whole = req
@@ -1001,16 +903,16 @@ let get_session_info req previous_extension_err =
   let rc = Ocsigen_extensions.Ocsigen_request_info.request_cache ri in
   let no_post_param, p =
     match Ocsigen_extensions.Ocsigen_request_info.post_params ri with
-      | None -> true, Lwt.return []
+      | None -> true, Lwt.return_nil
       | Some f -> false, f (ci.Ocsigen_extensions.uploaddir, ci.Ocsigen_extensions.maxuploadfilesize)
   in
   let no_file_param, file_params =
     match Ocsigen_extensions.Ocsigen_request_info.files ri with
-      | None -> true, Lwt.return []
+      | None -> true, Lwt.return_nil
       | Some f -> false, f (ci.Ocsigen_extensions.uploaddir, ci.Ocsigen_extensions.maxuploadfilesize)
   in
 
-  lwt post_params = p in
+  let%lwt post_params = p in
 
   let (previous_tab_cookies_info, tab_cookies, post_params) =
     try
@@ -1030,7 +932,7 @@ let get_session_info req previous_extension_err =
           let (tc, pp) =
             List.assoc_remove tab_cookies_param_name post_params
           in
-	  let tc = Json.from_string<(string * string) list> tc in
+          let tc = [%of_json: (string * string) list] tc in
           (List.fold_left (fun t (k,v) -> CookiesTable.add k v t) CookiesTable.empty tc, pp)
           (*Marshal.from_string (Ocsigen_lib.decode tc) 0, pp*)
         with Not_found ->
@@ -1038,7 +940,7 @@ let get_session_info req previous_extension_err =
             let tc = Ocsigen_headers.find tab_cookies_header_name
               (Ocsigen_extensions.Ocsigen_request_info.http_frame ri)
             in
-	    let tc = Json.from_string<(string * string) list> tc in
+            let tc = [%of_json: (string * string) list] tc in
             (List.fold_left (fun t (k,v) -> CookiesTable.add k v t) CookiesTable.empty tc,
              post_params)
           with Not_found -> CookiesTable.empty, post_params
@@ -1050,20 +952,20 @@ let get_session_info req previous_extension_err =
   let cpi =
     try (* looking for client process info in headers *)
       let cpi =
-	Ocsigen_headers.find
-	  tab_cpi_header_name
-	  (Ocsigen_extensions.Ocsigen_request_info.http_frame ri) in
-      Some (Json.from_string<cpi> cpi)
+        Ocsigen_headers.find
+          tab_cpi_header_name
+          (Ocsigen_extensions.Ocsigen_request_info.http_frame ri) in
+      Some ([%of_json: cpi] cpi)
     with Not_found -> None
   in
 
   let epd =
     lazy (try (* looking in headers *)
             let epd = Ocsigen_headers.find
-	      expecting_process_page_name
-	      (Ocsigen_extensions.Ocsigen_request_info.http_frame ri)
+              expecting_process_page_name
+              (Ocsigen_extensions.Ocsigen_request_info.http_frame ri)
             in
-            Json.from_string<bool> epd
+            [%of_json: bool] epd
       with Not_found -> false)
   in
 
@@ -1092,7 +994,7 @@ let get_session_info req previous_extension_err =
 
   let get_params0 = get_params in
   let post_params0 = post_params in
-  lwt file_params0 = file_params in
+  let%lwt file_params0 = file_params in
   let get_params, post_params, file_params,
       (all_get_params, all_post_params, all_file_params,
        nl_get_params, nl_post_params, nl_file_params,
@@ -1117,22 +1019,30 @@ let get_session_info req previous_extension_err =
        all_get_but_nl (*204FORMS*, internal_form *))
   in
 
-  let browser_cookies = Lazy.force (Ocsigen_extensions.Ocsigen_request_info.cookies ri) in
+  let browser_cookies =
+    try (* Cookie substitutes for iOS WKWebView *)
+      let tc = Ocsigen_headers.find cookie_substitutes_header_name
+          (Ocsigen_extensions.Ocsigen_request_info.http_frame ri) in
+      let tc = [%of_json: (string * string) list] tc in
+        List.fold_left (fun t (k,v) -> CookiesTable.add k v t)
+          CookiesTable.empty tc
+    with Not_found ->
+      Lazy.force (Ocsigen_extensions.Ocsigen_request_info.cookies ri) in
 
   let data_cookies = getcookies false `Session datacookiename browser_cookies in
   let service_cookies = getcookies false `Session servicecookiename browser_cookies in
   let persistent_cookies = getcookies false `Session persistentcookiename browser_cookies in
 
   let secure_cookie_info =
-    if Ocsigen_extensions.Ocsigen_request_info.ssl ri
-    then
-      let sdata_cookies = getcookies true `Session datacookiename browser_cookies in
-      let sservice_cookies = getcookies true `Session servicecookiename browser_cookies in
-      let spersistent_cookies =
-        getcookies true `Session persistentcookiename browser_cookies
-      in
-      Some (sservice_cookies, sdata_cookies, spersistent_cookies)
-    else None
+    let sdata_cookies = getcookies true `Session datacookiename browser_cookies
+    in
+    let sservice_cookies =
+      getcookies true `Session servicecookiename browser_cookies
+    in
+    let spersistent_cookies =
+      getcookies true `Session persistentcookiename browser_cookies
+    in
+    (sservice_cookies, sdata_cookies, spersistent_cookies)
   in
 
   let naservice_info,
@@ -1252,13 +1162,16 @@ let get_session_info req previous_extension_err =
   let persistent_cookies_tab = getcookies false `Client_process persistentcookiename tab_cookies in
 
   let secure_cookie_info_tab =
-    if Ocsigen_extensions.Ocsigen_request_info.ssl ri
-    then
-      let sdata_cookies = getcookies true `Client_process datacookiename tab_cookies in
-      let sservice_cookies = getcookies true `Client_process servicecookiename tab_cookies in
-      let spersistent_cookies = getcookies true `Client_process persistentcookiename tab_cookies in
-      Some (sservice_cookies, sdata_cookies, spersistent_cookies)
-    else None
+    let sdata_cookies =
+      getcookies true `Client_process datacookiename tab_cookies
+    in
+    let sservice_cookies =
+      getcookies true `Client_process servicecookiename tab_cookies
+    in
+    let spersistent_cookies =
+      getcookies true `Client_process persistentcookiename tab_cookies
+    in
+    (sservice_cookies, sdata_cookies, spersistent_cookies)
   in
 
   let get_params_string, url_string =
@@ -1321,11 +1234,7 @@ let get_session_info req previous_extension_err =
      si_nl_file_params= nl_file_params;
      si_persistent_nl_get_params= persistent_nl_get_params;
      si_all_get_but_nl= all_get_but_nl;
-     si_all_get_but_na_nl=
-        lazy
-          (List.remove_assoc naservice_name
-             (List.remove_assoc naservice_num
-                (remove_prefixed_param na_co_param_prefix all_get_but_nl)));
+     si_all_get_but_na_nl= lazy (remove_na_prefix_params all_get_but_nl);
      si_client_process_info= cpi;
      si_expect_process_data= epd;
 (*204FORMS*     si_internal_form= internal_form; *)
@@ -1335,14 +1244,6 @@ let get_session_info req previous_extension_err =
     ({ req_whole with Ocsigen_extensions.request_info = ri' }, sess,
      previous_tab_cookies_info)
 
-
-
-
-type ('a, 'b) foundornot = Found of 'a | Notfound of 'b
-
-
-
-(*****************************************************************************)
 type info =
     (Ocsigen_extensions.request *
        sess_info *
@@ -1371,7 +1272,7 @@ let create_persistent_table name =
 
 let persistent_cookies_table :
     (full_state_name * float option * timeout * perssessgrp option)
-    Ocsipersist.table Lazy.t =
+    Ocsipersist.table Lwt.t Lazy.t =
   lazy (create_persistent_table eliom_persistent_cookie_table)
 (* Another tables, containing the session info for each cookie *)
 (* the table contains:
@@ -1389,8 +1290,9 @@ let persistent_cookies_table :
 let remove_from_all_persistent_tables key =
   Perstables.fold (* could be replaced by a parallel map *)
     (fun thr t -> thr >>= fun () ->
-      Ocsipersist.remove (Ocsipersist.open_table t) key >>= Lwt_unix.yield)
-    (return ())
+      Ocsipersist.open_table t >>= fun table ->
+      Ocsipersist.remove table key >>= Lwt_unix.yield)
+    return_unit
     !perstables
 
 
@@ -1437,3 +1339,47 @@ let patch_request_info req =
 
 let get_site_dir sitedata = sitedata.site_dir
 let get_site_dir_string sitedata = sitedata.site_dir_string
+
+(* Returns if we want secure cookie *)
+let get_secure ~secure_o ~sitedata () =
+  match secure_o with
+  | None -> sitedata.secure_cookies
+  | Some s -> s
+
+module To_and_of_shared = struct
+
+  (* FIXME : work-around for weak polymorphism in create :( *)
+  type wrapper
+
+  type 'a t = {
+    server  : 'a to_and_of ;
+    client  : 'a to_and_of Eliom_client_value.t option;
+    wrapper : wrapper
+  }
+
+  let wrapper : wrapper = Obj.magic @@
+    Eliom_wrap.create_wrapper @@ function
+    | {client = Some tao} ->
+      tao
+    | {client = None} ->
+      failwith
+        "Cannot wrap user type parameter.\n\
+         Use the ?client_to_and_of parameter of Eliom_parameter.user_type\n\
+         or (Eliom_parameter.all_suffix_user)"
+
+  let to_string {server = {to_string}} = to_string
+
+  let of_string {server = {of_string}} = of_string
+
+  let to_and_of {server} = server
+
+  let create ?client_to_and_of server = {
+    server ;
+    client = client_to_and_of ;
+    wrapper
+  }
+
+end
+
+let client_html_file () =
+  failwith "client_html_file is only defined on client"
