@@ -20,28 +20,25 @@
 
 (** Low level functions for Eliom, exceptions and types. *)
 
-open Ocsigen_cookies
-
 open Eliom_lib
 
-
 (** {2 Scopes} *)
 (* those types are not available to the user, a scope must be created using
    create_..._scope functions *)
 type scope_hierarchy = Eliom_common_base.scope_hierarchy
 
 type cookie_scope = [ `Session of scope_hierarchy
-		    | `Client_process of scope_hierarchy ]
+                    | `Client_process of scope_hierarchy ]
 
 type user_scope = [ `Session_group of scope_hierarchy
                   | cookie_scope ]
 
 type scope = [ `Site
-	     | user_scope ]
+             | user_scope ]
 
 type all_scope = [ scope
                  | `Global
-		 | `Request ]
+                 | `Request ]
 
 type cookie_level = [ `Session | `Client_process ]
 
@@ -71,7 +68,7 @@ val level_of_user_scope : [< user_scope ] -> [> user_level ]
     use [~scope:Eliom_common.default_process_scope].
 
     You can create new scope
-    hierachies with {!Eliom_common.create_scope_hierarchy}.
+    hierarchies with {!Eliom_common.create_scope_hierarchy}.
     Thus it is possible to have for example several sessions that can
     be opened or closed independently. They use different cookies.
 
@@ -140,7 +137,6 @@ val eliom_service_session_expired :
 
 (**/**)
 
-
 (*VVV Warning: raising these exceptions will NOT send cookies!
   Do not use them inside services! *)
 exception Eliom_do_redirection of string
@@ -239,12 +235,6 @@ val datacookiename : string
 val servicecookiename : string
 val persistentcookiename : string
 
-val persistent_cookie_table_version : string
-val eliom_persistent_cookie_table : string
-
-val inline_class_name : string
-val nodisplay_class_name : string
-
 val appl_name_cookie_name : string
 val tab_cookies_param_name : string
 val to_be_considered_as_get_param_name : string
@@ -271,18 +261,18 @@ type sess_info = {
   si_persistent_session_cookies : string Full_state_name_table.t;
   si_secure_cookie_info:
     (string Full_state_name_table.t *
-       string Full_state_name_table.t *
-       string Full_state_name_table.t) option;
+     string Full_state_name_table.t *
+     string Full_state_name_table.t);
 
   si_service_session_cookies_tab: string Full_state_name_table.t;
   si_data_session_cookies_tab: string Full_state_name_table.t;
   si_persistent_session_cookies_tab: string Full_state_name_table.t;
   si_secure_cookie_info_tab:
     (string Full_state_name_table.t *
-       string Full_state_name_table.t *
-       string Full_state_name_table.t) option;
+     string Full_state_name_table.t *
+     string Full_state_name_table.t);
 
-  si_tab_cookies: string CookiesTable.t;
+  si_tab_cookies: string Ocsigen_cookies.CookiesTable.t;
 
   si_nonatt_info : na_key_req;
   si_state_info: (att_key_req * att_key_req);
@@ -321,7 +311,7 @@ val make_persistent_full_group_name :
 
 val getperssessgrp : perssessgrp ->
   (string * cookie_level *
-     (string, Ipaddr.t) leftright)
+   (string, Ipaddr.t) leftright)
 
 val string_of_perssessgrp : perssessgrp -> string
 
@@ -372,7 +362,7 @@ type 'a cookie_info1 =
 
 type 'a cookie_info =
     'a cookie_info1 (* unsecure *) *
-      'a cookie_info1 option (* secure, if https *)
+      'a cookie_info1  (* secure *)
 
 type 'a servicecookiestablecontent =
     full_state_name * 'a * float option ref * timeout ref *
@@ -386,9 +376,12 @@ type datacookiestablecontent =
       string Ocsigen_cache.Dlist.node
 type datacookiestable =
     datacookiestablecontent SessionCookies.t
+
+type meth = [`Get | `Post | `Put | `Delete | `Other]
+
 type page_table_key = {
   key_state : att_key_serv * att_key_serv;
-  key_kind : Ocsigen_http_frame.Http_header.http_method;
+  key_meth  : meth
 }
 
 module NAserv_Table : Map.S with type key = na_key_serv
@@ -407,6 +400,16 @@ type node_info = {
 
 module Hier_set : Set.S
 
+type 'a dircontent = Vide | Table of 'a direlt ref String.Table.t
+and 'a direlt = Dir of 'a dircontent ref | File of 'a ref
+
+type ('params, 'result) service = {
+  s_id              : anon_params_type * anon_params_type;
+  mutable s_max_use : int option;
+  s_expire          : (float * float ref) option;
+  s_f               : bool -> 'params -> 'result Lwt.t
+}
+
 type server_params = {
   sp_request : Ocsigen_extensions.request;
   sp_si : sess_info;
@@ -431,23 +434,11 @@ type server_params = {
 }
 and page_table = page_table_content Serv_Table.t
 
-and page_table_content =
-    Ptc of
+and page_table_content = [
+    `Ptc of
       (page_table ref * page_table_key, na_key_serv) leftright
-        Ocsigen_cache.Dlist.node option
-        (* for limitation of number of dynamic anonymous coservices *) *
-
-        ((anon_params_type * anon_params_type)
-           (* unique_id, computed from parameters type.
-              must be the same even if the actual service reference
-              is different (after reloading the site)
-              so that it replaces the former one
-           *) *
-            (int ref option (* max_use *) *
-               (float * float ref) option
-                 (* timeout and expiration date for the service *) *
-            (bool -> server_params -> Ocsigen_http_frame.result Lwt.t)
-            )) list
+        Ocsigen_cache.Dlist.node option *
+      (server_params, Ocsigen_http_frame.result) service list ]
 
 and naservice_table_content =
     (int (* generation (= number of reloads of sites
@@ -464,12 +455,10 @@ and naservice_table =
   | AVide
   | ATable of naservice_table_content NAserv_Table.t
 
-and dircontent = Vide | Table of direlt ref String.Table.t
-and direlt = Dir of dircontent ref | File of page_table ref
 and tables =
     {mutable table_services : (int (* generation *) *
-                         int (* priority *) *
-                         dircontent ref) list;
+                               int (* priority *) *
+                               page_table dircontent ref) list;
      table_naservices : naservice_table ref;
     (* Information for the GC: *)
      mutable table_contains_services_with_timeout : bool;
@@ -530,7 +519,7 @@ and sitedata = {
      ((full_state_name * (float option * bool)) list);
 
   site_value_table : Polytables.t; (* table containing evaluated
-					   lazy site values *)
+                                      lazy site values *)
 
   mutable registered_scope_hierarchies: Hier_set.t;
 
@@ -554,15 +543,19 @@ and sitedata = {
   mutable max_persistent_data_tab_sessions_per_group : int option * bool;
   mutable max_anonymous_services_per_session : int * bool;
   mutable max_anonymous_services_per_subnet : int * bool;
+  mutable secure_cookies : bool; (* Use secure cookies (default is false). *)
   dlist_ip_table : dlist_ip_table;
   mutable ipv4mask : int option * bool;
   mutable ipv6mask : int option * bool;
+  mutable application_script : bool (* defer *) * bool; (* async *)
+  mutable cache_global_data : (string list * int) option;
+  mutable html_content_type : string option;
 }
 
 type 'a lazy_site_value (** lazy site values, are lazy values with
-			   content available only in the context of a
-			   site: the closure one time for each site (
-			   requesting it ) *)
+                            content available only in the context of a
+                            site: the closure one time for each site (
+                            requesting it ) *)
 
 val force_lazy_site_value : 'a lazy_site_value -> 'a
 val lazy_site_value_from_fun : ( unit -> 'a ) -> 'a lazy_site_value
@@ -580,7 +573,7 @@ val make_server_params :
   Url.path option ->
   full_state_name option -> server_params
 val empty_page_table : unit -> page_table
-val empty_dircontent : unit -> dircontent
+val empty_dircontent : unit -> 'a dircontent
 val empty_naservice_table : unit -> naservice_table
 val service_tables_are_empty : tables -> bool
 val empty_tables : int -> bool -> tables
@@ -608,10 +601,10 @@ module Perstables :
     val fold : ('a -> 'b -> 'a) -> 'a -> 'b list -> 'a
   end
 val perstables : string list ref
-val create_persistent_table : string -> 'a Ocsipersist.table
+val create_persistent_table : string -> 'a Ocsipersist.table Lwt.t
 val persistent_cookies_table :
   (full_state_name * float option * timeout * perssessgrp option)
-  Ocsipersist.table Lazy.t
+  Ocsipersist.table Lwt.t Lazy.t
 val remove_from_all_persistent_tables : string -> unit Lwt.t
 val absolute_change_sitedata : sitedata -> unit
 val get_current_sitedata : unit -> sitedata
@@ -634,7 +627,7 @@ val get_site_data : unit -> sitedata
 val eliom_params_after_action :
   ((string * string) list *
    (string * string) list option *
-   (string * Eliom_lib.file_info) list option *
+   (string * file_info) list option *
      (string * string) list String.Table.t *
      (string * string) list String.Table.t *
    (string * file_info) list String.Table.t *
@@ -663,10 +656,10 @@ val find_dlist_ip_table :
 
 val get_cookie_info : server_params -> [< cookie_level ] -> tables cookie_info
 
-val tab_cookie_action_info_key : (tables cookie_info *
-                                    Ocsigen_cookies.cookieset *
-                                    string CookiesTable.t) Polytables.key
-
+val tab_cookie_action_info_key :
+  (tables cookie_info *
+   Ocsigen_cookies.cookieset *
+   string Ocsigen_cookies.CookiesTable.t) Polytables.key
 
 val sp_key : server_params Lwt.key
 val get_sp_option : unit -> server_params option
@@ -699,15 +692,48 @@ val nl_get_appl_parameter: string
 val patch_request_info: Ocsigen_extensions.request -> Ocsigen_extensions.request
 
 type eliom_js_page_data = {
-  ejs_global_data: global_data option;
-  ejs_request_data: request_data;
+  ejs_global_data:
+    (Eliom_runtime.global_data * Eliom_wrap.unwrapper) option;
+  ejs_request_data: Eliom_runtime.request_data;
   (* Event handlers *)
-  ejs_event_handler_table: Eliom_lib.RawXML.event_handler_table;
+  ejs_event_handler_table: Eliom_runtime.RawXML.event_handler_table;
   (* Client attrib *)
-  ejs_client_attrib_table: Eliom_lib.RawXML.client_attrib_table;
+  ejs_client_attrib_table: Eliom_runtime.RawXML.client_attrib_table;
   (* Session info *)
   ejs_sess_info: sess_info;
 }
 
 val get_site_dir : sitedata -> Url.path
 val get_site_dir_string : sitedata -> string
+val get_secure : secure_o:bool option -> sitedata:sitedata -> unit -> bool
+
+val is_client_app : bool ref
+
+val make_actual_path : string list -> string list
+
+type 'a to_and_of = {
+  of_string : string -> 'a;
+  to_string : 'a -> string
+}
+
+module To_and_of_shared : sig
+
+  type 'a t
+
+  val create :
+    ?client_to_and_of : 'a to_and_of Eliom_client_value.t ->
+    'a to_and_of ->
+    'a t
+
+  val to_string : 'a t -> 'a -> string
+
+  val of_string : 'a t -> string -> 'a
+
+  val to_and_of : 'a t -> 'a to_and_of
+
+end
+
+(*/*)
+
+(** Raises exception on server, only relevant for client apps *)
+val client_html_file : unit -> string
