@@ -31,15 +31,15 @@ type scope_hierarchy =
   | Default_comet_hier
 
 type user_scope = [ `Session_group of scope_hierarchy
-		  | `Session of scope_hierarchy
-		  | `Client_process of scope_hierarchy ]
+                  | `Session of scope_hierarchy
+                  | `Client_process of scope_hierarchy ]
 
 type scope = [ `Site
-	     | user_scope ]
+             | user_scope ]
 
 type all_scope = [ scope
                  | `Global
-		 | `Request ]
+                 | `Request ]
 
 type global_scope = [`Global]
 type site_scope = [`Site]
@@ -166,8 +166,6 @@ let na_co_param_prefix = "__na_eliom_"
 let nl_param_prefix = "__nl_"
 let pnl_param_prefix = nl_param_prefix^"p_"
 let npnl_param_prefix = nl_param_prefix^"n_"
-let nodisplay_class_name = "eliom_nodisplay"
-let inline_class_name = "eliom_inline"
 
 let eliom_internal_nlp_prefix = "__eliom"
 let tab_cookies_param_name = "__eliom_P_tab_cookies"
@@ -179,19 +177,14 @@ let half_xhr_redir_header = "X-Eliom-Location-Half"
 let response_url_header = "X-Eliom-Url"
 
 let default_group_name = "__eliom$%@default_group"
-(*204FORMS* old implementation of forms with 204 and change_page_event
-let internal_form_name = "_internal_form"
-let internal_form_bool_name = "b"
-
-let internal_form_full_name =
-  npnl_param_prefix^
-    eliom_internal_nlp_prefix^"-"^
-    internal_form_name^"."^
-    internal_form_bool_name
-*)
 
 let set_tab_cookies_header_name = "X-Eliom-Set-Process-Cookies"
 let tab_cookies_header_name = "X-Eliom-Process-Cookies"
+
+(* Cookie substitutes for iOS WKWebView *)
+let cookie_substitutes_header_name = "X-Eliom-Cookie-Substitutes"
+let set_cookie_substitutes_header_name = "X-Eliom-Set-Cookie-Substitutes"
+
 let tab_cpi_header_name = "X-Eliom-Process-Info"
 let expecting_process_page_name = "X-Eliom-Expecting-Process-Page"
 
@@ -228,9 +221,9 @@ type sess_info =
 
      si_secure_cookie_info:
        (string Full_state_name_table.t *
-          string Full_state_name_table.t *
-          string Full_state_name_table.t) option;
-     (* the same, but for secure cookies, if https *)
+        string Full_state_name_table.t *
+        string Full_state_name_table.t);
+     (* the same, but for secure cookies *)
 
      (* now for tab cookies: *)
      si_service_session_cookies_tab: string Full_state_name_table.t;
@@ -238,8 +231,8 @@ type sess_info =
      si_persistent_session_cookies_tab: string Full_state_name_table.t;
      si_secure_cookie_info_tab:
        (string Full_state_name_table.t *
-          string Full_state_name_table.t *
-          string Full_state_name_table.t) option;
+        string Full_state_name_table.t *
+        string Full_state_name_table.t);
 
      si_tab_cookies: string CookiesTable.t;
 
@@ -264,28 +257,29 @@ type sess_info =
    }
 
 type eliom_js_page_data = {
-  ejs_global_data: global_data option;
-  ejs_request_data: request_data;
+  ejs_global_data:
+    (Eliom_runtime.global_data * Eliom_wrap.unwrapper) option;
+  ejs_request_data: Eliom_runtime.request_data;
   (* Event handlers *)
-  ejs_event_handler_table: Eliom_lib.RawXML.event_handler_table;
+  ejs_event_handler_table: Eliom_runtime.RawXML.event_handler_table;
   (* Client Attributes *)
-  ejs_client_attrib_table: Eliom_lib.RawXML.client_attrib_table;
+  ejs_client_attrib_table: Eliom_runtime.RawXML.client_attrib_table;
   (* Session info *)
   ejs_sess_info: sess_info;
 }
 
 (************ unwrapping identifiers *********************)
 
-let tyxml_unwrap_id_int = Eliom_lib_base.tyxml_unwrap_id_int
+let tyxml_unwrap_id_int = Eliom_runtime.tyxml_unwrap_id_int
 let () = assert (tyxml_unwrap_id_int = 1)
 let comet_channel_unwrap_id_int = 2
 let react_up_unwrap_id_int = 3
 let react_down_unwrap_id_int = 4
 let signal_down_unwrap_id_int = 5
 let bus_unwrap_id_int = 6
-let client_value_unwrap_id_int = Eliom_lib_base.client_value_unwrap_id_int
+let client_value_unwrap_id_int = Eliom_runtime.client_value_unwrap_id_int
 let () = assert (client_value_unwrap_id_int = 7)
-let global_data_unwrap_id_int = Eliom_lib_base.global_data_unwrap_id_int
+let global_data_unwrap_id_int = Eliom_runtime.global_data_unwrap_id_int
 let () = assert (global_data_unwrap_id_int = 8)
 let server_function_unwrap_id_int = 9
 
@@ -297,3 +291,126 @@ type node_ref = string
 (* CCC take care: this must remain of the same syntax as non localised
    non persistent get parameter name *)
 let nl_get_appl_parameter = "__nl_n_eliom-process.p"
+
+
+(* make a path by going up when there is a '..' *)
+let make_actual_path path =
+  let rec aux accu path =
+    match (accu, path) with
+    | ([], ".."::path') -> aux accu path'
+    | (_::accu',  ".."::path') -> aux accu' path'
+    | (_,  a::path') -> aux (a::accu) path'
+    | (_,  []) -> accu
+  in match path with
+  | ""::path -> ""::List.rev (aux [] path)
+  | _ -> List.rev (aux [] path)
+
+let is_client_app = ref false
+
+(* Special version for non localized parameters *)
+let prefixlength = String.length nl_param_prefix
+let prefixlengthminusone = prefixlength - 1
+
+let split_nl_prefix_param l =
+    let rec aux other map = function
+      | [] -> (map, other)
+      | ((n, v) as a)::l ->
+          if String.first_diff
+            n nl_param_prefix 0 prefixlengthminusone = prefixlength
+          then
+            try
+              let last = String.index_from n prefixlength '.' in
+              let nl_param_name =
+                String.sub n prefixlength (last - prefixlength)
+              in
+              let previous =
+                try String.Table.find nl_param_name map
+                with Not_found -> []
+              in
+              aux
+                other
+                (String.Table.add nl_param_name (a::previous) map)
+                l
+            with Invalid_argument _ | Not_found -> aux (a::other) map l
+          else aux (a::other) map l
+    in
+    aux [] String.Table.empty l
+
+(* Split parameter list, removing those whose name starts with pref *)
+let split_prefix_param pref l =
+  let len = String.length pref in
+  List.partition (fun (n,_) ->
+    try
+      (String.sub n 0 len) = pref
+    with Invalid_argument _ -> false) l
+
+(* Remove all parameters whose name starts with pref *)
+let remove_prefixed_param pref l =
+  let len = String.length pref in
+  let rec aux = function
+    | [] -> []
+    | ((n,v) as a)::l ->
+        try
+          if (String.sub n 0 len) = pref then
+            aux l
+          else a::(aux l)
+        with Invalid_argument _ -> a::(aux l)
+  in aux l
+
+let remove_na_prefix_params l =
+  remove_prefixed_param na_co_param_prefix l
+  |> List.remove_assoc naservice_name
+  |> List.remove_assoc naservice_num
+
+let filter_na_get_params = List.filter @@ fun (s, (_ : string)) ->
+  s = naservice_name
+  || s = naservice_num
+  || String.sub s 0 (String.length na_co_param_prefix) = na_co_param_prefix
+
+exception Eliom_404
+
+type ('a, 'b) foundornot = Found of 'a | Notfound of 'b
+
+(** Service called with wrong parameter names *)
+
+exception Eliom_Wrong_parameter
+
+exception Eliom_duplicate_registration of string
+
+exception Eliom_page_erasing of string
+
+type 'a dircontent =
+  | Vide
+  | Table of 'a direlt ref String.Table.t
+
+and 'a direlt =
+  | Dir of 'a dircontent ref
+  | File of 'a ref
+
+let empty_dircontent () = Vide
+
+type meth = [`Get | `Post | `Put | `Delete | `Other]
+
+type page_table_key = {
+  key_state : att_key_serv * att_key_serv;
+  key_meth  : meth
+}
+
+type anon_params_type = int
+
+exception Eliom_Typing_Error of (string * exn) list
+
+type ('params, 'result) service = {
+  (* unique_id, computed from parameters type.  must be the same even
+     if the actual service reference is different (after reloading the
+     site) so that it replaces the former one *)
+  s_id              : anon_params_type * anon_params_type;
+  mutable s_max_use : int option;
+  s_expire          : (float * float ref) option;
+  s_f               : bool -> 'params -> 'result Lwt.t
+}
+
+type 'a to_and_of = {
+  of_string : string -> 'a;
+  to_string : 'a -> string
+}
