@@ -17,8 +17,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-(* ocamldoc/camlp4 hack ? : need an open first ? *)
-open Eliom_lib
+open Js_of_ocaml
 
 (** This module provides the creation of valid XML content, i.e. XML, SVG,
     and (X)HTML5.
@@ -51,7 +50,7 @@ module Svg : sig
 
        See {% <<a_api project="tyxml" | module Svg_sigs.T >> %} *)
   module F : sig
-    (** Cf. {% <<a_api project="tyxml" | module Html5_sigs.T >> %}. *)
+    (** Cf. {% <<a_api project="tyxml" | module Html_sigs.T >> %}. *)
     module Raw : Svg_sigs.Make(Xml).T
       with type +'a elt = 'a elt
        and type +'a attrib = 'a attrib
@@ -64,7 +63,7 @@ module Svg : sig
 
        See {% <<a_api project="tyxml" | module Svg_sigs.T >> %} *)
   module D : sig
-    (** Cf. {% <<a_api project="tyxml" | module Html5_sigs.T >> %}. *)
+    (** Cf. {% <<a_api project="tyxml" | module Html_sigs.T >> %}. *)
     module Raw : Svg_sigs.Make(Xml).T
       with type +'a elt = 'a elt
        and type +'a attrib = 'a attrib
@@ -74,7 +73,13 @@ module Svg : sig
 
   (** Creation of reactive content *)
   module R : sig
-    module Raw : Svg_sigs.MakeWrapped(Tyxml_js.Xml_wrap)(Xml).T
+
+    (** The function [node s] creates an SVG [elt] from a signal [s].
+        The resulting SVG [elt] can then be used like any other SVG
+        [elt]. *)
+    val node : 'a elt React.signal -> 'a elt
+
+    module Raw : Svg_sigs.Make(Eliom_content_core.Xml_wed).T
       with type +'a elt = 'a elt
        and type +'a attrib = 'a attrib
 
@@ -83,8 +88,8 @@ module Svg : sig
 
   (** Creation of content from client-side values. *)
   module C : sig
-    val node : ?init:'a D.elt -> 'a elt Eliom_pervasives.client_value -> 'a D.elt
-    val attr : ?init:'a attrib -> 'a attrib Eliom_pervasives.client_value -> 'a attrib
+    val node : ?init:'a D.elt -> 'a elt Eliom_client_value.t -> 'a D.elt
+    val attr : ?init:'a attrib -> 'a attrib Eliom_client_value.t -> 'a attrib
   end
 
 
@@ -107,6 +112,11 @@ module Svg : sig
         [create_named_elt ~id:(new_elt_id ()) elt]. *)
     val create_global_elt: 'a elt -> 'a elt
 
+    (** [create_request_elt ?reset elt] creates a referable copy of
+        [elt]. If [~reset = true] is provided (default: false), a new
+        ID is created even if [elt] has an ID already. *)
+    val create_request_elt: ?reset:bool -> 'a elt -> 'a elt
+
     (** [get_element id] returns
         the HTML element in the DOM with the given [id] if it exists. *)
     val get_element : 'a id -> 'a elt option
@@ -132,24 +142,24 @@ module Svg : sig
         the list of [e1] children. *)
     val appendChildren: ?before:'a elt -> 'b elt ->  'c elt list -> unit
 
-    (** [appendChildFirst p c] appends [c] as first child of [p] *)
-    val appendChildFirst: 'b elt ->  'c elt -> unit
+    (** [insertFirstChild p c] inserts [c] as first child of [p] *)
+    val insertFirstChild: 'b elt ->  'c elt -> unit
 
     (** [nth e n] returns the nth child of [e] (first is 0) *)
     val nth : 'a elt -> int -> 'b elt option
 
-    (** [childLength e] returns the number of chilren of [e] *)
+    (** [childLength e] returns the number of children of [e] *)
     val childLength : 'a elt -> int
 
     (** [removeChild e1 e2] removes for [e2] from the list of
-        [e1] children. *)
+        children of [e1]. *)
     val removeChild: 'a elt -> 'b elt -> unit
 
-    (** [replace e1 e2 e3] replaces for [e2] by [e3] in the
-        list of [e1] children. *)
+    (** [replace e1 e2 e3] replaces [e3] by [e2] in the
+        list of children of [e1]. *)
     val replaceChild: 'a elt -> 'b elt -> 'c elt -> unit
 
-    (** [removeChildren e1] removes [e1] children. *)
+    (** [removeChildren e1] removes all children of [e1]. *)
     val removeChildren: 'a elt -> unit
 
     (** [removeSelf e] removes element e from the DOM. *)
@@ -171,11 +181,11 @@ module Svg : sig
         if [elt] is not the first. *)
     val previousSibling: 'a elt -> 'b elt option
 
-    (** [appendBefore ~before elt] insert [elt] before [before]. *)
-    val appendBefore: before:'a elt -> 'b elt -> unit
+    (** [insertBefore ~before elt] insert [elt] before [before]. *)
+    val insertBefore: before:'a elt -> 'b elt -> unit
 
-    (** [appendAfter ~after elt] insert [elt] after [after]. *)
-    val appendAfter: after:'a elt -> 'b elt -> unit
+    (** [insertAfter ~after elt] insert [elt] after [after]. *)
+    val insertAfter: after:'a elt -> 'b elt -> unit
 
     (** [replaceSelf elt1 elt2] replaces [elt1] by [elt2]. *)
     val replaceSelf: 'a elt -> 'b elt -> unit
@@ -203,14 +213,19 @@ module Svg : sig
 
       (** see [appendChild] *)
       val appendChild: ?before:'a elt -> 'b Id.id -> 'c elt -> unit
+
       (** see [appendChildren] *)
       val appendChildren: ?before:'a elt -> 'b Id.id ->  'c elt list -> unit
+
       (** see [removeChild] *)
       val removeChild: 'a Id.id -> 'b elt -> unit
+
       (** see [replaceChild] *)
       val replaceChild: 'a Id.id -> 'b elt -> 'c elt -> unit
+
       (** see [removeChildren] *)
       val removeChildren: 'a Id.id -> unit
+
       (** see [replaceChildren] *)
       val replaceChildren: 'a Id.id -> 'b elt list -> unit
 
@@ -255,7 +270,7 @@ module Svg : sig
   end
 
   (** Conversion functions from DOM nodes ({% <<a_api project="js_of_ocaml"| type Dom_html.element>> %} {% <<a_api
-      project="js_of_ocaml"| type Js.t>> %}) to Eliom nodes ({% <<a_api | type Eliom_content.Html5.elt>> %}). *)
+      project="js_of_ocaml"| type Js.t>> %}) to Eliom nodes ({% <<a_api | type Eliom_content.Html.elt>> %}). *)
   module Of_dom : sig
     val of_element: Dom_html.element Js.t -> 'a elt
   end
@@ -263,7 +278,7 @@ module Svg : sig
 end
 
 (** Building valid (X)HTML5. *)
-module Html5 : sig
+module Html : sig
 
   (** See the Eliom manual for more information on {% <<a_manual
       chapter="clientserver-html" fragment="unique"| dom semantics vs. functional
@@ -273,131 +288,75 @@ module Html5 : sig
   type +'a elt
   type +'a attrib
   type uri = Xml.uri
+  type 'a form_param
 
   (** Creation of {e f}unctional HTML5 content (copy-able but not referable). *)
   module F : sig
     (** {2 Content creation}
-        See {% <<a_api project="tyxml" | module Html5_sigs.T >> %} *)
+        See {% <<a_api project="tyxml" | module Html_sigs.T >> %} *)
 
-    (** Cf. {% <<a_api project="tyxml" | module Html5_sigs.T >> %}. *)
-    module Raw : Html5_sigs.Make(Xml)(Svg.F.Raw).T
+    (** Cf. {% <<a_api project="tyxml" | module Html_sigs.T >> %}. *)
+    module Raw : Html_sigs.Make(Xml)(Svg.F.Raw).T
       with type +'a elt = 'a elt
        and type +'a attrib = 'a attrib
 
     include module type of Raw
 
-    (**/**)
-    type ('a, 'b, 'c) lazy_star =
-      ?a: (('a attrib) list) -> ('b elt) list Eliom_lazy.request -> 'c elt
+    include Eliom_content_sigs.LINKS_AND_FORMS
+      with type +'a elt := 'a elt
+       and type +'a attrib := 'a attrib
+       and type uri := uri
+       and type ('a, 'b, 'c) star := ('a, 'b, 'c) star
+       and type 'a form_param := 'a form_param
 
-    val lazy_form:
-      ([< Html5_types.form_attrib ], [< Html5_types.form_content_fun ], [> Html5_types.form ]) lazy_star
-
-    (**/**)
-
-    include "sigs/eliom_html5_forms2.mli"
-
-    (** Creates an untyped form. *)
-    val raw_form : ([< Html5_types.form_attrib ], [< Html5_types.form_content_fun ], [> Html5_types.form ]) star
-
-    (** This is an alias to
-        {% <<a_api|val Eliom_content.Html5.F.get_form>> %}
-        to avoid the untyped [Eliom_content_core.Html5.F.form]. *)
-    val form : ?absolute:bool -> ?absolute_path:bool -> ?https:bool -> ?a:Html5_types.form_attrib attrib list ->
-      service:('get, unit, [< get_service_kind ], _, _, [<suff ], 'gn, 'pn, [< registrable ], [< non_ocaml_service ]) service ->
-      ?hostname:string -> ?port:int -> ?fragment:string -> ?keep_nl_params:[ `All | `Persistent | `None ] ->
-      ?nl_params: Eliom_parameter.nl_params_set -> ?xhr:bool ->
-      ('gn -> Html5_types.form_content elt list) -> [> Html5_types.form ] elt
-
-    (** This is an alias to
-        {% <<a_api|val Eliom_content.Html5.F.string_input>> %}
-        to avoid the untyped [Eliom_content_core.Html5.F.input]. *)
-    val input : ?a:Html5_types.input_attrib attrib list -> input_type:[<
-	| `Url | `Tel | `Text | `Time | `Search | `Password | `Checkbox | `Range | `Radio | `Submit | `Reset | `Number | `Hidden
-	| `Month | `Week | `File | `Email | `Image | `Datetime_local | `Datetime | `Date | `Color | `Button]
-      -> ?name:[< string setoneradio ] param_name -> ?value:string -> unit -> [> Html5_types.input ] elt
-
-    (** This is an alias to
-        {% <<a_api|val Eliom_content.Html5.F.string_select>> %}
-        to avoid the untyped [Eliom_content_core.Html5.F.select]. *)
-    val select : ?a:Html5_types.select_attrib attrib list -> name:[`One of string ] param_name -> string select_opt -> string select_opt list -> [> Html5_types.select ] elt
   end
 
   (** Creation of HTML5 content with {e D}OM semantics (referable) *)
   module D : sig
     (** {2 Content creation}
-        See {% <<a_api project="tyxml" | module Html5_sigs.T >> %} *)
+        See {% <<a_api project="tyxml" | module Html_sigs.T >> %} *)
 
-    (** Cf. {% <<a_api project="tyxml" | module Html5_sigs.T >> %}. *)
-    module Raw : Html5_sigs.Make(Xml)(Svg.D.Raw).T
+    (** Cf. {% <<a_api project="tyxml" | module Html_sigs.T >> %}. *)
+    module Raw : Html_sigs.Make(Xml)(Svg.D.Raw).T
       with type +'a elt = 'a elt
        and type +'a attrib = 'a attrib
 
     include module type of Raw
 
-    (**/**)
-    type ('a, 'b, 'c) lazy_star =
-      ?a: (('a attrib) list) -> ('b elt) list Eliom_lazy.request -> 'c elt
+    include Eliom_content_sigs.LINKS_AND_FORMS
+      with type +'a elt := 'a elt
+       and type +'a attrib := 'a attrib
+       and type uri := uri
+       and type ('a, 'b, 'c) star := ('a, 'b, 'c) star
+       and type 'a form_param := 'a form_param
 
-    val lazy_form:
-      ([< Html5_types.form_attrib ], [< Html5_types.form_content_fun ], [> Html5_types.form ]) lazy_star
-    (**/**)
-
-
-    (** {2 Forms} *)
-    include "sigs/eliom_html5_forms2.mli"
-
-    (** Creates an untyped form. *)
-    val raw_form : ([< Html5_types.form_attrib ], [< Html5_types.form_content_fun ], [> Html5_types.form ]) star
-
-    (** This is an alias to
-        {% <<a_api|val Eliom_content.Html5.D.get_form>> %}
-        to avoid the untyped [Eliom_content_core.Html5.D.form]. *)
-    val form : ?absolute:bool -> ?absolute_path:bool -> ?https:bool -> ?a:Html5_types.form_attrib attrib list ->
-      service:('get, unit, [< get_service_kind ], _, _, [<suff ], 'gn, 'pn, [< registrable ], [< non_ocaml_service ]) service ->
-      ?hostname:string -> ?port:int -> ?fragment:string -> ?keep_nl_params:[ `All | `Persistent | `None ] ->
-      ?nl_params: Eliom_parameter.nl_params_set -> ?xhr:bool ->
-      ('gn -> Html5_types.form_content elt list) -> [> Html5_types.form ] elt
-
-    (** This is an alias to
-        {% <<a_api|val Eliom_content.Html5.D.string_input>> %}
-        to avoid the untyped [Eliom_content_core.Html5.D.input]. *)
-    val input : ?a:Html5_types.input_attrib attrib list -> input_type:[<
-	| `Url | `Tel | `Text | `Time | `Search | `Password | `Checkbox | `Range | `Radio | `Submit | `Reset | `Number | `Hidden
-	| `Month | `Week | `File | `Email | `Image | `Datetime_local | `Datetime | `Date | `Color | `Button]
-      -> ?name:[< string setoneradio ] param_name -> ?value:string -> unit -> [> Html5_types.input ] elt
-
-    (** This is an alias to
-        {% <<a_api|val Eliom_content.Html5.D.string_select>> %}
-        to avoid the untyped [Eliom_content_core.Html5.D.select]. *)
-    val select : ?a:Html5_types.select_attrib attrib list -> name:[`One of string ] param_name -> string select_opt -> string select_opt list -> [> Html5_types.select ] elt
   end
 
 
   (** Creation of HTML5 content from
-      {{: http://erratique.ch/software/react} React } signals.
+      {{:http://erratique.ch/software/react} React} signals.
       HTML5's trees are automatically updated whenever
       corresponding signals change.  *)
   module R : sig
     (** {2 Content creation}
-        See {% <<a_api project="tyxml" | module Html5_sigs.T >> %},
+        See {% <<a_api project="tyxml" | module Html_sigs.T >> %},
         If you want to create an untyped form,
-        you will have to use {% <<a_api|module Eliom_content.Html5.D.Raw>> %}
+        you will have to use {% <<a_api|module Eliom_content.Html.D.Raw>> %}
         otherwise, use the form module.
         For more information,
         see {{:http://ocsigen.org/howto/forms/}"how to make forms"} *)
 
     (** Function [node s] create an HTML5 [elt] from a signal [s].
-        The resulting HTML5 [elt] can then be used like anyother HTML5 [elt] *)
-    val node : 'a elt React.signal Eliom_pervasives.client_value -> 'a elt
+        The resulting HTML5 [elt] can then be used like any other HTML5 [elt] *)
+    val node : 'a elt React.signal Eliom_client_value.t -> 'a elt
 
     (** [filter_attrib att on] returns an attrib that
         behave like [att] when [on] is [true]
         and behave like if there was no attribute when [on] is [false] *)
     val filter_attrib : 'a attrib -> bool React.signal -> 'a attrib
 
-    (** Cf. {% <<a_api project="tyxml" | module Html5_sigs.T >> %}. *)
-    module Raw : Html5_sigs.MakeWrapped(Tyxml_js.Xml_wrap)(Xml)(Svg.R.Raw).T
+    (** Cf. {% <<a_api project="tyxml" | module Html_sigs.T >> %}. *)
+    module Raw : Html_sigs.Make(Eliom_content_core.Xml_wed)(Svg.R.Raw).T
       with type +'a elt = 'a elt
        and type +'a attrib = 'a attrib
 
@@ -415,8 +374,8 @@ module Html5 : sig
         (the [init] argument is ignored).
         See Eliom manual for more detail on
         {% <<a_manual chapter="clientserver-html" fragment="inject" | Dom & Client-values >>%}. *)
-    val node : ?init:'a D.elt -> 'a elt Eliom_pervasives.client_value -> 'a D.elt
-    val attr : ?init:'a attrib -> 'a attrib Eliom_pervasives.client_value -> 'a attrib
+    val node : ?init:'a D.elt -> 'a elt Eliom_client_value.t -> 'a D.elt
+    val attr : ?init:'a attrib -> 'a attrib Eliom_client_value.t -> 'a attrib
   end
 
 
@@ -440,6 +399,11 @@ module Html5 : sig
         [create_named_elt ~id:(new_elt_id ()) elt]. *)
     val create_global_elt: 'a elt -> 'a elt
 
+    (** [create_request_elt ?reset elt] creates a referable copy of
+        [elt]. If [~reset = true] is provided (default: false), a new
+        ID is created even if [elt] has an ID already. *)
+    val create_request_elt: ?reset:bool -> 'a elt -> 'a elt
+
     (** [get_element id] returns
         the HTML element in the DOM with the given [id] if it exists. *)
     val get_element : 'a id -> 'a elt option
@@ -452,7 +416,7 @@ module Html5 : sig
 
     (** Create a custom data field by providing string conversion functions.
         If the [default] is provided, calls to {% <<a_api project="eliom" subproject="client" |
-        val Eliom_content.Html5.Custom_data.get_dom>> %} return that instead of throwing an
+        val Eliom_content.Html.Custom_data.get_dom>> %} return that instead of throwing an
         exception [Not_found].  *)
     val create : name:string -> ?default:'a -> to_string:('a -> string) -> of_string:(string -> 'a) -> unit -> 'a t
 
@@ -461,7 +425,7 @@ module Html5 : sig
 
     (** [attrib my_data value ] creates a HTML5 attribute for the custom-data
         type [my_data] with value [value] for injecting it into an a HTML5 tree
-        ({% <<a_api | type Eliom_content.Html5.elt >> %}). *)
+        ({% <<a_api | type Eliom_content.Html.elt >> %}). *)
     val attrib : 'a t -> 'a -> [> | `User_data ] attrib
 
     val get_dom : Dom_html.element Js.t -> 'a t -> 'a
@@ -472,7 +436,7 @@ module Html5 : sig
   (** Conversion from HTML5 [elt]s to Javascript DOM elements ([<:] {% <<a_api
       project="js_of_ocaml"| class Dom_html.element >> %}).
       One conversion function per source type (stressed by the [of_] prefix). *)
-  module To_dom : Tyxml_cast_sigs.TO with type 'a elt = 'a elt
+  module To_dom : Js_of_ocaml_tyxml.Tyxml_cast_sigs.TO with type 'a elt = 'a elt
 
   (** DOM-like manipulation functions.
 
@@ -497,13 +461,13 @@ module Html5 : sig
         the list of [e1] children. *)
     val appendChildren: ?before:'a elt -> 'b elt ->  'c elt list -> unit
 
-    (** [appendChildFirst p c] appends [c] as first child of [p] *)
-    val appendChildFirst: 'b elt ->  'c elt -> unit
+    (** [insertFirstChild p c] inserts [c] as first child of [p] *)
+    val insertFirstChild: 'b elt ->  'c elt -> unit
 
     (** [nth e n] returns the nth child of [e] (first is 0) *)
     val nth : 'a elt -> int -> 'b elt option
 
-    (** [childLength e] returns the number of chilren of [e] *)
+    (** [childLength e] returns the number of children of [e] *)
     val childLength : 'a elt -> int
 
     (** The function [removeChild e1 e2] removes for [e2] from the list of
@@ -536,11 +500,11 @@ module Html5 : sig
         if [elt] is not the first. *)
     val previousSibling: 'a elt -> 'b elt option
 
-    (** [appendBefore ~before elt] insert [elt] before [before]. *)
-    val appendBefore: before:'a elt -> 'b elt -> unit
+    (** [insertBefore ~before elt] insert [elt] before [before]. *)
+    val insertBefore: before:'a elt -> 'b elt -> unit
 
-    (** [appendAfter ~after elt] insert [elt] after [after]. *)
-    val appendAfter: after:'a elt -> 'b elt -> unit
+    (** [insertAfter ~after elt] insert [elt] after [after]. *)
+    val insertAfter: after:'a elt -> 'b elt -> unit
 
     (** [replaceSelf elt1 elt2] replaces [elt1] by [elt2]. *)
     val replaceSelf: 'a elt -> 'b elt -> unit
@@ -571,14 +535,19 @@ module Html5 : sig
 
       (** see [appendChild] *)
       val appendChild: ?before:'a elt -> 'b Id.id -> 'c elt -> unit
+
       (** see [appendChildren] *)
       val appendChildren: ?before:'a elt -> 'b Id.id ->  'c elt list -> unit
+
       (** see [removeChild] *)
       val removeChild: 'a Id.id -> 'b elt -> unit
+
       (** see [replaceChild] *)
       val replaceChild: 'a Id.id -> 'b elt -> 'c elt -> unit
+
       (** see [removeChildren] *)
       val removeChildren: 'a Id.id -> unit
+
       (** see [replaceChildren] *)
       val replaceChildren: 'a Id.id -> 'b elt list -> unit
 
@@ -621,7 +590,7 @@ module Html5 : sig
     end
 
     module Elt : sig
-      val body : [`Body] elt
+      val body : unit -> [`Body] elt
     end
 
     module Ev : sig
@@ -859,7 +828,7 @@ module Html5 : sig
       val minHeightPx : 'a elt -> int -> unit
       val minWidth: 'a elt -> string -> unit
       val minWidthPx : 'a elt -> int -> unit
-      val opacity: 'a elt -> string option -> unit
+      val opacity: 'a elt -> string -> unit
       val outline: 'a elt -> string -> unit
       val outlineColor: 'a elt -> string -> unit
       val outlineOffset: 'a elt -> string -> unit
@@ -900,9 +869,29 @@ module Html5 : sig
   end
 
   (** Conversion functions from DOM nodes ({% <<a_api project="js_of_ocaml"| type Dom_html.element>> %} {% <<a_api
-      project="js_of_ocaml"| type Js.t>> %}) to Eliom nodes ({% <<a_api | type Eliom_content.Html5.elt>> %}). *)
-  module Of_dom : Tyxml_cast_sigs.OF with type 'a elt = 'a elt
+      project="js_of_ocaml"| type Js.t>> %}) to Eliom nodes ({% <<a_api | type Eliom_content.Html.elt>> %}). *)
+  module Of_dom : Js_of_ocaml_tyxml.Tyxml_cast_sigs.OF with type 'a elt = 'a elt
 
 end
 
 val force_link : unit
+
+
+(**/**)
+val set_client_fun :
+  ?app:string ->
+  service:('a, 'b, _, _, _, _, _, _, _, _, _) Eliom_service.t ->
+  ('a -> 'b -> Eliom_service.result Lwt.t) ->
+  unit
+
+(** With [set_form_error_handler f], [f] becomes the action to be
+    called when we are unable to call a client-side service due to
+    invalid form data.
+
+    If the handler returns [true], nothing happens.
+
+    If the handler returns [false], we proceed to call the server-side
+    service.
+
+    The default handler throws an exception (via [Lwt.fail_with]). *)
+val set_form_error_handler : (unit -> bool Lwt.t) -> unit
