@@ -18,33 +18,26 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-open Eliom_lib
-
-open Lwt
-open Eliom_state
-open Eliom_parameter
-open Eliom_service
-open Lazy
+module S = Eliom_service
+let (>>=) = Lwt.(>>=)
 
 let suffix_redir_uri_key = Polytables.make_key ()
 
-(****************************************************************************)
-
 type ('options,'page,'result) param =
     { send :
-	?options:'options ->
-	?charset:string ->
-	?code: int ->
-	?content_type:string ->
-	?headers: Http_headers.t ->
-	'page ->
-	Ocsigen_http_frame.result Lwt.t;
+        ?options:'options ->
+        ?charset:string ->
+        ?code: int ->
+        ?content_type:string ->
+        ?headers: Http_headers.t ->
+        'page ->
+        Ocsigen_http_frame.result Lwt.t;
 
-      send_appl_content : Eliom_service.send_appl_content;
+      send_appl_content : S.send_appl_content;
       (** Whether the service is capable to send application content when
-	  required. This field is usually [Eliom_service.XNever]. This
-	  value is recorded inside each service just after
-	  registration.  *)
+          required. This field is usually [Eliom_service.XNever]. This
+          value is recorded inside each service just after
+          registration.  *)
 
       result_of_http_result : Ocsigen_http_frame.result -> 'result; }
 
@@ -76,13 +69,13 @@ type ('options,'page,'result) param =
    point: we must wait for the page to be generated and then see if it
    is effectively application content. *)
 let check_before name service =
-  match Eliom_service.get_send_appl_content service
+  match S.send_appl_content service
   (* the appl name of the service *)
   with
-    | Eliom_service.XSame_appl (an, _)
-	when (an = name)
-	  -> (* Same appl, it is ok *) false
-    | Eliom_service.XAlways -> (* It is an action *) false
+    | S.XSame_appl (an, _)
+        when (an = name)
+          -> (* Same appl, it is ok *) false
+    | S.XAlways -> (* It is an action *) false
     | _ -> true
 
 (* This test check if there is a header set only by
@@ -105,22 +98,22 @@ let check_process_redir sp f param =
     if Eliom_request_info.expecting_process_page ()
     then
       match sp.Eliom_common.sp_client_appl_name with
-	(* the appl name as sent by browser *)
-	| None -> false (* should not happen *)
-	| Some anr -> f anr param
-	  (* the browser asked application eliom data
-	     (content only) for application anr *)
+        (* the appl name as sent by browser *)
+        | None -> false (* should not happen *)
+        | Some anr -> f anr param
+          (* the browser asked application eliom data
+             (content only) for application anr *)
     else false
   in
   if redir
   then
     let ri = Eliom_request_info.get_ri_sp sp in
-    raise_lwt
+    Lwt.fail
       (* we answer to the xhr
-	 by asking an HTTP redirection *)
+         by asking an HTTP redirection *)
       (Eliom_common.Eliom_do_half_xhr_redirection
-	 ("/"^
-             String.may_concat
+         ("/"^
+             Eliom_lib.String.may_concat
                   (Ocsigen_extensions.Ocsigen_request_info.original_full_path_string ri)
                   ~sep:"?"
                   (Eliom_parameter.construct_params_string
@@ -129,9 +122,9 @@ let check_process_redir sp f param =
                   )))
   (* We do not put hostname and port.
      It is ok with half or full xhr redirections. *)
-  (* If an action occured before,
+  (* If an action occurred before,
      it may have removed some get params form ri *)
-  else Lwt.return ()
+  else Lwt.return_unit
 
 let send_with_cookies
     sp
@@ -142,7 +135,7 @@ let send_with_cookies
     ?content_type
     ?headers
     content =
-  lwt result =
+  let%lwt result =
     pages.send
       ?options
       ?charset
@@ -151,8 +144,8 @@ let send_with_cookies
       ?headers
       content
   in
-  lwt () = check_process_redir sp check_after result in
-  lwt tab_cookies =
+  let%lwt () = check_process_redir sp check_after result in
+  let%lwt tab_cookies =
     Eliommod_cookies.compute_cookies_to_send
       sp.Eliom_common.sp_sitedata
       sp.Eliom_common.sp_tab_cookie_info
@@ -180,166 +173,151 @@ let register_aux pages
       ?content_type
       ?headers
       table
-      ~service
+      (type a)
+      ~(service : (_, _, _, a, _, _, _, _, _, _, _) S.t)
       ?(error_handler = fun l -> raise (Eliom_common.Eliom_Typing_Error l))
       page_generator =
-    Eliom_service.set_send_appl_content service (pages.send_appl_content);
+    S.set_send_appl_content service (pages.send_appl_content);
     begin
-      match get_info_ service with
-	| `Attached attser ->
-          let key_kind = get_or_post_ service in
-          let attserget = get_get_name_ attser in
-          let attserpost = get_post_name_ attser in
-          let suffix_with_redirect = get_redirect_suffix_ attser in
-          let priority = get_priority_ attser in
-          let sgpt = get_get_params_type_ service in
-          let sppt = get_post_params_type_ service in
+      match S.info service with
+        | S.Attached attser ->
+          let key_meth = S.which_meth_untyped service in
+          let attserget = S.get_name attser in
+          let attserpost = S.post_name attser in
+          let suffix_with_redirect = S.redirect_suffix attser in
+          let priority = S.priority attser in
+          let sgpt = S.get_params_type service
+          and sppt = S.post_params_type service in
+          let s_id =
+            if
+              attserget = Eliom_common.SAtt_no
+              || attserpost = Eliom_common.SAtt_no
+            then
+              Eliom_parameter.(
+                anonymise_params_type sgpt,
+                anonymise_params_type sppt
+              )
+            else
+              (0, 0)
+          and s_max_use = S.max_use service
+          and s_expire =
+            match S.timeout service with
+            | None -> None
+            | Some t -> Some (t, ref (t +. Unix.time ()))
+          in
           let f table ((attserget, attserpost) as attsernames) =
-            Eliommod_services.add_service
+            Eliom_route.add_service
               priority
               table
-              (get_sub_path_ attser)
+              (S.sub_path attser)
               {Eliom_common.key_state = attsernames;
-               Eliom_common.key_kind = key_kind}
-              ((if attserget = Eliom_common.SAtt_no
-                || attserpost = Eliom_common.SAtt_no
-                then (anonymise_params_type sgpt,
-                      anonymise_params_type sppt)
-                else (0, 0)),
-               ((match get_max_use_ service with
-                 | None -> None
-                 | Some i -> Some (ref i)),
-                (match get_timeout_ service with
-                  | None -> None
-                  | Some t -> Some (t, ref (t +. Unix.time ()))),
-                (fun nosuffixversion sp ->
+               Eliom_common.key_meth = (key_meth :> Eliom_common.meth)}
+              { s_id ; s_max_use ; s_expire ;
+                s_f = (fun nosuffixversion sp ->
                   Lwt.with_value Eliom_common.sp_key (Some sp)
                     (fun () ->
                       let ri = Eliom_request_info.get_ri_sp sp
                       and suff = Eliom_request_info.get_suffix_sp sp in
-                      (catch (fun () ->
-			reconstruct_params
-			  ~sp
-			  sgpt
-			  (Some (Lwt.return (force (Ocsigen_extensions.Ocsigen_request_info.get_params ri))))
-			  (Some (Lwt.return []))
-			  nosuffixversion
-			  suff
-			>>= fun g ->
-			let post_params =
-			  Eliom_request_info.get_post_params_sp sp
-			in
-			let files =
-			  Eliom_request_info.get_files_sp sp
-			in
-			reconstruct_params
-			  ~sp
-			  sppt
-			  post_params
-			  files
-			  false
-			  None
-			>>= fun p ->
-			(* GRGR TODO: avoid
-			   Eliom_uri.make_string_uri_. But we need to
-			   "downcast" the type of service to the
-			   correct "get service". *)
-			(if Eliom_request_info.get_http_method () =
+                      (Lwt.catch (fun () ->
+                         Eliom_parameter.reconstruct_params
+                          ~sp
+                          sgpt
+                          (Some (Lwt.return (Lazy.force (Ocsigen_extensions.Ocsigen_request_info.get_params ri))))
+                          (Some (Lwt.return_nil))
+                          nosuffixversion
+                          suff
+                        >>= fun g ->
+                        let post_params =
+                          Eliom_request_info.get_post_params_sp sp
+                        in
+                        let files =
+                          Eliom_request_info.get_files_sp sp
+                        in
+                        Eliom_parameter.reconstruct_params
+                          ~sp
+                          sppt
+                          post_params
+                          files
+                          false
+                          None
+                        >>= fun p ->
+                        (* GRGR TODO: avoid
+                           Eliom_uri.make_string_uri_. But we need to
+                           "downcast" the type of service to the
+                           correct "get service". *)
+                        (if Eliom_request_info.get_http_method () =
                            Ocsigen_http_frame.Http_header.GET
                          && nosuffixversion && suffix_with_redirect
-			 then
-			    (* it is a suffix service in version
-			       without suffix. We redirect. *)
-			    if not (Eliom_request_info.expecting_process_page ())
-			    then
-			      let redir_uri =
-			        Eliom_uri.make_string_uri_
-				  ~absolute:true
-				  ~service:
-				  (service :
-				     ('a, 'b, [< service_method],[<attached],[< Eliom_service.internal_service_kind ],
-				      [< Eliom_service.suff ], 'c, 'd, [ `Registrable ],
-				      'return) Eliom_service.service :>
-				     ('a, 'b, [< service_method],[<attached],Eliom_service.service_kind,
-				      [< Eliom_service.suff ], 'c, 'd,
-				      [< Eliom_service.registrable ], 'return)
-				     Eliom_service.service)
-				  g
-			      in
-			      Lwt.fail
-                                (Eliom_common.Eliom_do_redirection redir_uri)
-			    else begin
-			    (* It is an internal application form.
-			       We don't redirect but we set this
-			       special information for url to be displayed
-			       by the browser
-			       (see Eliom_request_info.rebuild_uri_without_iternal_form_info_)
-			    *)
-			      let redir_uri =
-			        Eliom_uri.make_string_uri_
-				  ~absolute:false
-				  ~absolute_path:true
-				  ~service:
-				  (service :
-				     ('a, 'b, [< service_method],[<attached],[< Eliom_service.internal_service_kind ],
-				      [< Eliom_service.suff ], 'c, 'd, [ `Registrable ],
-				      'return) Eliom_service.service :>
-				     ('a, 'b, [< service_method],[<attached],Eliom_service.service_kind,
-				      [< Eliom_service.suff ], 'c, 'd,
-				      [< Eliom_service.registrable ], 'return)
-				     Eliom_service.service)
-				  g
-			      in
+                         then
+                            (* it is a suffix service in version
+                               without suffix. We redirect. *)
+                            if not (Eliom_request_info.expecting_process_page ())
+                            then
                               let redir_uri =
-                                if String.length redir_uri > 0
-                                then String.sub redir_uri 1
-                                  (String.length redir_uri - 1)
-                                else redir_uri
+                                Eliom_uri.make_string_uri_
+                                  ~absolute:true
+                                  ~service:
+                                  (service :
+                                     ('a, 'b, _, _, _,
+                                      S.non_ext, S.reg, _,
+                                      'c, 'd, 'return)
+                                       S.t :>
+                                     ('a, 'b, _, _, _, _, _, _,
+                                      'c, 'd, 'return)
+                                     S.t)
+                                  g
                               in
-			      let rc = Eliom_request_info.get_request_cache_sp sp in
-			      Polytables.set ~table:rc ~key:suffix_redir_uri_key ~value:redir_uri;
-			      Lwt.return ()
-			    end
-			 else Lwt.return ())
-			>>= fun () ->
+                              Lwt.fail
+                                (Eliom_common.Eliom_do_redirection redir_uri)
+                            else begin
+                            (* It is an internal application form.
+                               We don't redirect but we set this
+                               special information for url to be displayed
+                               by the browser
+                               (see Eliom_request_info.rebuild_uri_without_iternal_form_info_)
+                            *)
+                              let redir_uri =
+                                Eliom_uri.make_string_uri_ ~service g in
+                              let rc = Eliom_request_info.get_request_cache_sp sp in
+                              Polytables.set ~table:rc ~key:suffix_redir_uri_key ~value:redir_uri;
+                              Lwt.return_unit
+                            end
+                         else Lwt.return_unit)
+                        >>= fun () ->
                         check_process_redir sp check_before service >>= fun () ->
                         page_generator g p)
                          (function
                            | Eliom_common.Eliom_Typing_Error l ->
                              error_handler l
-                           | e -> fail e)
-                        >>= fun content ->
-		       send_with_cookies sp pages
+                           | e -> Lwt.fail e)
+                       >>= fun content ->
+                       send_with_cookies sp pages
                          ?options
                          ?charset
                          ?code
                          ?content_type
                          ?headers
-                         content)))))
+                         content)))}
           in
-          (match (key_kind, attserget, attserpost) with
-            | (Ocsigen_http_frame.Http_header.POST, _,
-               Eliom_common.SAtt_csrf_safe (id, scope, secure_session))
-            | (Ocsigen_http_frame.Http_header.PUT, _,
-               Eliom_common.SAtt_csrf_safe (id, scope, secure_session))
-            | (Ocsigen_http_frame.Http_header.DELETE, _,
-               Eliom_common.SAtt_csrf_safe (id, scope, secure_session)) ->
+          (match (key_meth, attserget, attserpost) with
+            | (`Post | `Put | `Delete), _,
+               Eliom_common.SAtt_csrf_safe (id, scope, secure_session) ->
               let tablereg, forsession =
                 match table with
-                  | Left globtbl -> globtbl, false
-                  | Right (sp, ct, sec) ->
+                  | Eliom_lib.Left globtbl -> globtbl, false
+                  | Eliom_lib.Right (sp, ct, sec) ->
                     if secure_session <> sec || scope <> ct
                     then raise
-                      Wrong_session_table_for_CSRF_safe_coservice;
+                      S.Wrong_session_table_for_CSRF_safe_coservice;
                     !(Eliom_state.get_session_service_table
                         ?secure:secure_session ~scope ~sp ()),
                       true
               in
-              Eliom_service.set_delayed_post_registration_function
+              S.set_delayed_post_registration_function
                 tablereg
                 id
                 (fun ~sp attserget ->
-                  let n = Eliom_service.new_state () in
+                  let n = S.new_state () in
                   let attserpost = Eliom_common.SAtt_anon n in
                   let table =
                     if forsession
@@ -354,24 +332,24 @@ let register_aux pages
                   in
                   f table (attserget, attserpost);
                   n)
-            | (Ocsigen_http_frame.Http_header.GET,
-               Eliom_common.SAtt_csrf_safe (id, scope, secure_session),
-               _) ->
+            | `Get,
+              Eliom_common.SAtt_csrf_safe (id, scope, secure_session),
+              _ ->
               let tablereg, forsession =
                 match table with
                   | Left globtbl -> globtbl, false
                   | Right (sp, ct, sec) ->
                     if secure_session <> sec || ct <> scope
                     then raise
-                      Wrong_session_table_for_CSRF_safe_coservice;
+                      S.Wrong_session_table_for_CSRF_safe_coservice;
                     !(Eliom_state.get_session_service_table
                         ?secure:secure_session ~scope ~sp ()), true
               in
-              Eliom_service.set_delayed_get_or_na_registration_function
+              S.set_delayed_get_or_na_registration_function
                 tablereg
                 id
                 (fun ~sp ->
-                  let n = Eliom_service.new_state () in
+                  let n = S.new_state () in
                   let attserget = Eliom_common.SAtt_anon n in
                   let table =
                     if forsession
@@ -394,53 +372,53 @@ let register_aux pages
                         ?secure:secure_session ~scope ~sp ())
               in
               f tablereg (attserget, attserpost))
-	| `Nonattached naser ->
-          let na_name = get_na_name_ naser in
+        | S.Nonattached naser ->
+          let na_name = S.na_name naser in
           let f table na_name =
-            Eliommod_naservices.add_naservice
+            Eliom_route.add_naservice
               table
               na_name
-              ((match get_max_use_ service with
+              ((match S.max_use service with
                 | None -> None
                 | Some i -> Some (ref i)),
-               (match get_timeout_ service with
+               (match S.timeout service with
                  | None -> None
                  | Some t -> Some (t, ref (t +. Unix.time ()))),
                (fun sp ->
                  Lwt.with_value Eliom_common.sp_key (Some sp)
                    (fun () ->
                      let ri = Eliom_request_info.get_ri_sp sp in
-                     catch
+                     Lwt.catch
                        (fun () ->
-                         reconstruct_params
+                         Eliom_parameter.reconstruct_params
                            ~sp
-                           (get_get_params_type_ service)
+                           (S.get_params_type service)
                            (Some (Lwt.return
-                              (force (Ocsigen_extensions
+                              (Lazy.force (Ocsigen_extensions
                                       .Ocsigen_request_info.get_params ri))))
-                           (Some (Lwt.return []))
+                           (Some (Lwt.return_nil))
                            false
                            None
                          >>= fun g ->
                          let post_params =
-			   Eliom_request_info.get_post_params_sp sp
+                           Eliom_request_info.get_post_params_sp sp
                          in
                          let files = Eliom_request_info.get_files_sp sp in
-                         reconstruct_params
-			   ~sp
-			   (get_post_params_type_ service)
-			   post_params
-			   files
-			   false
-			   None
+                         Eliom_parameter.reconstruct_params
+                           ~sp
+                           (S.post_params_type service)
+                           post_params
+                           files
+                           false
+                           None
                          >>= fun p ->
                          check_process_redir sp check_before service >>= fun () ->
-			 page_generator g p)
+                         page_generator g p)
                        (function
                          | Eliom_common.Eliom_Typing_Error l ->
                            error_handler l
-                         | e -> fail e) >>= fun content ->
-		     send_with_cookies sp pages
+                         | e -> Lwt.fail e) >>= fun content ->
+                     send_with_cookies sp pages
                        ?options
                        ?charset
                        ?code
@@ -458,15 +436,15 @@ let register_aux pages
                   | Right (sp, ct, sec) ->
                     if secure_session <> sec || ct <> scope
                     then raise
-                      Wrong_session_table_for_CSRF_safe_coservice;
+                      S.Wrong_session_table_for_CSRF_safe_coservice;
                     !(Eliom_state.get_session_service_table
                         ?secure:secure_session ~scope ~sp ()), true
               in
-              set_delayed_get_or_na_registration_function
+              S.set_delayed_get_or_na_registration_function
                 tablereg
                 id
                 (fun ~sp ->
-                  let n = Eliom_service.new_state () in
+                  let n = S.new_state () in
                   let na_name = Eliom_common.SNa_get' n in
                   let table =
                     if forsession
@@ -488,15 +466,15 @@ let register_aux pages
                   | Right (sp, ct, sec) ->
                     if secure_session <> sec || ct <> scope
                     then raise
-                      Wrong_session_table_for_CSRF_safe_coservice;
+                      S.Wrong_session_table_for_CSRF_safe_coservice;
                     !(Eliom_state.get_session_service_table
                         ?secure:secure_session ~scope ~sp ()), true
               in
-              set_delayed_get_or_na_registration_function
+              S.set_delayed_get_or_na_registration_function
                 tablereg
                 id
                 (fun ~sp ->
-                  let n = Eliom_service.new_state () in
+                  let n = S.new_state () in
                   let na_name = Eliom_common.SNa_post' n in
                   let table =
                     if forsession
@@ -528,7 +506,7 @@ let send pages
     ?content_type
     ?headers
     content =
-  lwt result =
+  let%lwt result =
     pages.send
       ?options
       ?charset
@@ -540,6 +518,7 @@ let send pages
   Lwt.return (pages.result_of_http_result result)
 
 let register pages
+    ?app
     ?scope
     ?options
     ?charset
@@ -547,7 +526,9 @@ let register pages
     ?content_type
     ?headers
     ?secure_session
-    ~service
+    (type a)
+    ~(service :
+        (_, _, _, a, _, _, S.reg, _, _, _, _) S.t)
     ?error_handler
     page_gen =
   let sp = Eliom_common.get_sp_option () in
@@ -557,13 +538,13 @@ let register pages
       (match Eliom_common.global_register_allowed () with
         | Some get_current_sitedata ->
           let sitedata = get_current_sitedata () in
-          (match get_info_ service with
-            | `Attached attser ->
+          (match S.info service with
+            | S.Attached attser ->
               Eliom_common.remove_unregistered
-                sitedata (get_sub_path_ attser)
-            | `Nonattached naser ->
+                sitedata (S.sub_path attser)
+            | S.Nonattached naser ->
               Eliom_common.remove_unregistered_na
-                sitedata (get_na_name_ naser));
+                sitedata (S.na_name naser));
           register_aux pages
             ?options
             ?charset
@@ -584,7 +565,7 @@ let register pages
         ?content_type
         ?headers
         ?error_handler
-        (Left (get_global_table ()))
+        (Eliom_lib.Left (Eliom_state.get_global_table ()))
         ~service
         page_gen
     | _, None ->
@@ -607,9 +588,9 @@ let register pages
      like "let rec" for service...
   *)
 
-
-let register_service pages
+let create pages
     ?scope
+    ?app
     ?options
     ?charset
     ?code
@@ -617,424 +598,124 @@ let register_service pages
     ?headers
     ?secure_session
     ?https
-    ?priority
+    ?name
+    ?csrf_safe
+    ?csrf_scope
+    ?csrf_secure
+    ?max_use
+    ?timeout
+    ~meth
     ~path
+    ?error_handler
+    page =
+  let service =
+    S.create_unsafe
+      ?name
+      ?csrf_safe
+      ?csrf_scope:(csrf_scope :> Eliom_common.user_scope option)
+      ?csrf_secure
+      ?max_use ?timeout ?https ~meth ~path ()
+  in
+  register pages
+    ?scope
+    ?app
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ~service ?error_handler page;
+  service
+
+let create_attached_get pages
+    ?scope
+    ?app
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ?https
+    ?name
+    ?csrf_safe
+    ?csrf_scope
+    ?csrf_secure
+    ?max_use
+    ?timeout
+    ~fallback
     ~get_params
     ?error_handler
     page =
-  let u = Unsafe.service ?https ?priority ~path ~get_params () in
+  let service =
+    S.create_attached_get_unsafe
+      ?name
+      ?csrf_safe
+      ?csrf_scope:(csrf_scope :> Eliom_common.user_scope option)
+      ?csrf_secure
+      ?max_use ?timeout ?https ~fallback ~get_params ()
+  in
   register pages
     ?scope
+    ?app
     ?options
     ?charset
     ?code
     ?content_type
     ?headers
     ?secure_session
-    ~service:u ?error_handler page;
-  u
+    ~service ?error_handler page;
+  service
 
-let register_coservice pages
+let create_attached_post pages
     ?scope
+    ?app
     ?options
     ?charset
     ?code
     ?content_type
     ?headers
     ?secure_session
+    ?https
     ?name
     ?csrf_safe
     ?csrf_scope
     ?csrf_secure
     ?max_use
     ?timeout
-    ?https
-    ~(fallback: (unit, unit, [< Eliom_service.service_method > `Get ],
-                     [> Eliom_service.attached_kind ],
-                     [< Eliom_service.service_kind > `Service ],
-                     [ `WithoutSuffix ], unit, unit,
-                     [< Eliom_service.registrable ], 'returnT)
-                    Eliom_service.service)
-    ~get_params
+    ~fallback
+    ~post_params
     ?error_handler
     page =
-  let u =
-    Unsafe.coservice ?name
-      ?csrf_safe
-      ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
-      ?csrf_secure
-      ?max_use ?timeout ?https
-      ~fallback ~get_params ()
-  in
-  register pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ~service:u ?error_handler page;
-  u
-
-let register_coservice' pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ?name
-    ?csrf_safe
-    ?csrf_scope
-    ?csrf_secure
-    ?max_use
-    ?timeout
-    ?https
-    ~get_params
-    ?error_handler
-    page =
-  let u =
-    Unsafe.coservice'
+  let service =
+    S.create_attached_post_unsafe
       ?name
       ?csrf_safe
-      ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
+      ?csrf_scope:(csrf_scope :> Eliom_common.user_scope option)
       ?csrf_secure
-      ?max_use ?timeout ?https ~get_params ()
+      ?max_use ?timeout ?https ~fallback ~post_params ()
   in
   register pages
     ?scope
+    ?app
     ?options
     ?charset
     ?code
     ?content_type
     ?headers
     ?secure_session
-    ~service:u ?error_handler page;
-  u
+    ~service ?error_handler page;
+  service
 
-let register_post_service pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ?https
-    ?priority
-    ~fallback
-    ~post_params
-    ?error_handler
-    page_gen =
-  let u = Unsafe.post_service ?https ?priority
-    ~fallback:fallback ~post_params:post_params ()
-  in
-  register pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ~service:u ?error_handler page_gen;
-  u
-
-let register_post_coservice pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ?name
-    ?csrf_safe
-    ?csrf_scope
-    ?csrf_secure
-    ?max_use
-    ?timeout
-    ?https
-    ~fallback
-    ~post_params
-    ?error_handler
-    page_gen =
-  let u =
-    Unsafe.post_coservice ?name
-      ?csrf_safe
-      ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
-      ?csrf_secure
-      ?max_use ?timeout ?https
-      ~fallback ~post_params () in
-  register pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ~service:u ?error_handler page_gen;
-  u
-
-let register_post_coservice' pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ?name
-    ?csrf_safe
-    ?csrf_scope
-    ?csrf_secure
-    ?max_use
-    ?timeout
-    ?keep_get_na_params
-    ?https
-    ~post_params
-    ?error_handler
-    page_gen =
-  let u =
-    Unsafe.post_coservice'
-      ?name
-      ?csrf_safe
-      ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
-      ?csrf_secure
-      ?keep_get_na_params
-      ?max_use
-      ?timeout
-      ?https
-      ~post_params ()
-  in
-  register pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ~service:u ?error_handler page_gen;
-  u
-
-let register_put_service pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ?https
-    ?priority
-    ~path
-    ~get_params
-    ?error_handler
-    page_gen =
-  let u = Unsafe.put_service ?https ?priority ~path ~get_params () in
-  register pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ~service:u ?error_handler page_gen;
-  u
-
-let register_put_coservice pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ?name
-    ?csrf_safe
-    ?csrf_scope
-    ?csrf_secure
-    ?max_use
-    ?timeout
-    ?https
-    ~fallback
-    ~get_params
-    ?error_handler
-    page_gen =
-  let u =
-    Unsafe.put_coservice ?name
-      ?csrf_safe
-      ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
-      ?csrf_secure
-      ?max_use ?timeout ?https
-      ~fallback ~get_params ()
-  in
-  register pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ~service:u ?error_handler page_gen;
-  u
-
-let register_put_coservice' pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ?name
-    ?csrf_safe
-    ?csrf_scope
-    ?csrf_secure
-    ?max_use
-    ?timeout
-    ?https
-    ~get_params
-    ?error_handler
-    page_gen =
-  let u =
-    Unsafe.put_coservice'
-      ?name
-      ?csrf_safe
-      ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
-      ?csrf_secure
-      ?max_use
-      ?timeout
-      ?https
-      ~get_params ()
-  in
-  register pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ~service:u ?error_handler page_gen;
-  u
-
-let register_delete_service pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ?https
-    ?priority
-    ~path
-    ~get_params
-    ?error_handler
-    page_gen =
-  let u = Unsafe.delete_service ?https ?priority ~path ~get_params () in
-  register pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ~service:u ?error_handler page_gen;
-  u
-
-let register_delete_coservice pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ?name
-    ?csrf_safe
-    ?csrf_scope
-    ?csrf_secure
-    ?max_use
-    ?timeout
-    ?https
-    ~fallback
-    ~get_params
-    ?error_handler
-    page_gen =
-  let u =
-    Unsafe.delete_coservice ?name
-      ?csrf_safe
-      ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
-      ?csrf_secure
-      ?max_use ?timeout ?https
-      ~fallback ~get_params () in
-  register pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ~service:u ?error_handler page_gen;
-  u
-
-let register_delete_coservice' pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ?name
-    ?csrf_safe
-    ?csrf_scope
-    ?csrf_secure
-    ?max_use
-    ?timeout
-    ?https
-    ~get_params
-    ?error_handler
-    page_gen =
-  let u =
-    Unsafe.delete_coservice'
-      ?name
-      ?csrf_safe
-      ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
-      ?csrf_secure
-      ?max_use
-      ?timeout
-      ?https
-      ~get_params ()
-  in
-  register pages
-    ?scope
-    ?options
-    ?charset
-    ?code
-    ?content_type
-    ?headers
-    ?secure_session
-    ~service:u ?error_handler page_gen;
-  u
-
-module type REG_PARAM = "sigs/eliom_reg_param.mli"
-
-module MakeRegister(Pages : REG_PARAM) = struct
+module Make
+    (Pages : Eliom_registration_sigs.PARAM
+     with type frame := Ocsigen_http_frame.result) =
+struct
 
   type page = Pages.page
   type options = Pages.options
-  type return = Pages.return
+  type return = Eliom_service.non_ocaml
   type result = Pages.result
 
   let pages =
@@ -1043,58 +724,38 @@ module MakeRegister(Pages : REG_PARAM) = struct
       result_of_http_result = Pages.result_of_http_result; }
 
   let send ?options = send pages ?options
-  let register ?scope = register pages ?scope
-  let register_service ?scope = register_service pages ?scope
-  let register_coservice ?scope = register_coservice pages ?scope
-  let register_coservice' ?scope = register_coservice' pages ?scope
-  let register_post_service ?scope = register_post_service pages ?scope
-  let register_post_coservice ?scope = register_post_coservice pages ?scope
-  let register_post_coservice' ?scope = register_post_coservice' pages ?scope
-  let register_put_service ?scope = register_put_service pages ?scope
-  let register_put_coservice ?scope = register_put_coservice pages ?scope
-  let register_put_coservice' ?scope = register_put_coservice' pages ?scope
-  let register_delete_service ?scope = register_delete_service pages ?scope
-  let register_delete_coservice ?scope = register_delete_coservice pages ?scope
-  let register_delete_coservice' ?scope = register_delete_coservice' pages ?scope
+
+  let register ?app = register pages ?app
+
+  let create ?app = create pages ?app
+
+  let create_attached_get ?app = create_attached_get pages ?app
+
+  let create_attached_post ?app = create_attached_post pages ?app
 
 end
 
-module type REG_PARAM_ALPHA_RETURN =
-sig
-  type ('a, 'b) page
-  type 'a return
-  type 'a result
-  include "sigs/eliom_reg_param.mli"
-    subst type page := ('a, 'b) page
-      and type return := 'b return
-      and type result := 'a result
-end
+module Make_poly
+    (Pages : Eliom_registration_sigs.PARAM_POLY
+     with type frame := Ocsigen_http_frame.result) =
+struct
 
-module MakeRegister_AlphaReturn(Pages : REG_PARAM_ALPHA_RETURN) = struct
-
-  type ('a, 'b) page = ('a, 'b) Pages.page
+  type 'a page = 'a Pages.page
   type options = Pages.options
-  type 'b return = 'b Pages.return
-  type 'a result = 'a Pages.result
+  type 'a return = 'a Pages.return
 
-  let pages =
-    { send = Pages.send;
-      send_appl_content = Pages.send_appl_content;
-      result_of_http_result = Pages.result_of_http_result; }
+  let pages = {
+    send                  = Pages.send;
+    send_appl_content     = Pages.send_appl_content;
+    result_of_http_result = (fun x -> x)
+  }
 
-  let send ?options = send pages ?options
-  let register ?scope = register pages ?scope
-  let register_service ?scope = register_service pages ?scope
-  let register_coservice ?scope = register_coservice pages ?scope
-  let register_coservice' ?scope = register_coservice' pages ?scope
-  let register_post_service ?scope = register_post_service pages ?scope
-  let register_post_coservice ?scope = register_post_coservice pages ?scope
-  let register_post_coservice' ?scope = register_post_coservice' pages ?scope
-  let register_put_service ?scope = register_put_service pages ?scope
-  let register_put_coservice ?scope = register_put_coservice pages ?scope
-  let register_put_coservice' ?scope = register_put_coservice' pages ?scope
-  let register_delete_service ?scope = register_delete_service pages ?scope
-  let register_delete_coservice ?scope = register_delete_coservice pages ?scope
-  let register_delete_coservice' ?scope = register_delete_coservice' pages ?scope
+  let register ?app = register pages ?app
+
+  let create ?app = create pages ?app
+
+  let create_attached_get ?app = create_attached_get pages ?app
+
+  let create_attached_post ?app = create_attached_post pages ?app
 
 end
