@@ -20,6 +20,11 @@
 
 open Eliom_lib
 
+let section = Lwt_log.Section.make "eliom:parameter"
+
+type params = (string * Eliommod_parameters.param) list
+type params' = (string * string) list
+
 (** Type of names in a form *)
 type +'a param_name = string
 
@@ -39,7 +44,7 @@ type 'a setoneradio = [ `Set of 'a | `One of 'a | `Radio of 'a ]
 type 'a oneradio = [ `One of 'a | `Radio of 'a ]
 type 'a setone = [ `Set of 'a | `One of 'a ]
 
-type 'a to_and_from = {
+type 'a to_and_of = 'a Eliom_common.to_and_of = {
   of_string : string -> 'a;
   to_string : 'a -> string
 }
@@ -77,7 +82,7 @@ let to_from_of_atom x =
 
 type 'a filter = ('a -> unit) option
 
-type raw = (((string * string) * (string * string) list) option * string Ocsigen_stream.t option)
+type raw = Eliom_request_info.raw_post_data
 type 'a ocaml = string (* marshaled values of type 'a *)
 
 type suff = [ `WithoutSuffix | `WithSuffix | `Endsuffix ]
@@ -91,16 +96,19 @@ type (_,_) params_type_ =
 
   | TAtom : (string * 'a atom) -> ('a, [`One of 'a] param_name) params_type_
   | TCoord : string -> (coordinates,[`One of coordinates] param_name) params_type_
-  | TCoordv : (string * ('a,'an) params_type_) -> ('a * coordinates,[`One of 'a * coordinates] param_name) params_type_
   | TFile : string -> (file_info , [ `One of file_info ] param_name) params_type_
 
-  | TUserType : (string * 'a to_and_from) -> ('a, [ `One of 'a ] param_name) params_type_
+  | TUserType :
+      (string * 'a Eliom_common.To_and_of_shared.t) ->
+    ('a, [ `One of 'a ] param_name) params_type_
   | TTypeFilter : (('a,'an) params_type_ * 'a filter) -> ('a,'an) params_type_
   (* remove TCoord *)
 
   | TESuffix :  string -> (string list , [ `One of string list ] param_name) params_type_
   | TESuffixs : string -> (string, [ `One of string ] param_name) params_type_
-  | TESuffixu : (string * 'a to_and_from) -> ('a, [ `One of 'a ] param_name) params_type_
+  | TESuffixu :
+      (string * 'a Eliom_common.To_and_of_shared.t) ->
+    ('a, [ `One of 'a ] param_name) params_type_
   | TSuffix : (bool * ('s, 'sn) params_type_) -> ('s , 'sn) params_type_
   (* bool = redirect the version without suffix to the suffix version *)
 
@@ -141,18 +149,9 @@ let file (n : string) = TFile n
 let unit = TUnit
 
 let coordinates n = TCoord n
-let string_coordinates n = TCoordv (n,string n)
-let int_coordinates n =    TCoordv (n,int n)
-let int32_coordinates n =  TCoordv (n,int32 n)
-let int64_coordinates n =  TCoordv (n,int64 n)
-let float_coordinates n =  TCoordv (n,float n)
+
 let type_checker check t = TTypeFilter (t, Some check)
 
-let user_type ~(of_string : string -> 'a) ~(to_string : 'a -> string) (n : string) =
-  TUserType (n,{of_string;to_string})
-
-let user_type_coordinates ~of_string ~to_string n = TCoordv (n,user_type ~of_string ~to_string n)
-
 let sum t1 t2 = TSum (t1, t2)
 
 let prod t1 t2 = TProd (t1,t2)
@@ -177,9 +176,6 @@ let all_suffix (n : string) = TESuffix n
 
 let all_suffix_string (n : string) = TESuffixs n
 
-let all_suffix_user ~(of_string : string -> 'a) ~(to_string : 'a -> string) (n : string) =
-  TESuffixu (n, {of_string; to_string})
-
 let suffix ?(redirect_if_not_suffix = true) s = TSuffix (redirect_if_not_suffix, s)
 
 let suffix_prod ?(redirect_if_not_suffix = true)
@@ -190,10 +186,6 @@ let suffix_prod ?(redirect_if_not_suffix = true)
 
 let ocaml (n : string) typ = TJson (n, Some typ)
 
-type raw_post_data =
-  ((string * string) * (string * string) list) option *
-      string Ocsigen_stream.t option
-
 let raw_post_data = TRaw_post_data
 
 (******************************************************************)
@@ -208,10 +200,6 @@ let rec make_suffix : type a c. (a,'b,c) params_type -> a -> string list = fun t
   | TAtom (_,a) -> [string_of_atom a params]
   | TCoord n -> (make_suffix (TAtom ("",TInt)) (params.abscissa))@
                 (make_suffix (TAtom ("",TInt)) (params.ordinate))
-  | TCoordv (_,t) ->
-    (make_suffix t (fst params)) @
-    (make_suffix (TAtom("",TInt)) ((snd params).abscissa)) @
-    (make_suffix (TAtom("",TInt)) ((snd params).ordinate))
   | TUnit -> [""]
   | TConst v -> [v]
   | TOption (t,_) ->
@@ -230,7 +218,8 @@ let rec make_suffix : type a c. (a,'b,c) params_type -> a -> string list = fun t
       | a::l ->
         (make_suffix t a)@
         (make_suffix typ l))
-  | TUserType (_, to_and_from) -> [to_and_from.to_string params]
+  | TUserType (_, tao) ->
+    [ Eliom_common.To_and_of_shared.to_string tao params ]
   | TTypeFilter (t, check) -> make_suffix t params
   | TSum (t1, t2) ->
     (match params with
@@ -239,7 +228,8 @@ let rec make_suffix : type a c. (a,'b,c) params_type -> a -> string list = fun t
   | TESuffixs _ -> [params]
   (* | TAny ->       (match params with [] -> [""] | p -> p) *)
   | TESuffix _ -> (match params with [] -> [""] | p -> p)
-  | TESuffixu (_, to_and_from) -> [to_and_from.to_string params]
+  | TESuffixu (_, tao) ->
+    [ Eliom_common.To_and_of_shared.to_string tao params ]
   | TJson (_, typ) -> (* server or client side *)
     [ to_json ?typ params ]
   | _ -> raise (Eliom_Internal_Error "Bad parameter type in suffix")
@@ -273,21 +263,26 @@ let rec aux : type a c. (a,'b,c) params_type -> string list option -> 'y -> a ->
         (fun (psuff, nlp, l) v -> aux t psuff nlp v pref suff l)
         (psuff, nlp, l)
         params
+    | TAtom (name,TBool) ->
+      psuff, nlp,
+      if params then
+        (pref^name^suff, insert_string "on") :: l
+      else
+        l
     | TAtom (name,a) -> psuff,nlp,((pref^name^suff,insert_string (string_of_atom a params))::l )
     | TCoord name ->
       psuff, nlp,
       let coord = params in
       ((pref^name^suff^".x"), insert_string (string_of_int coord.abscissa))::
       ((pref^name^suff^".y"), insert_string (string_of_int coord.ordinate))::l
-    | TCoordv (name,t) -> aux (TProd (t, TCoord name)) psuff nlp params pref suff l
     | TSum (t1, t2) -> (match params with
         | Inj1 v -> aux t1 psuff nlp v pref suff l
         | Inj2 v -> aux t2 psuff nlp v pref suff l)
     | TFile name -> psuff, nlp, ((pref^name^suff),insert_file params)::l
-    | TUserType (name, {of_string=of_string; to_string=to_string}) ->
+    | TUserType (name, tao) ->
       psuff, nlp,
-      ((pref^name^suff),
-       insert_string (to_string params))::l
+      ((pref ^ name ^ suff),
+       insert_string (Eliom_common.To_and_of_shared.to_string tao params)) :: l
     | TTypeFilter (t, check) -> aux t psuff nlp params pref suff l
     | TUnit -> psuff, nlp, l
     | TAny -> psuff, nlp, l@(List.map (fun (x,v) -> x,insert_string v) params)
@@ -322,15 +317,17 @@ let construct_params_list_raw nlp typ params = aux typ None nlp params "" "" []
     - string
     - bool
  *)
-let rec get_to_and_from : type a c. (a,'b,c) params_type -> a to_and_from = function
+let rec get_to_and_of : type a c. (a,'b,c) params_type -> a to_and_of = function
   | TOption (o,_) ->
-    let {to_string;of_string} = get_to_and_from o in
+    let {to_string;of_string} = get_to_and_of o in
     {of_string=(fun s -> try Some (of_string s) with _ -> None);
      to_string=(function
          | Some alpha -> to_string alpha
          | None -> "")}
-  | TUserType (_, x) -> x
-  | TESuffixu (_, x) -> x
+  | TUserType (_, tao) ->
+    Eliom_common.To_and_of_shared.to_and_of tao
+  | TESuffixu (_, tao) ->
+    Eliom_common.To_and_of_shared.to_and_of tao
   | TAtom(_,a) -> to_from_of_atom a
   | TJson (_, typ) -> (* server or client side *)
     {of_string=(fun s -> of_json ?typ s);
@@ -338,26 +335,18 @@ let rec get_to_and_from : type a c. (a,'b,c) params_type -> a to_and_from = func
   | TUnit -> {of_string=(fun _ -> ());
               to_string=(fun () -> "")}
   | _ ->
-    failwith "get_to_and_from: not implemented"
-
-
-let guard construct name guard =
-  let {of_string; to_string} = get_to_and_from (construct name) in
-  TUserType (name, {
-      of_string= (fun s -> let alpha = of_string s in assert (guard alpha); alpha);
-      to_string})
+    failwith "get_to_and_of: not implemented"
 
 (** Walk the parameter tree to search for a parameter, given its name *)
-let rec walk_parameter_tree : type a c. string -> (a,'b,c) params_type -> a to_and_from option = fun name x ->
+let rec walk_parameter_tree : type a c. string -> (a,'b,c) params_type -> a to_and_of option = fun name x ->
   let get name' =
     if name = name' then
-      Some (get_to_and_from x)
+      Some (get_to_and_of x)
     else
       None in
   match x with
     | TUserType (name', _) -> get name'
     | TCoord name' -> get name'
-    | TCoordv (name',_) -> get name'
     | TAtom (name',_) -> get name'
     | TFile name' -> get name'
     | TConst name' -> get name'
@@ -378,7 +367,7 @@ let rec walk_parameter_tree : type a c. string -> (a,'b,c) params_type -> a to_a
     | TRaw_post_data -> failwith "walk_parameter_tree with raw post data"
 
 
-(* contruct the string of parameters (& separated) for GET and POST *)
+(* construct the string of parameters (& separated) for GET and POST *)
 let construct_params_string l =
   Url.make_encoded_parameters (Eliommod_parameters.get_param_list l)
 
@@ -403,7 +392,6 @@ let make_params_names params =
         issuffix, (a, b)
       | TAtom(name,a) -> issuffix, prefix^name^suffix
       | TCoord(name) -> issuffix, prefix^name^suffix
-      | TCoordv(name,_) -> issuffix, prefix^name^suffix
       | TFile name -> issuffix, prefix^name^suffix
       | TUserType (name, _) -> issuffix, prefix^name^suffix
       | TJson (name, _)        -> issuffix, prefix^name^suffix
@@ -458,7 +446,6 @@ let rec add_pref_params : type a c. string -> (a,'b,c) params_type -> (a,'b,c) p
   | TUserType (name, to_of) -> TUserType (pref^name, to_of)
 
   | TCoord name -> TCoord (pref^name)
-  | TCoordv (name,t) -> TCoordv (pref^name,add_pref_params pref t)
   | TUnit -> TUnit
   | TAny -> TAny
   | TConst v -> TConst v  (* ??? no recursive call ??? *)
@@ -551,13 +538,361 @@ let rec wrap_param_type : type a c. (a,'b,c) params_type -> (a,'b,c) params_type
   | TSet t -> TSet (wrap_param_type t)
   | TSum (t1, t2) -> TSum ((wrap_param_type t1),
                            (wrap_param_type t2))
-  | TUserType (name, _) ->
-    (*VVV *)
-    failwith ("User service parameters '"^name^"' type not supported client side.")
-  | TCoordv (name, t) -> TCoordv (name, wrap_param_type t)
-  (* We remove the type information here: not possible to send a closure.
-     marshaling is just basic json marshaling on client side. *)
+  | TUserType (name, tao) ->
+    (* Eliom_common.To_and_of_shared.wrapper will take care of tao *)
+    TUserType (name, tao)
+  (* We remove the type information here: not possible to send a
+     closure.  marshaling is just basic json marshaling on client
+     side. *)
   | TJson (name, _) -> TJson (name, None)
   (* the filter is only on server side (at least for now) *)
   | TTypeFilter (t, _) -> TTypeFilter (t, None)
   | t -> t
+
+type _ is_unit =
+  | U_not : _    is_unit
+  | U_yes : unit is_unit
+
+let is_unit : type a c . (a, _, c) params_type -> a is_unit =
+  function
+  | TUnit -> U_yes
+  | _     -> U_not
+
+type anon_params_type = int
+
+let anonymise_params_type (t : ('a, 'b, 'c) params_type) : anon_params_type =
+  Hashtbl.hash_param 1000 1000 t
+
+type files = (string * file_info) list
+
+type +'a res_reconstr_param =
+  | Res_ of 'a * params' * files
+  | Errors_ of ((string * string * exn) list * params' * files)
+
+let end_of_list lp pref =
+  let f (a, _) =
+    try
+      String.(sub a 0 (length pref)) = pref
+    with Invalid_argument _ ->
+      false
+  in
+  not (List.exists f lp)
+
+(* The following function reconstructs the value of parameters from
+   expected type and GET or POST parameters *)
+let reconstruct_params_ typ params files nosuffixversion urlsuffix : 'a =
+  let rec parse_suffix : type a c. (a,'b,c) params_type -> string list -> (a * string list) = fun typ suff ->
+    match (typ, suff) with
+      | TESuffix _, l -> l, []
+          (*VVV encode=false? *)
+      | TESuffixs _, l -> Url.string_of_url_path ~encode:false l, []
+      | TESuffixu (_, tao), l ->
+          (try
+             (*VVV encode=false? *)
+             Eliom_common.To_and_of_shared.of_string tao
+               (Url.string_of_url_path ~encode:false l),
+             []
+           with e ->
+             raise (Eliom_common.Eliom_Typing_Error [("<suffix>", e)]))
+      | TOption (t,_), [] -> None, []
+      | TOption (t,_), ""::l -> None, l
+      | TOption (t,_), l ->
+        let r, ll = parse_suffix t l in
+        Some r, ll
+      | TList _, [] -> [],[]
+      | TList (_, t), l ->
+        let b, l = parse_suffix t l in
+        (match l with
+          | [] -> raise Eliom_common.Eliom_Wrong_parameter
+          | [""] -> [b], []
+          | _ ->
+            let c, l = parse_suffix typ l in
+            (b::c), l)
+      | TSet (TAtom (_, TBool) as y), l ->
+        let b, l = parse_suffix y l in [b], l
+      | TSet t, l ->
+          let b, l = parse_suffix t l in
+          (match l with
+            | [] -> raise Eliom_common.Eliom_Wrong_parameter
+            | [""] -> [b], []
+            | _ ->
+              let c, l = parse_suffix typ l in
+              (b::c), l)
+
+      | TProd (TList _, _), _ ->
+        failwith "Lists or sets in suffixes must be last parameters"
+      | TProd (TSet _, _), _ ->
+        failwith "Lists or sets in suffixes must be last parameters"
+      | TProd (t1, t2), l ->
+        (match parse_suffix t1 l with
+          | _, [] -> raise Eliom_common.Eliom_Wrong_parameter
+          | r, l ->
+            let rr, ll = parse_suffix t2 l in
+            (r, rr), ll)
+      | TAtom (name, t), v::l ->
+        (try atom_of_string t v, l
+         with e -> raise (Eliom_common.Eliom_Typing_Error [("<suffix>", e)]))
+      | TUserType (name, tao), v::l ->
+        (try
+           Eliom_common.To_and_of_shared.of_string tao v, l
+         with e -> raise (Eliom_common.Eliom_Typing_Error [("<suffix>", e)]))
+      | TTypeFilter (t, None), _ -> failwith "Type filter without filter"
+      | TTypeFilter (t, Some check), l ->
+        let (v, _) as a = parse_suffix t l in
+        check v;
+        a
+      | TConst value, v::l ->
+          if v = value
+          then (), l
+          else raise Eliom_common.Eliom_Wrong_parameter
+      | TSum (t1, t2), l ->
+        (try let x, l = parse_suffix t1 l in Inj1 x,l
+         with Eliom_common.Eliom_Wrong_parameter ->
+           let x,l = parse_suffix t2 l in Inj2 x,l)
+      | TCoord _, l ->
+        (match parse_suffix (TAtom ("",TInt)) l with
+         | _, [] -> raise Eliom_common.Eliom_Wrong_parameter
+         | r, l ->
+           let rr, ll = parse_suffix (TAtom ("",TInt)) l in
+           {abscissa = r; ordinate=rr}, ll)
+      | TNLParams _, _ ->
+          failwith "It is not possible to have non localized parameters in suffix"
+      | TJson (_, Some typ), v::l -> Deriving_Json.from_string typ v, l
+      | TJson (_, None), v::l -> assert false (* client side only *)
+      | TAny, _ -> failwith "It is not possible to use any in suffix. May be try with all_suffix ?"
+      | TFile _, _ -> assert false
+      | TRaw_post_data, _ -> assert false
+      | TUnit, _ -> failwith "It is not possible to use TUnit in suffix."
+      | TSuffix _, _ -> failwith "It is not possible to use TSuffix in suffix."
+      | _,[]-> raise Eliom_common.Eliom_Wrong_parameter
+  in
+  let rec aux_list : type a c. (a,'b,c) params_type -> params' -> files -> string -> string -> string -> a list res_reconstr_param =
+    fun t params files name pref suff ->
+      let rec loop_list i lp fl pref =
+        if
+          match t with
+          | TFile _ ->
+            end_of_list fl pref
+          | _ ->
+            end_of_list lp pref
+        then
+          Res_ ([], lp, fl)
+        else
+          match aux t lp fl pref (make_list_suffix i) with
+            | Res_ (v, lp2, f) ->
+              (match loop_list (i+1) lp2 f pref with
+                | Res_ (v2,lp3,f2) -> Res_ ((v::v2),lp3,f2)
+                | err -> err)
+            | Errors_ errs -> Errors_ errs
+      in
+      loop_list 0 params files (pref^name^suff^".")
+  and aux : type a c. (a, 'b, c) params_type -> params' -> files -> string -> string -> a res_reconstr_param =
+    fun typ params files pref suff ->
+      match typ with
+        | TNLParams {param=t} -> aux t params files pref suff
+        | TProd (t1, t2) ->
+          (match aux t1 params files pref suff with
+            | Res_ (v1, l1, f) ->
+              (match aux t2 l1 f pref suff with
+                | Res_ (v2, l2, f2) -> Res_ ((v1, v2), l2, f2)
+                | Errors_ (err,params,files) -> Errors_ (err,params,files))
+            | Errors_ (errs, l, f) ->
+              (match aux t2 l f pref suff with
+                | Res_ (_, ll, ff) -> Errors_ (errs, ll, ff)
+                | Errors_ (errs2, ll, ff) -> Errors_ ((errs2@errs), ll, ff)))
+        | TOption (TAtom (_,TString) as t,b) ->
+          (try
+             (match aux t params files pref suff with
+               | Res_ (v, l, f) ->
+                 if b && String.length v = 0  (* Is the value an empty string? *)
+                 then Res_ (None, l, f)
+                 else Res_ (Some v, l, f)
+               | Errors_ (errs, ll, ff) when List.for_all (fun (_,s,_) -> s="") errs -> Res_ (None, ll, ff)
+               | Errors_ err   -> Errors_ err)
+           with Not_found -> Res_ (None, params, files))
+        | TOption (t,b) ->
+          (try
+             (match aux t params files pref suff with
+               | Res_ (v, l, f) -> Res_ (Some v, l, f)
+               | Errors_ (errs, ll, ff) when List.for_all (fun (_,s,_) -> s="") errs -> Res_ (None, ll, ff)
+               | Errors_ err   -> Errors_ err)
+           with Not_found -> Res_ (None, params, files))
+        | TList (n, t) -> aux_list t params files n pref suff
+        | TSet (TAtom (_, TBool) as y) ->
+          (match aux y params files pref suff with
+           | Res_ (vv, ll, ff) ->
+             Res_ ([vv], ll, ff)
+           | Errors_ (err, ll, ff) ->
+             Errors_ (err, ll, ff))
+        | TSet t ->
+          let rec aux_set params files =
+            try
+              match aux t params files pref suff with
+                | Res_ (vv, ll, ff) ->
+                  (match aux_set ll ff with
+                    | Res_ (vv2, ll2, ff2) ->
+                      Res_ (vv::vv2, ll2, ff2)
+                    | err -> err)
+                | Errors_ (errs, ll, ff) when ll = params && ff = files ->
+                  Res_ ([], params, files)
+                | Errors_ (errs, ll, ff) ->
+                  (match aux_set ll ff with
+                   | Res_ (_, ll2, ff2) -> Errors_ (errs, ll2, ff2)
+                   | Errors_ (errs2, ll2, ff2) -> Errors_ (errs@errs2, ll2, ff2))
+            with Not_found -> Res_ ([], params, files)
+          in aux_set params files
+        | TSum (t1, t2) ->
+          begin
+            (* We try to decode both cases, if both succeed,
+               we choose the one that consumes parameters
+               (or the 1st one if none consumes) *)
+            (try
+               match aux t1 params files pref suff with
+               | Res_ (v1,l1,files1) ->
+                 if l1 = params
+                 then try
+                     (match aux t2 params files pref suff with
+                      | Res_ (v2,l2,files2) when l2 <> params ->
+                        Res_ (Inj2 v2,l2,files2)
+                      | _ -> Res_ (Inj1 v1,l1,files1))
+                   with Not_found -> Res_ (Inj1 v1,l1,files1)
+                 else Res_ (Inj1 v1,l1,files1)
+               | Errors_ err -> Errors_ err
+             with Not_found ->
+               (match aux t2 params files pref suff with
+                | Res_ (v,l,files) -> Res_ (Inj2 v,l,files)
+                | Errors_ err -> Errors_ err))
+          end
+        | TAtom (name,TBool) ->
+          (try
+             let v,l = (List.assoc_remove (pref^name^suff) params) in
+             Res_ (true,l,files)
+           with Not_found -> Res_ (false, params, files))
+
+        | TAtom (name,a) ->
+          let v,l = (List.assoc_remove (pref^name^suff) params) in
+          (try
+             Res_ (atom_of_string a v, l, files)
+           with e ->
+             Errors_ ([pref^name^suff, v, e], l, files))
+        | TFile name ->
+          (try
+             let v,f = List.assoc_remove (pref^name^suff) files in
+             Res_ (v, params, f)
+           with e ->
+             Errors_ ([pref^name^suff,"",e], [], files))
+        | TCoord name ->
+          let r1 =
+            let v, l = (List.assoc_remove (pref^name^suff^".x") params) in
+            (try (Res_ ((int_of_string v), l, files))
+             with e -> Errors_ ([(pref^name^suff^".x"), v, e], l, files))
+          in
+          (match r1 with
+           | Res_ (x1, l1, f) ->
+             let v, l = (List.assoc_remove (pref^name^suff^".y") l1) in
+             (try (Res_ (
+                  {abscissa= x1;
+                   ordinate= int_of_string v}, l, f))
+              with e -> Errors_ ([(pref^name^suff^".y"), v, e], l, f))
+           | Errors_ (errs, l1, f) ->
+             let v, l = (List.assoc_remove (pref^name^suff^".y") l1) in
+             (try
+                ignore (int_of_string v);
+                Errors_ (errs, l, f)
+              with e -> Errors_ (((pref^name^suff^".y"), v, e)::errs, l, f)))
+        | TUserType (name, tao) ->
+          let v,l = (List.assoc_remove (pref^name^suff) params) in
+          (try Res_ (Eliom_common.To_and_of_shared.of_string tao v,l,files)
+           with e -> Errors_ ([(pref^name^suff),v,e], l, files))
+        | TTypeFilter (t, None) -> failwith "Type filter without filter"
+        | TTypeFilter (t, Some check) ->
+          (match aux t params files pref suff with
+            | Res_ (v, l, files) as a ->
+              (try check v; a
+               with e -> Errors_ (["<type_check>","<>",e], l, files))
+            | a -> a)
+        | TUnit -> Res_ ((), params, files)
+        | TAny -> Res_  (params, [], files)
+        | TConst _ -> Res_ ((), params, files)
+        | TESuffix n ->
+          let v,l = List.assoc_remove n params in
+          (* cannot have prefix or suffix *)
+          Res_ (Eliom_lib.Url.split_path v, l, files)
+        | TESuffixs n ->
+          let v,l = List.assoc_remove n params in
+          (* cannot have prefix or suffix *)
+          Res_ (v, l, files)
+        | TESuffixu (n, tao) ->
+          let v,l = List.assoc_remove n params in
+          (* cannot have prefix or suffix *)
+          (try Res_ (Eliom_common.To_and_of_shared.of_string tao v, l, files)
+           with e -> Errors_ ([(pref^n^suff), v, e], l, files))
+        | TSuffix (_, s) ->
+          (match urlsuffix with
+            | None ->
+              if nosuffixversion
+              (* the special page name "nosuffix" is present *)
+              then aux s params files pref suff
+              else raise Eliom_common.Eliom_Wrong_parameter
+            | Some urlsuffix ->
+              (match parse_suffix s urlsuffix with
+                | p, [] -> Res_ (p, params, files)
+                | _ -> raise Eliom_common.Eliom_Wrong_parameter))
+        | TJson (name, Some typ) ->
+          let v,l = List.assoc_remove (pref^name^suff) params in
+          Res_ ((of_json ~typ v),l,files)
+        | TJson (name, None) -> assert false
+        (* Never unmarshal server side without type! *)
+        | TRaw_post_data -> raise Eliom_common.Eliom_Wrong_parameter
+  in
+  try
+    match aux typ params files "" "" with
+      | Res_ (v, l, files) ->
+        if (l, files) = ([], [])
+        then v
+        else begin
+          if l <> []
+          then Lwt_log.ign_debug_f ~section
+              "Eliom_Wrong_parameter: params non-empty (ERROR): %a" (fun () l -> String.concat ", " (List.map (fun (x,k) -> x^"="^k) l)) l;
+          if files <> []
+          then Lwt_log.ign_debug_f ~section
+              "Eliom_Wrong_parameter: files non-empty (ERROR): %a" (fun () files -> String.concat ", " (List.map (fun (x,k) -> x) files)) files;
+          raise Eliom_common.Eliom_Wrong_parameter
+        end
+      | Errors_ (errs, l, files) ->
+        if (l, files) = ([], [])
+        then raise (Eliom_common.Eliom_Typing_Error (List.map (fun (v,l,e) -> (v,e)) errs))
+        else raise Eliom_common.Eliom_Wrong_parameter
+  with | Not_found -> raise Eliom_common.Eliom_Wrong_parameter
+
+
+let reconstruct_params ~sp (type a) (type c) (typ : (a,'b,c) params_type) params files nosuffixversion urlsuffix : a Lwt.t =
+  match typ, params, files with
+  (* FIXME *)
+  | TRaw_post_data, None, None ->
+    Eliom_request_info.raw_post_data sp
+  | typ, None, None ->
+    (try
+       Lwt.return
+         (reconstruct_params_
+            typ [] [] nosuffixversion urlsuffix)
+     with e -> Lwt.fail e)
+  | typ, _, _ ->
+    let%lwt params =
+      match params with
+      | Some params ->
+        params
+      | None ->
+        Lwt.return_nil
+    in
+    let%lwt files =
+      match files with
+      | Some files ->
+        files
+      | None ->
+        Lwt.return_nil
+    in
+    try
+       Lwt.return
+         (reconstruct_params_ typ params files nosuffixversion urlsuffix)
+     with e -> Lwt.fail e
