@@ -72,7 +72,7 @@ val discard_data :
   unit ->
   unit Lwt.t
 
-(** Remove all services registered for the given scope (the default beeing
+(** Remove all services registered for the given scope (the default being
     [`Session]). *)
 val discard_services :
   scope:[< Eliom_common.user_scope ] ->
@@ -128,9 +128,8 @@ val persistent_data_state_status :
     Web site).
 
     Cookies can have an expiration date, specified (in seconds
-    since the 1st of January 1970) in the optional parameter [?exp].
-    If the parameter is not set,
-    the expiration date will be when the browser is closed.
+    since the 1st of January 1970) in the optional parameter [?exp]
+    (see how to set default expiration dates below).
 
     Secure cookies are sent by the browser only with HTTPS (default: [false]).
 *)
@@ -429,8 +428,10 @@ val set_max_volatile_states_for_group_or_subnet :
 (** The functions in this section ask the browser to set the state cookie
     expiration date, for the different kinds of session, in seconds,
     since the 1st of January 1970. [None] means the cookie will expire
-    when the browser is closed. Note: there is no way to set cookies
+    when the browser is closed. There is no way to set cookies
     for an infinite time on browsers.
+    By default, Eliom sets default expiration date to
+    10 years after opening the session.
 
     By default, it will affect regular browser cookies (sessions).
     But if you set [~cookie_level:`Client_process],
@@ -510,13 +511,7 @@ val set_global_volatile_state_timeout :
   ?override_configfile:bool ->
   float option -> unit
 
-val set_default_global_service_state_timeout :
-  cookie_level:[< Eliom_common.cookie_level ] ->
-  ?override_configfile:bool ->
-  float option -> unit
-
-(** Sets the (server side) timeout for service states.
-*)
+(** Sets the (server side) timeout for service states. *)
 val set_global_service_state_timeout :
   cookie_scope:[< Eliom_common.cookie_scope ] ->
   ?secure: bool ->
@@ -669,6 +664,7 @@ val get_persistent_data_state_timeout :
 
 (** The type of (volatile) state data tables. *)
 type 'a volatile_table
+
 (** The type of persistent state data tables. *)
 type 'a persistent_table
 
@@ -683,7 +679,6 @@ val discard_all :
   unit ->
   unit Lwt.t
 (*VVV missing: scope group *)
-(*VVV missing ~secure? *)
 
 (** Discard server side data for all clients, for the given scope.
 
@@ -751,7 +746,10 @@ module Ext : sig
     ([> `Session_group ], [> `Service ]) state
 
   (** [current_volatile_session_state ~scope] returns the state corresponding
-      to current session in scope [scope]. *)
+      to current session in scope [scope].
+      Raises [Not_found] if not connected
+      or [Eliom_common.Eliom_Session_expired] if a cookie was present but
+      expired. *)
   val current_volatile_session_state :
     ?secure:bool ->
     ?scope:Eliom_common.session_scope ->
@@ -800,7 +798,8 @@ module Ext : sig
     (([< `Session | `Client_process ], 'k) state -> unit) ->
     unit
 
-  (** Fold all sessions in a groups, or all client processes in a session.
+  (** Fold all sessions in a groups, or all client processes in a session
+      (volatile and persistent).
       See {!fold_volatile_sub_states} for explanation about the [?sitedata]
       parameter.
   *)
@@ -811,7 +810,8 @@ module Ext : sig
     ('a -> ([< `Session | `Client_process ], 'k) state -> 'a Lwt.t) ->
     'a -> 'a Lwt.t
 
-  (** Iter on all sessions in a groups, or all client processes in a session.
+  (** Iter on all sessions in a groups, or all client processes in a session
+      (volatile and persistent).
       See {!fold_volatile_sub_states} for explanation about the [?sitedata]
       parameter.
   *)
@@ -1006,15 +1006,16 @@ val remove_volatile_data :
   unit
 (**/**)
 
-
 (**/**)
+
 (** {4 Persistent state data} *)
-(** creates a table on hard disk where you can store the session data for
-    all users. It uses {!Ocsipersist}.  (low level) *)
+
+(** creates a table on hard disk where you can store the session data
+    for all users. It uses {!Ocsipersist}.  (low level) *)
 val create_persistent_table :
   scope:Eliom_common.user_scope ->
   ?secure:bool ->
-  string -> 'a persistent_table
+  string -> 'a persistent_table Lwt.t
 
 (** gets persistent session data for the current persistent session (if any).
     (low level) *)
