@@ -32,14 +32,18 @@ open Ocsigen_extensions
 
 
 (****************************************************************************)
-let default_max_persistent_sessions_per_group = ref 5
+let default_max_persistent_data_sessions_per_group = ref 5
 let default_max_service_sessions_per_group = ref 5
 let default_max_service_sessions_per_subnet = ref 1000000
-let default_max_data_sessions_per_group = ref 5
-let default_max_data_sessions_per_subnet = ref 1000000
-let default_max_persistent_tab_sessions_per_group = ref 50
+let default_max_volatile_data_sessions_per_group = ref 5
+let default_max_volatile_data_sessions_per_subnet = ref 1000000
+let default_max_persistent_data_tab_sessions_per_group = ref 50
 let default_max_service_tab_sessions_per_group = ref 50
-let default_max_data_tab_sessions_per_group = ref 50
+let default_max_volatile_data_tab_sessions_per_group = ref 50
+let default_secure_cookies = ref false
+let default_application_script = ref (false, false)
+let default_cache_global_data = ref None
+let default_html_content_type = ref None
 
 (* Subnet defaults be large enough, because it must work behind a reverse proxy.
 
@@ -104,7 +108,7 @@ let new_sitedata =
           {Eliom_common.servtimeout = None, None, [];
            datatimeout =  None, None, [];
            perstimeout =  None, None, [];
-	   site_value_table = Polytables.create ();
+           site_value_table = Polytables.create ();
            site_dir = site_dir;
 (*VVV encode=false??? *)
            site_dir_string = Url.string_of_url_path
@@ -115,7 +119,7 @@ let new_sitedata =
               Eliom_common.empty_tables
                 !default_max_anonymous_services_per_subnet
                 false;
-	   registered_scope_hierarchies = Eliom_common.Hier_set.empty;
+           registered_scope_hierarchies = Eliom_common.Hier_set.empty;
            session_services = Eliommod_cookies.new_service_cookie_table ();
            session_data = Eliommod_cookies.new_data_cookie_table ();
            group_of_groups = gog;
@@ -124,29 +128,45 @@ let new_sitedata =
            exn_handler = Eliommod_pagegen.def_handler;
            unregistered_services = [];
            unregistered_na_services = [];
+
            max_service_sessions_per_group =
-              !default_max_service_sessions_per_group, false;
+  !default_max_service_sessions_per_group, false;
+
            max_volatile_data_sessions_per_group =
-              !default_max_service_sessions_per_group, false;
-           max_persistent_data_sessions_per_group =
-              Some !default_max_persistent_sessions_per_group, false;
+  !default_max_volatile_data_sessions_per_group, false;
+
+              max_persistent_data_sessions_per_group =
+Some !default_max_persistent_data_sessions_per_group, false;
+
            max_service_tab_sessions_per_group =
-              !default_max_service_tab_sessions_per_group, false;
+  !default_max_service_tab_sessions_per_group, false;
+
            max_volatile_data_tab_sessions_per_group =
-              !default_max_service_tab_sessions_per_group, false;
-           max_persistent_data_tab_sessions_per_group =
-              Some !default_max_persistent_tab_sessions_per_group, false;
+  !default_max_volatile_data_tab_sessions_per_group, false;
+
+              max_persistent_data_tab_sessions_per_group =
+Some !default_max_persistent_data_tab_sessions_per_group, false;
+
            max_service_sessions_per_subnet =
-              !default_max_data_sessions_per_subnet, false;
+  !default_max_service_sessions_per_subnet, false;
+
            max_volatile_data_sessions_per_subnet =
-              !default_max_data_sessions_per_subnet, false;
+  !default_max_volatile_data_sessions_per_subnet, false;
+
            max_anonymous_services_per_session =
-              !default_max_anonymous_services_per_session, false;
+  !default_max_anonymous_services_per_session, false;
+
            max_anonymous_services_per_subnet =
-              !default_max_anonymous_services_per_subnet, false;
+  !default_max_anonymous_services_per_subnet, false;
+
+           secure_cookies = !default_secure_cookies;
+
            dlist_ip_table = dlist_table;
            ipv4mask = None, false;
            ipv6mask = None, false;
+           application_script = !default_application_script;
+           cache_global_data = !default_cache_global_data;
+           html_content_type = !default_html_content_type;
           }
         in
         Ocsigen_cache.Dlist.set_finaliser_after
@@ -200,8 +220,6 @@ let new_sitedata =
 (****************************************************************************)
 (****************************************************************************)
 (****************************************************************************)
-open Simplexmlparser
-
 
 (* The following is common to global config and site config *)
 let parse_eliom_option
@@ -221,8 +239,12 @@ let parse_eliom_option
      set_max_services_per_session,
      set_max_services_per_subnet,
      set_max_volatile_groups_per_site,
+     set_secure_cookies,
      set_ipv4mask,
-     set_ipv6mask
+     set_ipv6mask,
+     set_application_script,
+     set_global_data_caching,
+     set_html_content_type
     )
     =
   let parse_timeout_attrs tn attrs =
@@ -261,30 +283,74 @@ let parse_eliom_option
     if (not globaloption) || sn = None
     then
       let sn = match sn with
-        | None -> None
-        | Some "" -> Some None
+        | None -> None (* all hierarchies *)
+        | Some "" -> Some None (* default hierarchy *)
         | c -> Some c
       in (a, sn, ct)
     else
       raise
         (Error_in_config_file
-           ("Eliom: sessionname attribute not allowed for "^tn^" tag in global configuration"))
+           ("Eliom: hierarchyname attribute not allowed for "^tn^" tag in global configuration"))
+  in
+  let convert_attr tag f v =
+    try
+      f v
+    with Invalid_argument _ ->
+      raise (Error_in_config_file
+               (Printf.sprintf
+                  "Eliom: Wrong attribute value for tag %s \
+                   in element cacheglobaldata"
+                  tag))
+  in
+  let parse_application_script_attrs attrs =
+    let rec aux defer async attrs =
+      match attrs with
+      | [] -> (defer, async)
+      | ("defer", v) :: rem ->
+        aux (convert_attr "defer" bool_of_string v) async rem
+      | ("async", v) :: rem ->
+        aux defer (convert_attr "async" bool_of_string v) rem
+      | (tag, _) :: _ ->
+        raise
+          (Error_in_config_file
+             (Printf.sprintf
+                "Eliom: attribute %s not allowed in element applicationscript"
+                tag))
+    in
+    aux false false attrs
+  in
+  let parse_global_data_caching_attrs attrs =
+    let rec aux path max_age attrs =
+      match attrs with
+      | [] -> Some (path, max_age)
+      | ("path", p) :: rem ->
+        aux (Eliom_lib.Url.split_path p) max_age rem
+      | ("cache", v) :: rem ->
+        aux path (convert_attr "cache" int_of_string v) rem
+      | (tag, _) :: _ ->
+        raise
+          (Error_in_config_file
+             (Printf.sprintf
+                "Eliom: attribute %s not allowed in element cacheglobaldata"
+                tag))
+    in
+    aux [] 0 attrs
   in
   function
-  | (Element ("volatiletimeout", attrs, [])) ->
+  | (Xml.Element ("volatiletimeout", attrs, [])) ->
       let t, snoo, ct = parse_timeout_attrs "volatiletimeout" attrs in
       set_volatile_timeout ct snoo (t : float option)
-  | (Element ("datatimeout", attrs, [])) ->
+  | (Xml.Element ("datatimeout", attrs, [])) ->
       let t, snoo, ct = parse_timeout_attrs "datatimeout" attrs in
       set_data_timeout ct snoo t
-  | (Element ("servicetimeout", attrs, [])) ->
+  | (Xml.Element ("servicetimeout", attrs, [])) ->
       let t, snoo, ct = parse_timeout_attrs "servicetimeout" attrs in
       set_service_timeout ct snoo t
-  | (Element ("persistenttimeout", attrs, [])) ->
+  | (Xml.Element ("persistenttimeout", attrs, [])) ->
       let t, snoo, ct = parse_timeout_attrs "persistenttimeout" attrs in
       set_persistent_timeout ct snoo t
 
-  | (Element ("maxvolatilesessionspergroup", [("value", v)], [])) ->
+  | (Xml.Element ("maxvolatilesessionspergroup", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_service_sessions_per_group i;
@@ -293,21 +359,21 @@ let parse_eliom_option
          raise
            (Error_in_config_file
               ("Eliom: Wrong attribute value for maxvolatilesessionspergroup tag")))
-  | (Element ("maxservicesessionspergroup", [("value", v)], [])) ->
+  | (Xml.Element ("maxservicesessionspergroup", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_service_sessions_per_group i;
        with Failure _ ->
          raise (Error_in_config_file
                   ("Eliom: Wrong attribute value for maxservicesessionspergroup tag")))
-  | (Element ("maxdatasessionspergroup", [("value", v)], [])) ->
+  | (Xml.Element ("maxdatasessionspergroup", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_data_sessions_per_group i
        with Failure _ ->
          raise (Error_in_config_file
                   ("Eliom: Wrong attribute value for maxdatasessionspergroup tag")))
-  | (Element ("maxvolatilesessionspersubnet", [("value", v)], [])) ->
+  | (Xml.Element ("maxvolatilesessionspersubnet", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_service_sessions_per_subnet i;
@@ -315,21 +381,21 @@ let parse_eliom_option
        with Failure _ ->
          raise (Error_in_config_file
                   ("Eliom: Wrong attribute value for maxvolatilesessionspersubnet tag")))
-  | (Element ("maxservicesessionspersubnet", [("value", v)], [])) ->
+  | (Xml.Element ("maxservicesessionspersubnet", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_service_sessions_per_subnet i;
        with Failure _ ->
          raise (Error_in_config_file
                   ("Eliom: Wrong attribute value for maxservicesessionspersubnet tag")))
-  | (Element ("maxdatasessionspersubnet", [("value", v)], [])) ->
+  | (Xml.Element ("maxdatasessionspersubnet", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_data_sessions_per_subnet i
        with Failure _ ->
          raise (Error_in_config_file
                   ("Eliom: Wrong attribute value for maxdatasessionspersubnet tag")))
-  | (Element ("maxpersistentsessionspergroup", [("value", v)], [])) ->
+  | (Xml.Element ("maxpersistentsessionspergroup", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_persistent_sessions_per_group i;
@@ -337,7 +403,7 @@ let parse_eliom_option
          raise
            (Error_in_config_file
               ("Eliom: Wrong attribute value for maxpersistentsessionspergroup tag")))
-  | (Element ("maxvolatiletabsessionspergroup", [("value", v)], [])) ->
+  | (Xml.Element ("maxvolatiletabsessionspergroup", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_service_tab_sessions_per_group i;
@@ -346,21 +412,21 @@ let parse_eliom_option
          raise
            (Error_in_config_file
               ("Eliom: Wrong attribute value for maxvolatiletabsessionspergroup tag")))
-  | (Element ("maxservicetabsessionspergroup", [("value", v)], [])) ->
+  | (Xml.Element ("maxservicetabsessionspergroup", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_service_tab_sessions_per_group i;
        with Failure _ ->
          raise (Error_in_config_file
                   ("Eliom: Wrong attribute value for maxservicetabsessionspergroup tag")))
-  | (Element ("maxdatatabsessionspergroup", [("value", v)], [])) ->
+  | (Xml.Element ("maxdatatabsessionspergroup", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_data_tab_sessions_per_group i
        with Failure _ ->
          raise (Error_in_config_file
                   ("Eliom: Wrong attribute value for maxdatatabsessionspergroup tag")))
-  | (Element ("maxpersistenttabsessionspergroup", [("value", v)], [])) ->
+  | (Xml.Element ("maxpersistenttabsessionspergroup", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_persistent_tab_sessions_per_group i;
@@ -368,21 +434,21 @@ let parse_eliom_option
          raise
            (Error_in_config_file
               ("Eliom: Wrong attribute value for maxpersistenttabsessionspergroup tag")))
-  | (Element ("maxanonymouscoservicespersession", [("value", v)], [])) ->
+  | (Xml.Element ("maxanonymouscoservicespersession", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_services_per_session i;
        with Failure _ ->
          raise (Error_in_config_file
                   ("Eliom: Wrong attribute value for maxanonymouscoservicespersession tag")))
-  | (Element ("maxanonymouscoservicespersubnet", [("value", v)], [])) ->
+  | (Xml.Element ("maxanonymouscoservicespersubnet", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_services_per_subnet i;
        with Failure _ ->
          raise (Error_in_config_file
                   ("Eliom: Wrong attribute value for maxanonymouscoservicespersubnet tag")))
-  | (Element ("maxvolatilegroupspersite", [("value", v)], [])) ->
+  | (Xml.Element ("maxvolatilegroupspersite", [("value", v)], [])) ->
       (try
          let i = int_of_string v in
          set_max_volatile_groups_per_site i
@@ -390,14 +456,26 @@ let parse_eliom_option
          raise (Error_in_config_file
                   ("Eliom: Wrong attribute value for maxvolatilegroupspersite tag")))
 
-  | (Element ("ipv4subnetmask", [("value", v)], [])) ->
+  | (Xml.Element ("securecookies", [("value", v)], [])) ->
+      (try
+         let i = match v with
+           | "true" -> true
+           | "false" -> false
+           | _ -> failwith ""
+         in
+         set_secure_cookies i
+       with Failure _ ->
+         raise (Error_in_config_file
+                  ("Eliom: Wrong attribute value for securecookies tag")))
+
+  | (Xml.Element ("ipv4subnetmask", [("value", v)], [])) ->
       (try
          let mask = int_of_string v in
          set_ipv4mask mask
        with _ ->
          raise (Error_in_config_file
                   ("Eliom: Wrong attribute value for ipv4subnetmask tag")))
-  | (Element ("ipv6subnetmask", [("value", v)], [])) ->
+  | (Xml.Element ("ipv6subnetmask", [("value", v)], [])) ->
       (try
          let mask = int_of_string v in
          set_ipv6mask mask
@@ -405,7 +483,16 @@ let parse_eliom_option
          raise (Error_in_config_file
                   ("Eliom: Wrong attribute value for ipv6subnetmask tag")))
 
-  | (Element (s, _, _)) ->
+  | (Xml.Element ("applicationscript", attrs, [])) ->
+      set_application_script (parse_application_script_attrs attrs)
+
+  | (Xml.Element ("cacheglobaldata", attrs, [])) ->
+      set_global_data_caching (parse_global_data_caching_attrs attrs)
+
+  | (Xml.Element ("htmlcontenttype", [("value", v)], [])) ->
+    set_html_content_type v
+
+  | (Xml.Element (s, _, _)) ->
       raise (Error_in_config_file
                ("Unexpected content <"^s^"> inside eliom config"))
   | _ -> raise (Error_in_config_file ("Unexpected content inside eliom config"))
@@ -416,7 +503,7 @@ let parse_eliom_options f l =
     | [] -> rest
     | e::l ->
         try
-          parse_eliom_option true f e;
+          parse_eliom_option false f e;
           aux rest l
         with Error_in_config_file _ -> aux (e::rest) l
   in List.rev (aux [] l)
@@ -427,7 +514,7 @@ let parse_eliom_options f l =
 
 let rec parse_global_config = function
   | [] -> ()
-  | (Element ("sessiongcfrequency", [("value", s)], p))::ll ->
+  | (Xml.Element ("sessiongcfrequency", [("value", s)], p))::ll ->
       (try
         let t = float_of_string s in
         Eliommod_gc.set_servicesessiongcfrequency (Some t);
@@ -441,7 +528,7 @@ let rec parse_global_config = function
         else raise (Error_in_config_file
                       "Eliom: Wrong value for <sessiongcfrequency>"));
       parse_global_config ll
-  | (Element ("servicesessiongcfrequency", [("value", s)], p))::ll ->
+  | (Xml.Element ("servicesessiongcfrequency", [("value", s)], p))::ll ->
       (try
         Eliommod_gc.set_servicesessiongcfrequency (Some (float_of_string s))
       with Failure _ ->
@@ -450,7 +537,7 @@ let rec parse_global_config = function
         else raise (Error_in_config_file
                       "Eliom: Wrong value for <servicesessiongcfrequency>"));
       parse_global_config ll
-  | (Element ("datasessiongcfrequency", [("value", s)], p))::ll ->
+  | (Xml.Element ("datasessiongcfrequency", [("value", s)], p))::ll ->
       (try
         Eliommod_gc.set_datasessiongcfrequency (Some (float_of_string s))
       with Failure _ ->
@@ -459,7 +546,7 @@ let rec parse_global_config = function
         else raise (Error_in_config_file
                       "Eliom: Wrong value for <datasessiongcfrequency>"));
       parse_global_config ll
-  | (Element ("persistentsessiongcfrequency",
+  | (Xml.Element ("persistentsessiongcfrequency",
               [("value", s)], p))::ll ->
                 (try
                   Eliommod_gc.set_persistentsessiongcfrequency
@@ -473,24 +560,30 @@ let rec parse_global_config = function
                 parse_global_config ll
   | e::ll ->
       parse_eliom_option
-        false
-        ((fun ct _ -> Eliommod_timeouts.set_default_volatile_timeout ct),
-         (fun ct _ -> Eliommod_timeouts.set_default_data_timeout ct),
-         (fun ct _ -> Eliommod_timeouts.set_default_service_timeout ct),
-         (fun ct _ -> Eliommod_timeouts.set_default_persistent_timeout ct),
+        true
+        ((fun ct _ m ->
+           Eliommod_timeouts.set_default `Data ct m;
+           Eliommod_timeouts.set_default `Service ct m),
+         (fun ct _ -> Eliommod_timeouts.set_default `Data ct),
+         (fun ct _ -> Eliommod_timeouts.set_default `Service ct),
+         (fun ct _ -> Eliommod_timeouts.set_default `Persistent ct),
          (fun v -> default_max_service_sessions_per_group := v),
          (fun v -> default_max_service_sessions_per_subnet := v),
-         (fun v -> default_max_data_sessions_per_group := v),
-         (fun v -> default_max_data_sessions_per_subnet := v),
-         (fun v -> default_max_persistent_sessions_per_group := v),
+         (fun v -> default_max_volatile_data_sessions_per_group := v),
+         (fun v -> default_max_volatile_data_sessions_per_subnet := v),
+         (fun v -> default_max_persistent_data_sessions_per_group := v),
          (fun v -> default_max_service_tab_sessions_per_group := v),
-         (fun v -> default_max_data_tab_sessions_per_group := v),
-         (fun v -> default_max_persistent_tab_sessions_per_group := v),
+         (fun v -> default_max_volatile_data_tab_sessions_per_group := v),
+         (fun v -> default_max_persistent_data_tab_sessions_per_group := v),
          (fun v -> default_max_anonymous_services_per_session := v),
          (fun v -> default_max_anonymous_services_per_subnet := v),
          (fun v -> default_max_volatile_groups_per_site := v),
+         (fun v -> default_secure_cookies := v),
          (fun v -> Eliom_common.ipv4mask := v),
-         (fun v -> Eliom_common.ipv6mask := v)
+         (fun v -> Eliom_common.ipv6mask := v),
+         (fun v -> default_application_script := v),
+         (fun v -> default_cache_global_data := v),
+         (fun v -> default_html_content_type := Some v)
         )
         e;
       parse_global_config ll
@@ -586,7 +679,7 @@ let handle_init_exn = function
          "\nPlease correct your modules and make sure you have linked in all the modules...")
   | Eliom_common.Eliom_site_information_not_available f ->
       ("Eliom: Bad use of function \""^f^
-          "\". Must be used only during site intialisation phase (or, sometimes, also during request).")
+          "\". Must be used only during site initialisation phase (or, sometimes, also during request).")
   | Eliom_common.Eliom_page_erasing s ->
       ("Eliom: You cannot create a page or directory here. "^s^
        " already exists. Please correct your modules.")
@@ -601,14 +694,19 @@ let site_init_ref = ref []
 let register_site_init e = site_init_ref := e:: !site_init_ref
 
 let config = ref []
+let config_in_tag = ref "" (* the parent tag of the currently handled tag *)
 
 type module_to_load = Files of string list | Name of string
 
-let load_eliom_module sitedata cmo_or_name content =
+let load_eliom_module sitedata cmo_or_name parent_tag content =
   let preload () =
     config := content;
+    config_in_tag := parent_tag;
     Eliom_common.begin_load_eliom_module ();
-    List.iter (fun f -> f ()) !site_init_ref
+    (* I want to be able to define global client values during that phase: *)
+    Eliom_syntax.set_global true;
+    List.iter (fun f -> f ()) !site_init_ref;
+    Eliom_syntax.set_global false
   in
   let postload () =
     Eliom_common.end_load_eliom_module ();
@@ -693,13 +791,13 @@ let parse_config hostpattern conf_info site_dir =
         raise
           (Error_in_config_file ("Wrong attribute for <eliom>: "^s))
   in fun _ parse_site -> function
-    | Element ("eliommodule", atts, content) ->
+    | Xml.Element ("eliommodule", atts, content) ->
       Eliom_extension.register_eliom_extension
         default_module_action;
       (match parse_module_attrs None atts with
         | Some file_or_name ->
           exception_during_eliommodule_loading := true;
-          load_eliom_module sitedata file_or_name content;
+          load_eliom_module sitedata file_or_name "eliommodule" content;
           exception_during_eliommodule_loading := false
         | _ -> ());
       if Eliom_extension.get_eliom_extension () != default_module_action
@@ -707,7 +805,7 @@ let parse_config hostpattern conf_info site_dir =
         (Some (Eliom_extension.get_eliom_extension ()))
         sitedata
       else gen_nothing ()
-    | Element ("eliom", atts, content) ->
+    | Xml.Element ("eliom", atts, content) ->
 (*--- if we put the line "new_sitedata" here, then there is
   one service table for each <eliom> tag ...
   I think the other one is the best,
@@ -718,31 +816,43 @@ let parse_config hostpattern conf_info site_dir =
         let set_timeout (f : ?full_st_name:Eliom_common.full_state_name ->
                          ?cookie_level:[< Eliom_common.cookie_level ] ->
                          recompute_expdates:bool ->
-                         bool -> bool -> Eliom_common.sitedata ->
+                         bool (* override configfile *) ->
+                         bool (* from config file *) ->
+                         Eliom_common.sitedata ->
                          float option -> unit)
             cookie_type state_hier_oo v =
-	  let make_full_st_name state_hier =
-	    let state_hier : Eliom_common.scope_hierarchy =
-	      match state_hier with
-		| None -> Eliom_common_base.Default_ref_hier
-		| Some s when String.lowercase s = "default" ->
-                  Eliom_common_base.Default_ref_hier
-		| Some s when String.lowercase s = "comet" ->
-                  Eliom_common_base.Default_comet_hier
-		| Some s -> Eliom_common_base.User_hier s
-	    in
-	    let scope =
-	      match cookie_type with
-		| `Session -> `Session state_hier
-		| `Client_process -> `Client_process state_hier
-	    in
-	    Eliom_common.make_full_state_name2
+          let make_full_st_name secure state_hier =
+            let state_hier : Eliom_common.scope_hierarchy =
+              match state_hier with
+              | None -> Eliom_common_base.Default_ref_hier
+              | Some s when String.lowercase_ascii s = "default" ->
+                Eliom_common_base.Default_ref_hier
+              | Some s when String.lowercase_ascii s = "comet" ->
+                Eliom_common_base.Default_comet_hier
+              | Some s -> Eliom_common_base.User_hier s
+            in
+            let scope =
+              match cookie_type with
+              | `Session -> `Session state_hier
+              | `Client_process -> `Client_process state_hier
+            in
+            Eliom_common.make_full_state_name2
               sitedata.Eliom_common.site_dir_string
-              false (*VVV and secure??? *)
+              secure
               ~scope
-	  in
+          in
+          (*VVV We set timeout for both secure and unsecure states.
+            Make possible to customize this? *)
           f
-            ?full_st_name:(Option.map make_full_st_name state_hier_oo)
+            ?full_st_name:(Option.map (make_full_st_name false) state_hier_oo)
+            ?cookie_level:(Some cookie_type)
+            ~recompute_expdates:false
+            true
+            true
+            sitedata
+            v;
+          f
+            ?full_st_name:(Option.map (make_full_st_name true) state_hier_oo)
             ?cookie_level:(Some cookie_type)
             ~recompute_expdates:false
             true
@@ -754,13 +864,17 @@ let parse_config hostpattern conf_info site_dir =
         let content =
           parse_eliom_options
             ((fun ct snoo v ->
-          print_endline "svt";
-              set_timeout Eliommod_timeouts.set_global_data_timeout_ ct snoo v;
-              set_timeout Eliommod_timeouts.set_global_service_timeout_ ct snoo v
+              set_timeout
+                (Eliommod_timeouts.set_global_ ~kind:`Data) ct snoo v;
+              set_timeout
+                (Eliommod_timeouts.set_global_ ~kind:`Service) ct snoo v
              ),
-             (set_timeout Eliommod_timeouts.set_global_data_timeout_),
-             (set_timeout Eliommod_timeouts.set_global_service_timeout_),
-             (set_timeout Eliommod_timeouts.set_global_persistent_timeout_),
+             (set_timeout
+                (Eliommod_timeouts.set_global_ ~kind:`Data)),
+             (set_timeout
+                (Eliommod_timeouts.set_global_ ~kind:`Service)),
+             (set_timeout
+                (Eliommod_timeouts.set_global_ ~kind:`Persistent)),
              (fun v -> sitedata.Eliom_common.max_service_sessions_per_group <- v, true),
              (fun v -> sitedata.Eliom_common.max_service_sessions_per_subnet <- v, true),
              (fun v -> sitedata.Eliom_common.max_volatile_data_sessions_per_group <- v, true),
@@ -784,13 +898,17 @@ let parse_config hostpattern conf_info site_dir =
                     Ipaddr.(V6 V6.localhost)
                   in
                   ignore (Ocsigen_cache.Dlist.set_maxsize dlist v)
-                with Not_found -> () (* should not occure *)
+                with Not_found -> () (* should not occur *)
              ),
              (fun v ->
                ignore (Ocsigen_cache.Dlist.set_maxsize
                          sitedata.Eliom_common.group_of_groups v)),
+             (fun v -> sitedata.Eliom_common.secure_cookies <- v),
              (fun v -> sitedata.Eliom_common.ipv4mask <- Some v, true),
-             (fun v -> sitedata.Eliom_common.ipv6mask <- Some v, true)
+             (fun v -> sitedata.Eliom_common.ipv6mask <- Some v, true),
+             (fun v -> sitedata.Eliom_common.application_script <- v),
+             (fun v -> sitedata.Eliom_common.cache_global_data <- v),
+             (fun v -> sitedata.Eliom_common.html_content_type <- Some v)
             )
             content
         in
@@ -804,7 +922,7 @@ let parse_config hostpattern conf_info site_dir =
         (match parse_module_attrs None atts with
           | Some file_or_name ->
             exception_during_eliommodule_loading := true;
-            load_eliom_module sitedata file_or_name content;
+            load_eliom_module sitedata file_or_name "eliom" content;
             exception_during_eliommodule_loading := false
           | _ -> ());
 
@@ -825,11 +943,11 @@ let parse_config hostpattern conf_info site_dir =
         else begin
           if not !eliommodulewarningdisplayed
           then Lwt_log.ign_warning ~section:Lwt_log.eliom
-              "Tag <eliom> used several times in the same site: will run Eliom only the first time. Prefer <eliommodule> to load a module, and <eliom/> without attibute only once at the position you want to generate your Eliom pages for this site.";
+              "Tag <eliom> used several times in the same site: will run Eliom only the first time. Prefer <eliommodule> to load a module, and <eliom/> without attribute only once at the position you want to generate your Eliom pages for this site.";
           eliommodulewarningdisplayed := true;
           gen_nothing ()
         end
-    | Element (t, _, _) ->
+    | Xml.Element (t, _, _) ->
         raise (Ocsigen_extensions.Bad_config_tag_for_extension t)
     | _ -> raise (Error_in_config_file "(Eliommod extension)")
 
