@@ -26,7 +26,7 @@ open Ocsigen_cookies
 include Eliom_cookies_base
 
 (*****************************************************************************)
-let make_new_session_id = make_cryptographic_safe_string
+let make_new_session_id = Ocsigen_lib.make_cryptographic_safe_string
 
 
 
@@ -58,8 +58,7 @@ let get_cookie_info
           match !expref with
           | Some t when t < now ->
               (* session expired by timeout *)
-              Eliom_common.SessionCookies.remove
-                sitedata.Eliom_common.session_services value;
+              Eliommod_sessiongroups.Serv.remove sessgrpnode;
               ((Eliom_common.Full_state_name_table.add
                   name
                   (Some value          (* value sent by the browser *),
@@ -115,9 +114,7 @@ let get_cookie_info
             match !expref with
               | Some t when t < now ->
                   (* session expired by timeout *)
-                  sitedata.Eliom_common.remove_session_data value;
-                  Eliom_common.SessionCookies.remove
-                    sitedata.Eliom_common.session_data value;
+                  Eliommod_sessiongroups.Data.remove sessgrpnode;
                   (Some value                 (* value sent by the browser *),
                    ref Eliom_common.SCData_session_expired
                                               (* ask the browser
@@ -155,8 +152,8 @@ let get_cookie_info
         lazy
           (catch
              (fun () ->
-               Ocsipersist.find
-                 (Lazy.force Eliom_common.persistent_cookies_table) value >>=
+               Lazy.force Eliom_common.persistent_cookies_table >>= fun table ->
+               Ocsipersist.find table value >>=
                fun (full_state_name, persexp, perstimeout, sessgrp) ->
 
                  Eliommod_sessiongroups.Pers.up value sessgrp >>= fun () ->
@@ -231,14 +228,11 @@ let get_cookie_info
   let persoktable = f_pers persistent_cookies in
 
   let sec, sservfailedlist =
-    match secure_cookies with
-      | None (* not https *) -> None, []
-      | Some (sc, dc, pc) ->
-          let (servoktable, servfailedlist) = f_serv sc in
-          let dataoktable = f_data dc in
-          let persoktable = f_pers pc in
-          (Some (ref servoktable, ref dataoktable, ref persoktable),
-           servfailedlist)
+    let (sc, dc, pc) = secure_cookies in
+    let (servoktable, servfailedlist) = f_serv sc in
+    let dataoktable = f_data dc in
+    let persoktable = f_pers pc in
+    ((ref servoktable, ref dataoktable, ref persoktable), servfailedlist)
   in
 
   (((ref servoktable, ref dataoktable, ref persoktable), sec),
@@ -366,15 +360,16 @@ let compute_session_cookies_to_send
   aux getpersvexp Eliom_common.persistentcookiename false !pers_cookies_info
     (aux getdatavexp Eliom_common.datacookiename false !data_cookie_info
        (aux getservvexp Eliom_common.servicecookiename false !service_cookie_info
-          (match secure_ci with
-             | None -> return endlist
-             | Some (service_cookie_info,
-                     data_cookie_info,
-                     pers_cookies_info) ->
-                 aux getpersvexp Eliom_common.persistentcookiename true !pers_cookies_info
-                   (aux getdatavexp Eliom_common.datacookiename true !data_cookie_info
-                      (aux getservvexp Eliom_common.servicecookiename true !service_cookie_info
-                         (return endlist))))))
+          (let (service_cookie_info, data_cookie_info, pers_cookies_info) =
+             secure_ci
+           in
+           aux getpersvexp Eliom_common.persistentcookiename
+             true !pers_cookies_info
+             (aux getdatavexp Eliom_common.datacookiename
+                true !data_cookie_info
+                (aux getservvexp Eliom_common.servicecookiename
+                   true !service_cookie_info
+                   (return endlist))))))
 
 
 let compute_cookies_to_send = compute_session_cookies_to_send
@@ -504,8 +499,6 @@ let compute_new_ri_cookies
   in
   f false ci ric
   >>= fun ric ->
-  match secure_ci with
-    | None -> Lwt.return ric
-    | Some ci -> f true ci ric
+  f true secure_ci ric
 (*VVV We always keep secure cookies, even if the protocol is not secure,
   because this function is for actions only. Is that right? *)
