@@ -47,7 +47,7 @@ let close_all_service_states2 full_st_name sitedata =
       Lwt_unix.yield ()
     )
     sitedata.Eliom_common.session_services
-    (return ())
+    return_unit
 
 (** Close all service states for one session name.
     If the optional parameter [?state_name] (session name) is not present,
@@ -74,7 +74,7 @@ let close_all_data_states2 full_st_name sitedata =
       Lwt_unix.yield ()
     )
     sitedata.Eliom_common.session_data
-    (return ())
+    return_unit
 
 (** Close all in memory data sessions for one session name.
     If the optional parameter [?state_name] (session name) is not present,
@@ -94,15 +94,15 @@ let close_all_data_states ~scope ~secure sitedata =
 
 
 let close_all_persistent_states2 full_st_name sitedata =
+  Lazy.force Eliommod_persess.persistent_cookies_table >>=
   Ocsipersist.iter_table
     (fun k ((scope, _, _) as full_st_name2, old_exp, old_t, sessiongrp) ->
       if full_st_name = full_st_name2 && old_t = Eliom_common.TGlobal
       then Eliommod_persess.close_persistent_state2
         ~scope sitedata sessiongrp k >>=
         Lwt_unix.yield
-      else return ()
+      else return_unit
     )
-    (Lazy.force Eliommod_persess.persistent_cookies_table)
 
 (** Close all persistent sessions for one session name.
     If the optional parameter [?state_name] (session name) is not present,
@@ -153,7 +153,7 @@ let update_serv_exp full_st_name sitedata old_glob_timeout new_glob_timeout =
         Lwt_unix.yield ()
       )
       sitedata.Eliom_common.session_services
-      (return ())
+      return_unit
 
 (* Update the expiration date for all in memory data sessions                *)
 let update_data_exp full_st_name sitedata old_glob_timeout new_glob_timeout =
@@ -183,7 +183,7 @@ let update_data_exp full_st_name sitedata old_glob_timeout new_glob_timeout =
         Lwt_unix.yield ()
       )
       sitedata.Eliom_common.session_data
-      (return ())
+      return_unit
 
 
 (* Update the expiration date for all sessions                               *)
@@ -195,6 +195,7 @@ let update_pers_exp full_st_name sitedata old_glob_timeout new_glob_timeout =
       close_all_persistent_states2 full_st_name sitedata
   | _ ->
     let now = Unix.time () in
+    Lazy.force Eliommod_persess.persistent_cookies_table >>= fun table ->
     Ocsipersist.iter_table
       (fun k ((scope, _, _) as full_st_name2, old_exp, old_t, sessgrp) ->
         if full_st_name = full_st_name2 && old_t =
@@ -210,12 +211,8 @@ let update_pers_exp full_st_name sitedata old_glob_timeout new_glob_timeout =
           | Some t when t <= now ->
               Eliommod_persess.close_persistent_state2
                 ~scope sitedata sessgrp k
-          | _ ->
-              Ocsipersist.add
-                (Lazy.force Eliommod_persess.persistent_cookies_table)
-                k
-                (full_st_name2, newexp,
-                 Eliom_common.TGlobal, sessgrp) >>= Lwt_unix.yield
-        else return ()
+          | _ -> Ocsipersist.add table k (full_st_name2, newexp,
+                   Eliom_common.TGlobal, sessgrp) >>= Lwt_unix.yield
+        else return_unit
       )
-      (Lazy.force Eliommod_persess.persistent_cookies_table)
+      table
