@@ -413,7 +413,7 @@ module Pers = struct
 (*VVV Verify this carefully! *)
 (*VVV VEOcsigen_request_infoFY concurrent access *)
 
-  let grouptable : (nbmax * string list) Ocsipersist.table Lazy.t =
+  let grouptable : (nbmax * string list) Ocsipersist.table Lwt.t Lazy.t =
     lazy (Ocsipersist.open_table "__eliom_session_group_table")
       (* It is lazy because if the module is linked statically,
          the creation of the table must happen after initialisation
@@ -422,15 +422,16 @@ module Pers = struct
 
   let find g =
     match g with
-    | None -> Lwt.return []
+    | None -> Lwt.return_nil
     | Some g ->
         Lwt.catch
           (fun () ->
-            Ocsipersist.find !!grouptable
+            !!grouptable >>= fun grouptable ->
+            Ocsipersist.find grouptable
               (Eliom_common.string_of_perssessgrp g) >>= fun (_, a) ->
             Lwt.return a)
           (function
-            | Not_found -> Lwt.return []
+            | Not_found -> Lwt.return_nil
             | e -> Lwt.fail e)
 
   let add ?set_max defaultmax sess_id sess_grp =
@@ -439,7 +440,8 @@ module Pers = struct
       let sg = Eliom_common.string_of_perssessgrp sg in
       Lwt.catch
         (fun () ->
-          Ocsipersist.find !!grouptable sg >>= fun (max2, cl) ->
+          !!grouptable >>= fun grouptable ->
+          Ocsipersist.find grouptable sg >>= fun (max2, cl) ->
           let max, newmax = match set_max with
             | None -> ((match max2 with
                 | Default -> defaultmax
@@ -449,7 +451,7 @@ module Pers = struct
             | Some (Some v) -> Some v, Val v
           in
           let cl, toclose = cut max cl in
-          Ocsipersist.replace_if_exists !!grouptable sg (newmax, (sess_id::cl))
+          Ocsipersist.replace_if_exists grouptable sg (newmax, (sess_id::cl))
           >>= fun () ->
           Lwt.return toclose)
         (function
@@ -459,10 +461,11 @@ module Pers = struct
               | Some None -> Nolimit
               | Some (Some v) -> Val v
             in
-            Ocsipersist.add !!grouptable sg (max, [sess_id]) >>= fun () ->
-            Lwt.return []
+            !!grouptable >>= fun grouptable ->
+            Ocsipersist.add grouptable sg (max, [sess_id]) >>= fun () ->
+            Lwt.return_nil
           | e -> Lwt.fail e)
-    | None -> Lwt.return []
+    | None -> Lwt.return_nil
 
 
   let rec remove_group ~cookie_level sitedata sess_grp =
@@ -473,7 +476,6 @@ module Pers = struct
     Lwt.catch
       (fun () ->
         (* First we close all sessions in the group *)
-
         find sess_grp >>= fun cl ->
         Lwt_list.iter_p
           (close_persistent_session2
@@ -502,19 +504,20 @@ module Pers = struct
                                       belonging to the group grp *)
             (* group_name is the cookie value *)
             remove sitedata group_name grp >>= fun () ->
-            Ocsipersist.remove
-              (!!Eliom_common.persistent_cookies_table) group_name
-          | _ -> Lwt.return ())
+            !!Eliom_common.persistent_cookies_table >>= fun table ->
+            Ocsipersist.remove table group_name
+          | _ -> Lwt.return_unit)
         >>= fun () ->
 
         (* Then, we remove group from group table: *)
         match sess_grp with
           | Some sg ->
             let sg = Eliom_common.string_of_perssessgrp sg in
-            Ocsipersist.remove !!grouptable sg
-          | None -> Lwt.return ()
+            !!grouptable >>= fun grouptable ->
+            Ocsipersist.remove grouptable sg
+          | None -> Lwt.return_unit
       )
-      (function Not_found -> Lwt.return () | e -> Lwt.fail e)
+      (function Not_found -> Lwt.return_unit | e -> Lwt.fail e)
 
   (* close a persistent session (tab or browser)
      and the associated group (if browser session) by cookie value *)
@@ -526,7 +529,8 @@ module Pers = struct
         match cookie_level with
           | `Client_process -> begin
             (* We remove cookie info from the table *)
-            Ocsipersist.remove (!!Eliom_common.persistent_cookies_table) cookie
+            !!Eliom_common.persistent_cookies_table >>= fun table ->
+            Ocsipersist.remove table cookie
             >>= fun () ->
 
             (* We remove the session from its group: *)
@@ -545,7 +549,7 @@ module Pers = struct
                  (Some cookie))
       )
       (function
-        | Not_found -> Lwt.return ()
+        | Not_found -> Lwt.return_unit
         | e -> Lwt.fail e)
 
 
@@ -555,43 +559,37 @@ module Pers = struct
       let sg = Eliom_common.string_of_perssessgrp sg0 in
       Lwt.catch
         (fun () ->
-          Ocsipersist.find !!grouptable sg >>= fun (max, cl) ->
+          !!grouptable >>= fun grouptable ->
+          Ocsipersist.find grouptable sg >>= fun (max, cl) ->
           let newcl = List.remove_first_if_any sess_id cl in
-          (match newcl with
-            | [] ->
-              (* The last session has been removed from the group.
-                 If it was a browser session, we close the group,
-                 by removing group data.
-                 For tab sessions, no (but see in volatile table clean_session
-                 function for a better behaviour).
-              *)
-              (match Eliom_common.getperssessgrp sg0 with
-                | (_, `Session, _) ->
-                  remove_group ~cookie_level:`Session sitedata sess_grp
-                | _ -> Lwt.return ()
-              ) >>= fun () ->
-              Ocsipersist.remove !!grouptable sg
-            | _ -> Ocsipersist.replace_if_exists !!grouptable sg (max, newcl)
-          )
+          (* Before 2018-10-18, we were closing the session group
+             when newcl was empty (no more session in the group).
+             But persistent session groups are usually used to store persistent
+             information about users. It makes no sense cleaning this
+             information when user closes all their sessions.
+             I remove this. -- Vincent
+          *)
+          Ocsipersist.replace_if_exists grouptable sg (max, newcl)
         )
         (function
-          | Not_found -> Lwt.return ()
+          | Not_found -> Lwt.return_unit
           | e -> Lwt.fail e)
-    | None -> Lwt.return ()
+    | None -> Lwt.return_unit
 
   let up sess_id grp =
     match grp with
-      | None -> Lwt.return ()
+      | None -> Lwt.return_unit
       | Some sg ->
         let sg = Eliom_common.string_of_perssessgrp sg in
         Lwt.catch
           (fun () ->
-            Ocsipersist.find !!grouptable sg >>= fun (max, cl) ->
+            !!grouptable >>= fun grouptable ->
+            Ocsipersist.find grouptable sg >>= fun (max, cl) ->
             let newcl = List.remove_first_if_any sess_id cl in
-            Ocsipersist.replace_if_exists !!grouptable sg (max, sess_id::newcl)
+            Ocsipersist.replace_if_exists grouptable sg (max, sess_id::newcl)
           )
           (function
-            | Not_found -> Lwt.return ()
+            | Not_found -> Lwt.return_unit
             | e -> Lwt.fail e)
 
   let move sitedata ?set_max max sess_id grp1 grp2 =
@@ -599,8 +597,8 @@ module Pers = struct
       remove sitedata sess_id grp1 >>= fun () ->
       add ?set_max max sess_id grp2
     end
-    else Lwt.return []
+    else Lwt.return_nil
 
-  let nb_of_groups () = Ocsipersist.length !!grouptable
+  let nb_of_groups () = !!grouptable >>= Ocsipersist.length
 
 end
