@@ -13,20 +13,24 @@ let usage () =
   Printf.eprintf "SPECIFIC OPTIONS:\n";
   Printf.eprintf "  -package <name>\tRefer to package when compiling\n";
   Printf.eprintf "  -no-autoload\t\tDo not load commonly used syntax extensions (deriving, lwt, js_of_ocaml, tyxml)\n";
-  Printf.eprintf "  -type-conv\t\tUse type_conv syntax extensions instead of deriving one\n";
-  Printf.eprintf "\t\t\tIt has no effect if used in conjunction with -no-autoload\n";
   if !kind <> `Client then
     Printf.eprintf "  -infer\t\tOnly infer the type of values sent by the server\n";
   Printf.eprintf "  -dir <dir>\t\tThe directory for generated files (default %S)\n"
     (if !kind = `Client then default_client_dir else default_server_dir);
   Printf.eprintf "  -type-dir <dir>\tThe directory to read server type files from (default %S)\n"
-     default_type_dir;
+    default_type_dir;
   Printf.eprintf "  -server-types-ext <ext> The extension for server type files (default %S)\n"
     default_server_types_ext;
   if !kind = `Client then
     Printf.eprintf "  -jsopt <opt>\t\tAppend option <opt> to js_of_ocaml invocation\n";
   Printf.eprintf "  -ppopt <p>\t\tAppend option <opt> to preprocessor invocation\n";
   Printf.eprintf "  -predicates <p>\tAdd predicate <p> when resolving package properties\n";
+  Printf.eprintf "  -ppx";
+  Printf.eprintf "\t\t\tUse the PPX Eliom syntax extension (default: Camlp4)\n";
+  Printf.eprintf "\t\t\tThe above description only applies to the first\n";
+  Printf.eprintf "\t\t\tappearance of -ppx in the list of flags. Subsequent\n";
+  Printf.eprintf "\t\t\tappearances require an argument and specify a\n";
+  Printf.eprintf "\t\t\tPPX preprocessor to use (see STANDARD OPTIONS).\n";
   if !kind = `Client then
     Printf.eprintf "  -dont-force-linkall\t\tDo not add linkall option by default\n";
   create_filter !compiler ["-help"] (help_filter 2 "STANDARD OPTIONS:");
@@ -55,15 +59,15 @@ let rec check_or_create_dir name =
     try ignore(Unix.stat name) with Unix.Unix_error _ ->
       check_or_create_dir (Filename.dirname name);
       try Unix.mkdir name 0o777 with
-        (* this append sometime with // compilation *)
-        | Unix.Unix_error (Unix.EEXIST,_,_) -> ()
-        | Unix.Unix_error (_,_,_) ->
-          Printf.eprintf "Unexpected error while creating directory %S" name
+      (* this append sometime with // compilation *)
+      | Unix.Unix_error (Unix.EEXIST,_,_) -> ()
+      | Unix.Unix_error (_,_,_) ->
+        Printf.eprintf "Unexpected error while creating directory %S" name
 
 let prefix_output_dir name =
   match !build_dir with
-    | "" -> name
-    | d -> d ^ "/" ^ name
+  | "" -> name
+  | d -> d ^ "/" ^ name
 
 let chop_extension_if_any name =
   try Filename.chop_extension name with Invalid_argument _ -> name
@@ -72,66 +76,70 @@ let output_prefix ?(ty = false) name =
   let name =
     match !output_name with
     | None ->
-	if !mode = `Infer || ty
-	then prefix_type_dir name
-	else prefix_output_dir name
+      if !mode = `Infer || ty
+      then prefix_type_dir name
+      else prefix_output_dir name
     | Some n ->
-	if !mode = `Compile || !mode = `Infer
-	then (output_name := None; n)
-	else prefix_output_dir name in
+      if !mode = `Compile || !mode = `Infer
+      then (output_name := None; n)
+      else prefix_output_dir name in
   check_or_create_dir (Filename.dirname name);
   chop_extension_if_any name
 
 let set_mode m =
- if !mode = `Link then
-   ( if
-       (m = `Shared && !kind <> `ServerOpt) ||
-       (m = `Infer && !kind = `Client) ||
-       (m = `Interface && !kind = `Client)
-     then
-       usage ()
-     else
-       mode := m )
- else
-   let args =
-     let basic_args = ["-pack"; "-a"; "-c"; "output-obj"] in
-     let infer_args = if !kind <> `Client then ["-i"; "-infer"] else [] in
-     let shared_args = if !kind <> `ServerOpt then ["-shared"] else [] in
-     basic_args @ infer_args @ shared_args
-   in
-   Printf.eprintf
-     "Please specify at most one of %s\n%!"
-     (String.concat ", " args);
-  exit 1
+  if !mode = `Link then
+    ( if
+      (m = `Shared && !kind <> `ServerOpt) ||
+      (m = `Infer && !kind = `Client) ||
+      (m = `Interface && !kind = `Client)
+      then
+        usage ()
+      else
+        mode := m )
+  else
+    let args =
+      let basic_args = ["-pack"; "-a"; "-c"; "output-obj"] in
+      let infer_args = if !kind <> `Client then ["-i"; "-infer"] else [] in
+      let shared_args = if !kind <> `ServerOpt then ["-shared"] else [] in
+      basic_args @ infer_args @ shared_args
+    in
+    Printf.eprintf
+      "Please specify at most one of %s\n%!"
+      (String.concat ", " args);
+    exit 1
 
 let get_product_name () = match !output_name with
   | None ->
-      Printf.eprintf
-	"Please specify the name of the output file, using option -o\n%!";
-      exit 1
+    Printf.eprintf
+      "Please specify the name of the output file, using option -o\n%!";
+    exit 1
   | Some name ->
-      check_or_create_dir (Filename.dirname name);
-      name
+    check_or_create_dir (Filename.dirname name);
+    name
 
 let build_library () =
-  create_process !compiler ( ["-a" ; "-o"  ; get_product_name () ]
-			     @ get_common_include ()
-			     @ !args )
+  create_process !compiler (
+    ["-a" ; "-o"  ; get_product_name () ]
+    @ get_common_include ()
+    @ !args )
 
 let build_pack () =
-  create_process !compiler ( [ "-pack" ; "-o"  ; get_product_name () ]
-			     @ get_common_include ()
-			     @ !args )
+  create_process !compiler (
+    [ "-pack" ; "-o"  ; get_product_name () ]
+    @ get_common_include ()
+    @ !args )
 
 let build_obj () =
-  create_process !compiler ( ["-output-obj" ; "-o"  ; get_product_name () ]
-			     @ get_common_include ()
-			     @ !args )
+  create_process !compiler (
+    ["-output-obj" ; "-o"  ; get_product_name () ]
+    @ get_common_include ()
+    @ !args )
 
 let build_shared () =
-  create_process !compiler ( ["-shared" ; "-o"  ; get_product_name () ]
-			     @ get_common_include ()
-			     @ !args )
+  create_process !compiler (
+    ["-shared" ; "-o"  ; get_product_name () ]
+    @ get_common_include ()
+    @ !args )
 
 let get_thread_opt () = match !kind with
   | `Client -> []
@@ -148,18 +156,24 @@ let compile_ocaml ~impl_intf file =
       | `Intf -> ".cmi"
     in
     output_prefix file ^ ext in
-  create_process !compiler ( ["-c" ;
-                              "-o" ; obj ;
-                              "-pp"; get_pp [] !ppopt]
-                             @ !args
-			     @ get_thread_opt ()
-			     @ get_common_include ()
-			     @ [impl_intf_opt impl_intf; file] )
+  create_process !compiler (
+    ["-c" ;
+     "-o" ; obj ]
+    @ preprocess_opt ~ocaml:true !ppopt
+    @ !args
+    @ get_thread_opt ()
+    @ get_common_include ()
+    @ get_common_ppx ()
+    @ [impl_intf_opt impl_intf; file] )
 
 let output_ocaml_interface file =
-    create_process !compiler ( ["-i"; "-pp"; get_pp [] !ppopt] @ !args
-                               @ get_common_include ()
-                               @ [file] )
+  create_process !compiler (
+    [ "-i" ]
+    @ preprocess_opt ~ocaml:true !ppopt
+    @ !args
+    @ get_common_include ()
+    @ get_common_ppx ()
+    @ [file] )
 
 let process_ocaml ~impl_intf file =
   if do_compile () then
@@ -174,20 +188,49 @@ let compile_obj file =
 
 (* Process eliom and eliomi files *)
 
-let get_ppopts ?kind ~impl_intf file =
-  let pkg =
-    match get_kind kind with
-      | `Client -> ["eliom.syntax.client"]
-      | `Server | `ServerOpt -> ["eliom.syntax.server"]
-  in
-  pkg, type_opt impl_intf file @ !ppopt @ [impl_intf_opt impl_intf]
+let run_command s =
+  let v = Sys.command s in
+  if v != 0 then
+    failwith (Printf.sprintf "Warning: command [%s] returned %d" s v)
+
+(* WARNING: if you change this, also change inferred_type_prefix in
+   ppx/ppx_eliom_utils.ml and ocamlbuild/ocamlbuild_eliom.ml *)
+let inferred_type_prefix = "eliom_inferred_type_"
+
+(* FIXME!
+
+   run_sed is a temporary hack to parse weakly monomorphic types of
+   the following forms:
+
+   '_a
+   _[< ... ]
+   _[> ... ]
+
+   It also removes type indices foo/2 (in case of ambiguity in
+   module interfaces).
+
+   These appear in type_mli files, but they are not accepted by
+   the OCaml parser.  *)
+let run_sed file =
+  run_command ("sed -i -e 's$/[1-9][0-9]*$$g' " ^ file);
+  run_command ("sed -i -e 's/_\\[\\([<>]\\)/[\\1/g' " ^ file);
+  run_command
+    (Printf.sprintf
+       "sed -i -e \"s/'\\(_[a-z0-9_]*\\)/'%s\\1/g\" %s"
+       inferred_type_prefix file);
+  (* For some sed implementations, [-e] is interpreted as an argument
+     to [-i], resulting in copy of the original file named [file ^
+     "-e"]. If we find such a file, remove it. Slightly less messy
+     than doing OS detection beforehand. *)
+  let file_e = file ^ "-e" in
+  if Sys.file_exists file_e then Sys.remove file_e
 
 let compile_server_type_eliom file =
   let obj = output_prefix ~ty:true file ^ !server_types_file_ext
-  and ppopts = !ppopt @ ["-impl"] in
+  and ppopts = !ppopt @ (if !pp_mode = `Ppx then [] else ["-impl"]) in
   if !do_dump then begin
     let camlp4, ppopt =
-      get_pp_dump ["eliom.syntax.type"] ("-printer" :: "o" :: ppopts @ [file]) in
+      get_pp_dump [] ("-printer" :: "o" :: ppopts @ [file]) in
     create_process camlp4 ppopt;
     exit 0
   end;
@@ -196,12 +239,15 @@ let compile_server_type_eliom file =
     Unix.close out;
     Sys.remove obj
   in
-  create_process ~out ~on_error
-    !compiler ( [ "-i" ; "-pp"; get_pp ["eliom.syntax.type"] ppopts]
-		@ !args
-    		@ get_common_include ()
-		@ ["-impl"; file] );
-  Unix.close out
+  create_process ~out ~on_error !compiler (
+    [ "-i" ]
+    @ preprocess_opt ~kind:`Types ppopts
+    @ !args
+    @ get_common_include ()
+    @ get_common_ppx ()
+    @ ["-impl"; file] );
+  Unix.close out;
+  if !pp_mode = `Ppx then run_sed obj
 
 let output_eliom_interface ~impl_intf file =
   if !do_dump then begin
@@ -209,28 +255,48 @@ let output_eliom_interface ~impl_intf file =
     exit 1
   end;
   let indent ch =
+    let spaces =
+      match !pp_mode with
+      | `Ppx -> ""
+      | `Camlp4 -> "  "
+    in
     try
       while true do
         let line = input_line ch in
-        Printf.printf "  %s\n" line
+        Printf.printf "%s%s\n" spaces line
       done
     with End_of_file -> ()
   in
   let args kind =
-    let pkg, ppopts = get_ppopts ~kind ~impl_intf file in
-    let pp = get_pp pkg ppopts in
-    [ "-i" ; "-pp" ; pp; "-intf-suffix"; ".eliomi" ]
+    let ppopts = get_ppopts ~impl_intf file in
+    [ "-i" ]
+    @ preprocess_opt ~kind ppopts
+    @ [ "-intf-suffix"; ".eliomi" ]
     @ !args
     @ get_common_include ~kind ()
+    @ get_common_ppx ~kind ()
     @ [ impl_intf_opt impl_intf; file ]
+  and open_block str =
+    match !pp_mode with
+    | `Ppx ->
+      Printf.printf "[%%%%%s.start]\n" str
+    | `Camlp4 ->
+      Printf.printf "{%s{\n" str
+  and close_block () =
+    match !pp_mode with
+    | `Ppx ->
+      ()
+    | `Camlp4 ->
+      print_endline "}}"
   in
-  Printf.printf "(* WARNING `eliomc -i' generated this pretty ad-hoc - use with care! *)\n";
-  Printf.printf "{server{\n";
+  Printf.printf
+    "(* WARNING generated in an ad-hoc fashion. Use with care! *)\n";
+  open_block "server";
   create_filter !compiler (args `Server) indent;
-  Printf.printf "}}\n";
-  Printf.printf "{client{\n";
+  close_block ();
+  open_block "client";
   create_filter !compiler (args `Client) indent;
-  Printf.printf "}}\n"
+  close_block ()
 
 let compile_eliom ~impl_intf file =
   let obj =
@@ -240,57 +306,60 @@ let compile_eliom ~impl_intf file =
     in
     output_prefix file ^ ext
   in
-  let pkg, ppopts = get_ppopts ~impl_intf file in
-  if !do_dump then begin
-    let camlp4, ppopt = get_pp_dump pkg ("-printer" :: "o" :: ppopts @ [file]) in
-    create_process camlp4 ppopt;
-    exit 0
-  end;
-  create_process !compiler ( [ "-c" ;
-                               "-o"  ; obj ;
-                               "-pp" ; get_pp pkg ppopts;
-                               "-intf-suffix"; ".eliomi" ]
-                             @ get_thread_opt ()
-			     @ !args
-			     @ get_common_include ()
-			     @ [impl_intf_opt impl_intf; file] );
+  let ppopts = get_ppopts ~impl_intf file in
+  (* if !do_dump then begin *)
+  (*   let camlp4, ppopt = get_pp_dump pkg ("-printer" :: "o" :: ppopts @ [file]) in *)
+  (*   create_process camlp4 ppopt; *)
+  (*   exit 0 *)
+  (* end; *)
+  create_process !compiler (
+    [ "-c" ; "-o"  ; obj ]
+    @ preprocess_opt ppopts
+    @ [ "-intf-suffix"; ".eliomi" ]
+    @ get_thread_opt ()
+    @ !args
+    @ get_common_include ()
+    @ get_common_ppx ()
+    @ [impl_intf_opt impl_intf; file] );
   args := !args @ [obj]
 
 let process_eliom ~impl_intf file =
   match !mode with
   | `Infer when impl_intf = `Impl ->
-      compile_server_type_eliom file
+    compile_server_type_eliom file
   | `Interface ->
-      output_eliom_interface ~impl_intf file
+    output_eliom_interface ~impl_intf file
   | _ ->
-      compile_eliom ~impl_intf file
+    compile_eliom ~impl_intf file
 
 let build_server ?(name = "a.out") () =
   fail "Linking eliom server is not yet supported"
-  (* TODO ? Build a staticaly linked ocsigenserver. *)
+(* TODO ? Build a staticaly linked ocsigenserver. *)
 
 let build_client () =
   let name = chop_extension_if_any (get_product_name ()) in
   let exe = prefix_output_dir (Filename.basename name) in
   check_or_create_dir (Filename.dirname exe);
   let js = name ^ ".js" in
-  create_process !compiler ( ["-o"  ;  exe ]
-           @ link_all ()
-			     @ get_common_include ()
-			     @ get_client_lib ()
-			     @ !args );
-  create_process !js_of_ocaml ( ["-o" ; js ]
-				@ get_client_js ()
-				@ !jsopt
-				@ [exe] )
+  create_process !compiler (
+    ["-o"  ;  exe ]
+    @ link_all ()
+    @ get_common_include ()
+    @ get_client_lib ()
+    @ !args );
+  create_process !js_of_ocaml (
+    ["-o" ; js ]
+    @ get_client_js ()
+    @ !jsopt
+    @ [exe] )
 
 let process_option () =
-  let i = ref 1 in
+  let i = ref 1
+  and ppx = ref false in
   while !i < Array.length Sys.argv do
     match Sys.argv.(!i) with
     | "-help" | "--help" -> usage ()
     | "-no-autoload" -> autoload_predef := false; incr i
-    | "-type-conv" -> type_conv := true; incr i
     | "-dont-force-linkall"->
       if !kind <> `Client then usage ();
       force_link_all := false; incr i
@@ -310,6 +379,10 @@ let process_option () =
     | "-dump" ->
       do_dump := not !do_dump;
       i := !i+1
+    | "-ppx" when not !ppx ->
+      ppx := true;
+      pp_mode := `Ppx;
+      i := !i+1
     | "-dir" ->
       if !i+1 >= Array.length Sys.argv then usage ();
       build_dir := Sys.argv.(!i+1);
@@ -386,26 +459,26 @@ let main () =
   let k =
     match cmd with
     | "eliomopt" ->
-	    compiler := ocamlopt;
-	    build_dir := default_server_dir;
-	    `ServerOpt
+      compiler := ocamlopt;
+      build_dir := default_server_dir;
+      `ServerOpt
     | "eliomcp" ->
-	    compiler := ocamlcp;
-	    build_dir := default_server_dir;
-	    `Server
+      compiler := ocamlcp;
+      build_dir := default_server_dir;
+      `Server
     | "js_of_eliom" ->
-	    compiler := ocamlc;
-	    build_dir := default_client_dir;
-	    `Client
+      compiler := ocamlc;
+      build_dir := default_client_dir;
+      `Client
     | "eliomc" ->
-	    compiler := ocamlc;
-	    build_dir := default_server_dir;
-	    `Server
+      compiler := ocamlc;
+      build_dir := default_server_dir;
+      `Server
     | s ->
       Format.eprintf "exec name not recognize %S: fallback to ocamlc@." s;
-  	  compiler := ocamlc;
-	    build_dir := default_server_dir;
-	    `Server
+      compiler := ocamlc;
+      build_dir := default_server_dir;
+      `Server
   in
   kind := k;
   process_option ()
